;; About dialog.

;; Copyright 2002, 2003, 2004, 2005, 2006, 2007, 2008 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(define-module (chart about)
  #:use-module (srfi srfi-13)
  #:use-module (gtk-2.0 gdk-pixbuf)
  #:use-module (gtk gtk)
  #:use-module (gtk gdk)
  #:use-module (chart config)
  #:use-module (chart extra-glib)
  #:use-module (chart extra-gtk)
  #:use-module (chart gnome)
  #:use-module (chart gzip)
  #:use-module (chart https)
  #:use-module (chart kex)   ;; for lha-version
  #:use-module (chart misc)
  #:use-module (chart unzip)
  #:use-module (chart zlib))


(define copyright-string
  (string-append
   "Copyright " ;; never translated
   ;; TRANSLATORS: use the copyright symbol char here when it's displayable
   ;; TRANSLATORS: (which probably means always for utf8 with Gtk 2)
   (_ "(C)")
   " 2002, 2003, 2004, 2005, 2006, 2007 "
   (_ "Kevin Ryde")))


;;-----------------------------------------------------------------------------
;; GtkLinkButton -- made public in Gtk 2.10, not wrapped in guile-gtk 2.0

(if (not (defined? 'gtk-link-button-new))
    (begin
      (gtk-class-new 'GtkButton "ChartGtkLinkButton")

      (define (gtk-link-button-new uri)
	(let ((button (gtk-widget-new 'ChartGtkLinkButton #:label uri)))
	  (c-gtk-signal-connect-once button "realize"
	    (lambda ()
	      (gdk-window-set-cursor (gtk-widget-window button)
				     (gdk-cursor-new 'hand2))))
	  button))

      (define (gtk-link-button-set-uri button uri)
	(set-object-property! button 'uri uri))
      (define (gtk-link-button-get-uri button)
	(object-property button 'uri))))


;;-----------------------------------------------------------------------------
;; GtkAboutDialog
;;

;; not wrapped in Guile-Gtk 2.0
(if (not (defined? 'gtk-about-dialog-new))
    (let-toplevel ((url-proc #f))

      (define (c-gdk-window-get-depth win)
	(receive-list (x y width height depth)
	    (gdk-window-get-geometry win)
	  depth))

      (define (set-label about key str)
	(define label (object-property about key))
	(or label
	    (error "missing" about key))
	(gtk-label-set-text label str)
	(gtk-widget-show label))

      (gtk-class-new 'GtkDialog "ChartGtkAboutDialog")

      (define-toplevel (gtk-about-dialog-set-url-hook proc)
	(set! url-proc proc))

      (let ((old gtk-object-set))
	(set! gtk-object-set
	      (lambda (about key val)
		(case key
		  ((#:name)
		   (gtk-window-set-title about val)
		   (set-label about 'name-label val))

		  ((#:version)
		   (set-label about 'version-label
			      (format #f (_ "version ~a") val)))

		  ((#:authors)
		   (set-label about 'authors-label
			      (string-append
			       (ngettext "Author" "Authors" (length val))
			       ": " (string-join val (_ ", ")))))

		  ((#:copyright)
		   (set-label about 'copyright-label (string-append val "\n")))

		  ((#:comments)
		   (set-label about 'comments-label val))

		  ((#:logo)
		   ;; VAL can be #f for no logo
		   (let ((logo-pixmap (object-property about 'logo-pixmap)))
		     (gtk-widget-realize logo-pixmap)
		     (if val
			 (let* ((width     (gdk-pixbuf-get-width val))
				(height    (gdk-pixbuf-get-height val))
				(gdkpixmap (gdk-pixmap-new (gtk-widget-window logo-pixmap)
							   width height -1)))
			   (gdk-pixbuf-render-to-drawable
			    val gdkpixmap
			    (gtk-style-fg-gc (gtk-widget-get-style logo-pixmap) 'normal)
			    0 0 0 0 width height 'normal 0 0)
			   (gtk-pixmap-set logo-pixmap gdkpixmap)
			   (gtk-widget-show logo-pixmap)))))

		  ((#:website)
		   (let ((button (object-property about 'website-button)))
		     (gtk-link-button-set-uri button val)
		     (gtk-button-set-label button (string-append (_ "Web: ")
								 val "\n"))
		     (gtk-widget-show button)))

		  ((#:license)
		   (gtk-dialog-add-button about (_ "_License") 1)
		   (gtk-signal-connect about "response"
		     (lambda (id)
		       (if (= id 1)
			   (let* ((dialog (gtk-dialog-new-with-buttons
					   (c-make-window-title (_ "License"))
					   #f '()
					   "gtk-close" gtk-response-cancel))
				  (textbuf  (gtk-text-buffer-new #f))
				  (textview (gtk-text-view-new-with-buffer
					     textbuf))
				  (scrolled (gtk-scrolled-window-new)))
			     (gtk-text-buffer-set-text textbuf val)

			     (gtk-scrolled-window-set-policy
			      scrolled 'never 'always)
			     (c-gtk-widget-set-usize-units
			      textview '(em . 45) '(height . 20))
			     (gtk-text-view-set-editable textview #f)
			     (gtk-text-view-set-wrap-mode textview 'char)
			     (gtk-container-add scrolled textview)

			     (gtk-widget-show-all scrolled)
			     (gtk-box-pack-start (gtk-dialog-vbox dialog)
						 scrolled #t #t 0)

			     (gtk-signal-connect dialog "response"
			       (lambda (id)
				 (if (= id gtk-response-cancel)
				     (gtk-widget-destroy dialog))))
			     (gtk-widget-show dialog))))))

		  (else
		   (old about key val))))))

      (define-toplevel (gtk-about-dialog-new)
	(define about (gtk-widget-new 'ChartGtkAboutDialog))
	(gtk-dialog-add-button about "gtk-ok" gtk-response-ok)
	(gtk-dialog-set-default-response about gtk-response-ok)

	;; nasty hack to get the window depth now, to make logo setup easier
	(gtk-widget-realize about)
	(let* ((dummy-pixmap  (gdk-pixmap-new #f 1 1
					      (c-gdk-window-get-depth
					       (gtk-widget-window about))))
	       (logo-pixmap   (gtk-pixmap-new dummy-pixmap))
	       (name-label    (gtk-label-new #f))
	       (version-label (gtk-label-new #f))
	       (hbox          (gtk-hbox-new #f 0))
	       (em            (c-gtk-widget-units->width name-label
							 '(em . 1))))
	  ;; keep this around to avoid warning under guile-gtk 0.41
	  (set-object-property! about 'dummy-pixmap dummy-pixmap)

	  (set-object-property! about 'logo-pixmap logo-pixmap)
	  (set-object-property! about 'name-label name-label)
	  (set-object-property! about 'version-label version-label)
	  (gtk-misc-set-alignment name-label 0 0.5)
	  (gtk-misc-set-alignment version-label 0 0.5)
	  (gtk-box-pack-start hbox logo-pixmap #f #f em)
	  (gtk-box-pack-start hbox name-label #f #f em)
	  (gtk-box-pack-start hbox version-label #f #f em)
	  (gtk-box-pack-start (gtk-dialog-vbox about) hbox #t #t
			      (c-gtk-widget-units->height name-label '(height . 1)))
	  (gtk-widget-show hbox))

	(for-each (lambda (key)
		    (let ((label (gtk-label-new #f)))
		      (set-object-property! about key label)
		      (gtk-label-set-justify label 'left)
		      (gtk-object-set label #:wrap #t)
		      (gtk-misc-set-alignment label 0 0.5)
		      (gtk-misc-set-padding
		       label (c-gtk-widget-units->width label '(em . 1)) 0)
		      (gtk-box-pack-start (gtk-dialog-vbox about) label #t #t)))
		  '(authors-label copyright-label comments-label))

	(let* ((button (gtk-link-button-new ""))
	       (label  (gtk-bin-get-child button)))
	  (set-object-property! about 'website-button button)
	  (gtk-button-set-relief button 'none)
	  (gtk-label-set-justify label 'left)
	  (gtk-misc-set-alignment label 0 0.5)
	  (gtk-misc-set-padding label
				(c-gtk-widget-units->width button '(em . 1)) 0)
	  (gtk-signal-connect button "clicked"
	    (lambda ()
	      (if (gtk-widget-mapped button)
		  (catch 'g-error
		    (lambda ()
		      (url-proc (gtk-link-button-get-uri button)))
		    (lambda args
		      (format (current-error-port)
			      "chart: cannot open URL: ~s" args)
		      (gdk-beep))))))
	  (gtk-box-pack-start (gtk-dialog-vbox about) button))

	about)))


;;-----------------------------------------------------------------------------
;; about, created on demand and destroyed when closed

(define (available-things-string)
  (let ((avail     '())
	(not-avail '())
	(result    ""))

    (define (try name version)
      (if version
	  (set! avail (append avail (list (format #f "~a (~a)" name version))))
	  (set! not-avail (append not-avail (list name)))))
    (try "Zlib"    (zlib-version))
    (try "gzip"    (gzip-version))
    (try "gnutls"  (gnutls-version))
    (try "lha"     (lha-version))
    (try "openssl" (openssl-version))
    (try "unzip"   (unzip-version))

    (if (not (null? avail))
	(set! result (string-append (_ "Available: ")
				    (string-join avail ", ")
				    "\n")))
    (if (not (null? not-avail))
	(set! result (string-append result
				    (_ "Not available: ")
				    (string-join not-avail ", ")
				    "\n")))

    result))

(gtk-about-dialog-set-url-hook (lambda (about url)
				 (gnome-url-show url)))

(define-public (about-dialog-popup)
  (define about (gtk-about-dialog-new))

  (gtk-signal-connect about "response"
    (lambda (id)
      (if (= id gtk-response-cancel)
	  (gtk-widget-destroy about)))) ;; close button or delete-event

  (gtk-object-set about #:name (_ "Chart"))
  (gtk-object-set about #:version chart-version)
  (gtk-object-set about #:copyright copyright-string)
  (gtk-object-set about #:license
		  (call-with-input-file
		      (g-build-filename chart-config-chartdatadir "COPYING")
		    read-string-to-eof))

  (gtk-object-set about #:website
		  "http://www.geocities.com/user42_kevin/chart/index.html")

  (gtk-object-set about #:comments
		  (string-append
		   (_ "\
Chart is Free Software, distributed under the terms of the \
GNU General Public License as published by the Free Software \
Foundation, either version 3 of the License, or (at your option) \
any later version.  \
Click on the License button below for the full text.")
		   "\n\n"
		   (_ "\
Chart is distributed in the hope that it will be useful, but \
WITHOUT ANY WARRANTY; without even the implied warranty of \
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. \
See the license for more.")
		   "\n\n"
		   (format #f (_ "\
You are running under: Guile ~a, Gtk ~a.~a.~a\n")
			   (version)
			   gtk-major-version
			   gtk-minor-version
			   gtk-micro-version)
		   (available-things-string)
		   (if (not iconv-available?)
		       (_ "Iconv not available, so limited to mostly ASCII.")
		       "")))

  ;; (gtk-object-set about #:authors (list (_ "Kevin Ryde")))

  (gtk-object-set about #:logo
		  (gdk-pixbuf-new-from-file
		   (g-build-filename chart-config-pixmapsdir "chart.xpm")))

  (gtk-widget-show-all about))
