;; Accumulation/Distribution index.

;; Copyright 2005, 2006, 2007, 2009 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(define-module (chart accdist)
  #:use-module (chart indicator)
  #:use-module (chart misc))


;; PREV is unused, so `accumulation-series' ends up one point shorter than
;; it could be at the start of data.

(define (accdist-func array prev this)
  (let* ((close      (array-ref array this 3))
	 (high       (or (array-ref array this 1) close))
	 (low        (or (array-ref array this 2) close))
	 (volume     (or (array-ref array this 4) 0)))

    (let ((den (- high low)))
      (if (zero? den)
	  0
	  (* volume (1- (* 2.0 (/ (- close low)
				  den))))))))

(define-public (accdist-series series)
  (accumulation-series series (_ "Acc/Dist") accdist-datatype accdist-func))

(define-public accdist-datatype
  (datatype-new!
   #:key    'accdist
   #:name   (_ "Accumulation/Distribution Index")
   #:units  'accdist
   #:manual (_ "CONTEXT(manual-node): Accumulation/Distribution")
   #:proc   accdist-series))
