;; Popup menus for back/forward navigation.

;; Copyright 2007 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(define-module (chart back-menu)
  #:use-module (ice-9 receive)
  #:use-module (srfi srfi-1)
  #:use-module (gtk gtk)
  #:use-module (gtk gdk)
  #:use-module (chart extra-gtk)
  #:use-module (chart main)
  #:use-module (chart misc))


(define (c-gtk-container-num-children container)
  (length (gtk-container-children container)))


;;-----------------------------------------------------------------------------
;; generated menu

(gtk-class-new 'GtkMenu "ChartGeneratedMenu")

(let-toplevel ((rec-type (make-record-type "generated-menu"
					   '(lenproc getproc callback dirty))))
  (define rec-new  	  (record-constructor rec-type))
  (define rec-lenproc  (rec-accessor rec-type 'lenproc))
  (define rec-getproc  (rec-accessor rec-type 'getproc))
  (define rec-callback (rec-accessor rec-type 'callback))
  (define rec-dirty!   (rec-modifier rec-type 'dirty))

  (define (freshen genmenu)
    (let* ((rec      (c-prop genmenu))
	   (wantlen  ((rec-lenproc rec)))
	   (getproc  (rec-getproc rec)))

      ;; create extra items if necessary
      (do ((i (- (c-gtk-container-num-children genmenu) 2) (1+ i)))
	  ((>= i wantlen))
	(let ((item (gtk-menu-item-new-with-label "")))
	  (gtk-menu-append genmenu item)
	  (gtk-signal-connect item "activate"
	    (lambda ()
	      ((rec-callback rec) i)))))

      ;; hide any excess items
      (for-each gtk-widget-hide
		(drop (gtk-container-children genmenu) (+ wantlen 2)))

      (for-each (lambda (n item)
		  (gtk-label-set-text (gtk-bin-get-child item)
				      (getproc n))
		  (gtk-widget-show item))
		(iota wantlen) (cddr (gtk-container-children genmenu)))

      (rec-dirty! rec #f)))

  (define-public (genmenu-new heading lenproc getproc callback)
    (let ((genmenu (gtk-widget-new 'ChartGeneratedMenu))
	  (rec  (rec-new lenproc getproc callback #t)))
      (set! (c-prop genmenu) rec)

      (let ((item (gtk-menu-item-new-with-label heading)))
	(gtk-menu-shell-append genmenu item)
	(gtk-widget-show item))

      (let ((item (gtk-tearoff-menu-item-new)))
	(gtk-menu-shell-append genmenu item)
	(gtk-widget-show item))

      (gtk-signal-connect genmenu "map-event"
	(lambda (event)
	  (freshen genmenu)))

      genmenu))

  (define-public (genmenu-refresh genmenu)
    (rec-dirty! (c-prop genmenu) #t)
    (if (gtk-widget-mapped genmenu)
	(freshen genmenu))))


;;-----------------------------------------------------------------------------
;; back/forward menus

(define (go-back n)
  (receive (pre post)
      (split-at chart-open-history (1+ n))
    (set! chart-open-forward (append (reverse pre) chart-open-forward))
    (set! chart-open-history post))
  (receive-list (symbol name smarker . rest)
      (first chart-open-history)
    (chart-open symbol #:smarker smarker)))

(define (go-forward n)
  (receive (pre post)
      (split-at chart-open-forward (1+ n))
    (set! chart-open-history (append pre chart-open-history))
    (set! chart-open-forward post))
  (receive-list (symbol name smarker . rest)
      (first chart-open-history)
    (chart-open symbol #:smarker smarker)))

(define (history-element->menu-string elem)
  (receive-list (symbol name)
      elem
    (format #f "~a - ~a" symbol name)))

(define back-menu #f)
(define (back-menu-init)
  (set! back-menu
	(genmenu-new (_ "Back to:")
		     (lambda ()
		       (max 0 (1- (length chart-open-history))))
		     (lambda (n)
		       (history-element->menu-string
			(list-ref chart-open-history (1+ n))))
		     go-back))
  (notify-connect 'chart-open-history-changed
    (lambda (dummy)
      (genmenu-refresh back-menu)))
  (set! back-menu-init noop)) ;; once only

(define forward-menu #f)
(define (forward-menu-init)
  (set! forward-menu (genmenu-new (_ "Forward to:")
				  (lambda ()
				    (length chart-open-forward))
				  (lambda (n)
				    (history-element->menu-string
				     (list-ref chart-open-forward n)))
				  go-forward))
  (notify-connect 'chart-open-history-changed
    (lambda (dummy)
      (genmenu-refresh forward-menu)))
  (set! forward-menu-init noop)) ;; once only

(define-public (back-menu-popup event)
  (back-menu-init)
  (gtk-menu-popup back-menu #f #f
		  (gdk-event-button event)
		  (gdk-event-time event)))

(define-public (forward-menu-popup event)
  (forward-menu-init)
  (gtk-menu-popup forward-menu #f #f
		  (gdk-event-button event)
		  (gdk-event-time event)))
