;; Bars drawing.

;; Copyright 2003, 2004, 2005, 2006, 2007 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(define-module (chart bar)
  #:use-module (ice-9 and-let-star)
  #:use-module (gtk gdk)
  #:use-module (gtk gtk)
  #:use-module (chart extra-gdk)
  #:use-module (chart graph)
  #:use-module (chart indicator)
  #:use-module (chart misc)
  #:use-module (chart view))


;; Draw the data in SERIES onto the given GRAPH widget, as bars extending
;; from 0 to each closing price.  This is for volume or open interest, but
;; negative prices are allowed and result in bars extending downwards from
;; zero.
;;
(define-public (bar-draw graph gc series)
  (let* ((lo         (graph-draw-x-lo graph series))
	 (hi         (graph-draw-x-hi graph series))
	 (array      (series-array-closes series lo hi))
	 (win        (gtk-widget-window graph))
	 (height     (c-gdk-drawable-get-height win))
	 (scale-y    (graph-scale-y-proc graph))
	 ;; bars 2/3 of step and at least 1 pixel
	 (bar-width  (max 1 (round-exact (* 0.666 scale-x-step))))
	 ;; centre in step, and round down so in 2 pixels it's at the left
	 (bar-offset (quotient (- scale-x-step bar-width) 2))
	 (win-p-hi   (graph-y->price-inexact graph -1))
	 (win-p-lo   (graph-y->price-inexact graph height)))

    (do ((i lo (1+ i))
	 (x (+ bar-offset (graph-scale-x graph lo)) (+ x scale-x-step)))
	((> i hi))
      (and-let* ((p1 (array-ref array i)))
	(let* ((p-lo (min 0 p1))
	       (p-hi (max 0 p1)))
	  (if (overlap-inclusive? p-lo p-hi win-p-lo win-p-hi)
	      (let ((y-lo (scale-y (min win-p-hi p-hi)))
		    (y-hi (scale-y (max win-p-lo p-lo))))
		(gdk-draw-rectangle win gc #t x y-lo
				    bar-width
				    (- y-hi y-lo)))))))))
