;; Common code for barchart.com customers KCBT and MGEX.

;; Copyright 2005, 2006, 2007 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(define-module (chart barchart-customer)
  #:use-module (ice-9 and-let-star)
  #:use-module (ice-9 receive)
  #:use-module (ice-9 regex)
  #:use-module (srfi srfi-1)
  #:use-module (chart download)
  #:use-module (chart http)
  #:use-module (chart latest)
  #:use-module (chart misc))


;;-----------------------------------------------------------------------------
;; intraday
;;
;; The charts at KCBT and MGEX come from barchart.com and are the same style
;; as barchart itself, ie. download a page which contains a generated
;; 4-digit numbered .gif file.  The links from www.kcbt.com and www.mgex.com
;; are preferred though, since the pages to get that gif file are only
;; 26kbytes instead of 50k from barchart.com's native links.
;;
;; Z10 MEDHI gives 7 days, Z05 MED gives 1.5 or 2, Z10 HIGH gives about 10 days

(define mode-seven (_ "_7 Days"))

(define-public (barchart-customer-intraday
		symbol-pred      ;; called (SYMBOL-PRED symbol)
		tdate-time-proc  ;; called (TDATE-TIME_PROC symbol)
		base-url)        ;; string

  (define (customer-intraday symbol mode)
    (and-let* ((mdate (latest-symbol-mdate
		       symbol (first (tdate-time-proc symbol)))))
      (receive (headers body)
	  (http-request (string-append base-url
				       "&sly=N&sym="
				       (chart-symbol-commodity symbol)
				       (mdate->MYY-str mdate)
				       (cond ((string=? mode mode-seven)
					      "&data=Z10&den=MEDHI")
					     (else ;; 1 1/2 Days
					      "&data=Z05"))))
	(and-let* ((  (http-headers-ok? headers))
		   (m (must-match
		       (string-match "<img src=\"([^\"]+)\"" body))))
	  (match:substring m 1)))))

  (intraday-url-proc! #:symbol-pred symbol-pred
		      #:url-proc    customer-intraday
		      #:modes-proc  (lambda ()
				      (list (_ "_1 1/2 Days")
					    mode-seven))))
