;; Cairo and Alexandria Stock Exchange (CASE) setups.

;; Copyright 2005, 2006, 2007, 2008 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(define-module (chart cairo)
  #:use-module (srfi srfi-13)
  #:use-module (chart download)
  #:use-module (chart misc)
  #:use-module (chart yahoo))


;; Eg. CASE30.CA, but perhaps nothing else as of March 2007
;;
(define (cairo-symbol? symbol)
  (string-suffix? ".CA" symbol))

;; ^CASE30
;; ^CCSI
;; ^DJEG20
;; ^DJEG20E
;; ^DJEG20D
;; ^DJEG20T
;; ^DJEG20ET
;; ^DJEG20DT
;; ^DWEG -- dow/wiltshire egypt
;;
(define (yahoo-index-symbol-cairo? symbol)
  (or (string-prefix? "^CASE" symbol)
      (string-prefix? "^CC" symbol)
      (string-prefix? "^DJEG" symbol)
      (string-prefix? "^DWEG" symbol)))

(define-public timezone-cairo
  (make-procedure-with-cache-0
   (lambda ()
     (timezone-choose (_ "Cairo")
		      '("Africa/Cairo"
			;; last Fri in Apr to last Thu in Aug, at midnight
			;; (that's midnight at the beginning of Fri to
			;; start, and midnight at the end of the Thu to
			;; stop, it seems)
			;; tzdata has this as an assumption rather than a
			;; certainty though
			"EET-2EEDT,M4.5.5/0,M8.5.4/24")
		      "EET-2"))))

(symbol-timezone! cairo-symbol?             (_ "Cairo") timezone-cairo)
(symbol-timezone! yahoo-index-symbol-cairo? (_ "Cairo") timezone-cairo)
(yahoo-quote-delay! cairo-symbol? 0) ;; indexes not delayed

;; only the home page for now ...
;;
(weblink-handler! (list cairo-symbol? yahoo-index-symbol-cairo?)
  (lambda (symbol)
    (list (_ "_CASE Home Page")
	  (_ "Open web browser at the Cairo and Alexandria Stock Exchange home page")
	  (lang-select '(("ar" "http://www.egyptse.com")
			 ("en" "http://www.egyptse.com/index.asp"))))))
