;; Caracas Stock Exchange setups.

;; Copyright 2007 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(define-module (chart caracas)
  #:use-module (srfi srfi-13)
  #:use-module (chart download)
  #:use-module (chart misc)
  #:use-module (chart yahoo))


;; There's a couple like IBC.CR which show up in an index search (for
;; caracas) at yahoo, but no actual data it seems (as of March 2007).
;;

(define-public (caracas-symbol? symbol)
  (string-suffix? ".CR" symbol))

;; ^VEDOW
;; ^VEDOWD
;; ^DWVE
;; ^DWVED
;; ^DWVET
;; ^DWVEDT
;;
(define (yahoo-index-symbol-caracas? symbol)
  (or (string-prefix? "^VE" symbol)
      (string-prefix? "^DWVE" symbol)))

(define-public timezone-caracas
  (make-procedure-with-cache-0
   (lambda ()
     (timezone-choose (_ "Caracas")
		      '("America/Caracas"
			;; no DST, 4:30 post 2008
			"VET+4:30")
		      "VET+4:30"))))

(symbol-timezone! caracas-symbol?             (_ "Caracas") timezone-caracas)
(symbol-timezone! yahoo-index-symbol-caracas? (_ "Caracas") timezone-caracas)
(yahoo-quote-delay! caracas-symbol? 20)


;; only the home page for now ...
;;
(weblink-handler! (list caracas-symbol? yahoo-index-symbol-caracas?)
  (lambda (symbol)
    (list (_ "_Caracas Stock Exchange")
	  (_ "Open web browser at the Caracas Stock Exchange home page")
	  (lang-select '(("es" "http://www.bolsadecaracas.com")
			 ("en" "http://www.bolsadecaracas.com/eng/index.jsp"))))))
