;; Chicago Board of Trade (CBOT) setups.

;; Copyright 2005, 2006, 2007 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(define-module (chart cbot)
  #:use-module (ice-9 and-let-star)
  #:use-module (srfi srfi-1)
  #:use-module (srfi srfi-13)
  #:use-module (chart barchart)
  #:use-module (chart download)
  #:use-module (chart misc)
  #:use-module (chart yahoo))


(define-public (cbot-symbol? symbol)
  (string-suffix? ".CBOT" symbol))

(define (cbot-european-symbol? symbol)
  (and (cbot-symbol? symbol)
       (member (chart-symbol-commodity symbol)
	       '("GBL"     ;; bund
		 "GBM"     ;; bobl
		 "GBS")))) ;; schatz

(symbol-timezone! cbot-european-symbol? (_ "Europe")  timezone-europe-cet)
(symbol-timezone! cbot-symbol?          (_ "Chicago") timezone-chicago)


;;-----------------------------------------------------------------------------
;; weblink - contract specs

(load "cbot-info.scm")

(weblink-handler! cbot-symbol?
  (lambda (symbol)
    (list (_ "CBOT _Contract Specifications")
	  (_ "Open web browser at the Chicago Board of Trade contract specifications for this commodity")
	  (and-let* ((elem (assoc (chart-symbol-commodity symbol)
				  cbot-specs-list member)))
	    (cdr elem)))))


;;-----------------------------------------------------------------------------
;; yahoo for quotes

(set! yahoo-suffix-transform-alist
      (cons '(".CBOT" . ".CBT")
	    yahoo-suffix-transform-alist))
(set! yahoo-futures-symbol-predlist
      (cons cbot-symbol?
	    yahoo-futures-symbol-predlist))
(yahoo-quote-delay! cbot-symbol?  10)


;;-----------------------------------------------------------------------------
;; barchart for database and for intraday graphs

(set! barchart-download-symbol-predlist
      (cons cbot-symbol? barchart-download-symbol-predlist))

(set! barchart-suffix-delay-alist (acons ".CBOT" 20
					 barchart-suffix-delay-alist))

(set! barchart-commodity-mung-alist
      (acons cbot-symbol?
	     '(;; agricultural
	       ;; "C"         ;; corn
	       ;; "ZC"        ;; corn electronic
	       ("XC" . "XN")  ;; corn mini-size
	       ;; "YC"        ;; corn mini-size (electronic)
	       ;; "S"         ;; soybeans
	       ;; "ZS"        ;; soybeans electronic
	       ("XB" . "XK")  ;; soybeans mini-size
	       ;; "YK"        ;; soybeans mini-size (electronic)
	       ;; "BO"        ;; soybean oil
	       ;; "ZL"        ;; soybean oil electronic
	       ;; "SM"        ;; soybean meal
	       ;; "ZM"        ;; soybean meal electronic
	       ("BS" . #f)    ;; south american soybeans
	       ("ZK" . "ZA")  ;; south american soybeans electronic
	       ("BCX" . "CS") ;; soybean crush
	       ;; "W"         ;; wheat
	       ;; "ZW"        ;; wheat electronic
	       ;; "XW"        ;; wheat mini-size
	       ;; "YW"        ;; wheat mini-size (electronic)
	       ("AC" . "AK")  ;; ethanol
	       ("ZE" . "ZK")  ;; ethanol electronic
	       ("FZE" . "FZ") ;; ethanol forward swap
	       ;; "O"         ;; oats
	       ;; "ZO"        ;; oats electronic
	       ;; "RR"        ;; rough rice
	       ;; "ZR"        ;; rough rice electronic

	       ;; interest rate
	       ;; "US"        ;; 30-year bond
	       ;; "ZB"        ;; 30-year bond electronic
	       ;; "TY"        ;; 10-year bond
	       ;; "ZN"        ;; 10-year bond electronic
	       ;; "FV"        ;; 5-year bond
	       ;; "ZF"        ;; 5-year bond electronic
	       ;; "TU"        ;; 2-year bond
	       ;; "ZT"        ;; 2-year bond electronic
	       ;; ("DJCBTI" . #f) ;; DJ CBOT treasury index
	       ;;                 ;; only an index, not traded (?)
	       ;; "NZ"        ;; 30-year swaps
	       ;; "QS"        ;; 30-year swaps electronic
	       ;; "NI"        ;; 10-year interest rate swap
	       ;; "SR"        ;; 10-year interest rate swap electronic
	       ("NG" . "NJ")  ;; 5-year interest rate swap
	       ;; "SA"        ;; 5-year interest rate swap electronic
	       ;; "YE"        ;; eurodollar mini (electronic)
	       ;; "FF"        ;; fed funds
	       ;; "ZQ"        ;; fed funds electronic
	       ;; "MB"        ;; 10-year muni bond
	       ;; "ZU"        ;; 10-year muni bond electronic

	       ;; Dow
	       ;; "YM"       ;; DJIA mini $5 (electronic)
	       ;; "DJ"       ;; DJIA $10
	       ;; "ZD"       ;; DJIA $10 electronic
	       ;; "DD"       ;; Big DJIA $25 (electronic)
	       ("ER" . "AH") ;; DJ AIG excess index
	       ("RE" . "DH") ;; DJ real estate index
	       ;; "CX"       ;; liquid 50 swap index

	       ;; Metals
	       ;; "ZG"       ;; gold
	       ;; "YG"       ;; gold mini
	       ;; "ZI"       ;; silver
	       ;; "YI"       ;; silver mini
	       )
	     barchart-commodity-mung-alist))

;; these gone as of 2007 (?)
;; 	       ;; "AI"       ;; DJ AIG commodity index
;; 	       (#f . "AJ")   ;; DJ AIG commodity index yield
;; 	       ("GBL" . "GL") ;; bund (electronic)
;; 	       ("GBM" . "GM") ;; bobl (electronic)
;; 	       ("GBS" . "GS") ;; schatz (electronic)
