;; Chaikin A/D Oscillator.

;; Copyright 2006, 2007, 2009 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(define-module (chart chaikin-osc)
  #:use-module (chart accdist)
  #:use-module (chart indicator)
  #:use-module (chart ema)
  #:use-module (chart misc))


(define (chaikin-osc-fill series new-array lo hi)
  (receive-list (fast-count slow-count warmup accdist-series)
      (series-data series)
    (let* ((start     (series-prev-n-closes-pos accdist-series
						#:before lo #:count warmup))
	   (array     (series-array accdist-series start hi))
	   (fast-proc (ema-calc-proc fast-count))
	   (slow-proc (ema-calc-proc slow-count)))

      ;; warmup
      (do ((i start (1+ i)))
	  ((>= i lo))
	(if (array-ref array i)
	    (begin
	      (fast-proc (array-ref array i))
	      (slow-proc (array-ref array i)))))

      ;; points lo to hi inclusive
      (do ((i lo (1+ i)))
	  ((> i hi))
	(if (array-ref array i)
	    (let ((fast (fast-proc (array-ref array i)))
		  (slow (slow-proc (array-ref array i))))
	      (and fast slow
		   (array-set! new-array (- fast slow) i))))))))

(define-public (chaikin-osc-series series fast-count slow-count)
  (let ((new-series (series-copy series))
	(warmup     (1+ (ema-warmup-count slow-count))))
    (series-name!     new-series (format #f (_ "Chaikin Osc ~a,~a")
					 fast-count slow-count))
    (series-datatype! new-series chaikin-osc-datatype)
    (series-parent!   new-series series)
    (series-data!     new-series (list fast-count
				       slow-count
				       warmup
				       (accdist-series series)))
    (series-require!  new-series (indicator-require-parts chaikin-osc-fill
							  warmup))
    (series-array!    new-series #f)

    (series-decimals!  new-series 0)
    (series-dividends! new-series '())
    (series-notes!     new-series '())
    new-series))

(define-public chaikin-osc-datatype
  (datatype-new!
   #:key       'chaikin-osc
   #:name      (_ "Chaikin Oscillator")
   #:units     'chaikin-osc
   #:linestyle 'line
   #:hlines    '(0)
   #:manual    (_ "CONTEXT(manual-node): Chaikin Oscillator")
   #:proc      chaikin-osc-series
   #:parameter-list
   (list (datatype-param-new
	  #:name    (_ "Fast days")
	  #:key     'chaikin-osc-fast-days
	  #:type    'integer
	  #:default 3
	  #:minimum 1)
	 (datatype-param-new
	  #:name    (_ "Slow days")
	  #:key     'chaikin-osc-slow-days
	  #:type    'integer
	  #:default 10
	  #:minimum 1))))
