;; Clock widget.

;; Copyright 2005, 2007 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(define-module (chart clock)
  #:use-module (ice-9 optargs)
  #:use-module (gtk gtk)
  #:use-module (chart main))


;; PAIR is seconds and microseconds from a `gettimeofday' call.
;; Return the number of milliseconds until the next minute ticks over.
;;
;; An extra `milliseconds-margin' is added, to try to ensure we don't wake
;; up earlier than desired if gtk-timeout-add ends up rounding to a 10ms
;; clock tick boundary.  `milliseconds-margin' is 2 clock ticks (ie. 20ms),
;; and a minimum 1 millisecond for the unlikely case a tick is smaller than
;; a millisecond.
;;
(define (timeofday->milliseconds-to-next-minute pair)
  (let* ((secs  (modulo (car pair) 60))
         (usecs (cdr pair)))
    (+ (- 60000 (* secs 1000) (quotient usecs 1000))
       milliseconds-margin)))

(define milliseconds-margin
  (max 1 (quotient 2000 internal-time-units-per-second)))


;; It's not essential to make a separate class for things like this, just
;; using a plain label each time would work fine.  But a class should allow
;; colours and fonts to be customized from the user's .gtkrc file.
;;
(gtk-class-new 'GtkLabel "ChartClock")

;; return a clock widget
;; ZONE is an optional timezone, a TZ string as for `localtime'
;;
(define*-public (clock-new #:optional zone)
  ;; for `apply' on localtime
  (set! zone (if zone (list zone) '()))

  (let ((clock (gtk-widget-new 'ChartClock)))
    (define (update-time)
      (let* ((pair (gettimeofday))
	     (secs (car pair)))
	(gtk-label-set-text clock
			    (strftime "%H:%M" (apply localtime secs zone)))
	(gtk-timeout-add (timeofday->milliseconds-to-next-minute pair)
	  update-time)))
    (update-time)
    clock))

;; Put a clock widget in the main menubar (at the right).
;; ZONE is an optional timezone, a TZ string as for `localtime'.
;;
(define*-public (clock-in-menubar #:optional zone)
  (let ((item (gtk-menu-item-new)))
    (gtk-menu-item-right-justify item)
    (gtk-container-add item (clock-new zone))
    (gtk-menu-shell-append menubar-widget item)))
