;; Close, high, low and open series.

;; Copyright 2007 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(define-module (chart closes)
  #:use-module (chart misc))



(define (make-column-require-proc column)
  (lambda (series lo hi)
    (make-shared-array-column (series-array (series-parent series) lo hi)
			      column)))


;;-----------------------------------------------------------------------------

(define (opens-series series)
  (let ((new-series (series-copy series)))
    (series-parent!    new-series series)
    (series-require!   new-series (make-column-require-proc 0))
    (series-datatype!  new-series opens-datatype)
    (series-name!      new-series (_ "Opens"))
    (series-array!     new-series #f)
    new-series))

(define-public opens-datatype
  (datatype-new! #:key      'opens
		 #:name     (_ "Opens")
		 #:priority -1000
		 #:proc     opens-series))

;;-----------------------------------------------------------------------------

(define (highs-series series)
  (let ((new-series (series-copy series)))
    (series-parent!    new-series series)
    (series-require!   new-series (make-column-require-proc 1))
    (series-datatype!  new-series highs-datatype)
    (series-name!      new-series (_ "Highs"))
    (series-array!     new-series #f)
    new-series))

(define-public highs-datatype
  (datatype-new! #:key      'highs
		 #:name     (_ "Highs")
		 #:priority -1000
		 #:proc     highs-series))

;;-----------------------------------------------------------------------------

(define (lows-series series)
  (let ((new-series (series-copy series)))
    (series-parent!    new-series series)
    (series-require!   new-series (make-column-require-proc 2))
    (series-datatype!  new-series lows-datatype)
    (series-name!      new-series (_ "Lows"))
    (series-array!     new-series #f)
    new-series))

(define-public lows-datatype
  (datatype-new! #:key      'lows
		 #:name     (_ "Lows")
		 #:priority -1000
		 #:proc     lows-series))

;;-----------------------------------------------------------------------------

(define (closes-series series)
  (let ((new-series (series-copy series)))
    (series-parent!    new-series series)
    (series-require!   new-series (make-column-require-proc 3))
    (series-datatype!  new-series closes-datatype)
    (series-name!      new-series (_ "Closes"))
    (series-array!     new-series #f)
    new-series))

(define-public closes-datatype
  (datatype-new! #:key      'closes
		 #:name     (_ "Closes")
		 #:priority -1000
		 #:proc     closes-series))
