/* Chart compatibility code, shared among libraries.

   Copyright 2003, 2005, 2006, 2007 Kevin Ryde

   This file is part of Chart.

   Chart is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 3, or (at your option) any later
   version.

   Chart is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
   FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
   details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>. */


#include "config.h"

#include <ctype.h>
#include <errno.h>
#include <string.h>
#include <stdio.h>     /* for NULL */

#if HAVE_UNISTD_H
#include <unistd.h>
#endif

#if HAVE_MALLOC_H
#include <malloc.h>
#endif

#include <libguile.h>

#include "compat.h"



/*---------------------------------------------------------------------------*/
/* guile 1.8 C stuff make available in guile 1.6 */

#if SCM_MAJOR_VERSION == 1 && SCM_MINOR_VERSION == 6

/* This code is included in both misc.c and zlib.c.  It's duplicated in the
   two modules because according to libtool having one module call functions
   in another isn't portable.  */

void *
chart_scm_malloc (size_t size)
{
  char *p = scm_must_malloc (size, "chart");
  scm_done_free (size);
  return p;
}

char *
chart_scm_to_locale_stringn (SCM str, size_t *lenp)
#define FUNC_NAME "scm_to_locale_stringn"
{
  const char *c_str;
  size_t c_len;
  char *ret;

  /* ordinary strings and shared substrings */
  SCM_ASSERT_TYPE (scm_is_string (str), str, SCM_ARG1,
                   "scm_to_locale_stringn", "string");
  c_str = SCM_ROCHARS (str);
  c_len = SCM_STRING_LENGTH (str);

  if (lenp == NULL)
    {
      if (memchr (c_str, '\0', c_len))
        scm_misc_error (FUNC_NAME, "string contains #\\nul character: ~S",
                        scm_list_1 (str));
    }
  else
    *lenp = c_len;

  ret = scm_malloc (c_len + 1);
  memcpy (ret, c_str, c_len);
  scm_remember_upto_here_1 (str);
  ret[c_len] = '\0';
  return ret;
}
#undef FUNC_NAME

char *
chart_scm_to_locale_string (SCM obj)
{
  return scm_to_locale_stringn (obj, NULL);
}


SCM
chart_scm_car (SCM obj)
#define FUNC_NAME "scm_car"
{
  SCM_VALIDATE_CONS (SCM_ARG1, obj);
  return SCM_CAR (obj);
}
#undef FUNC_NAME

SCM
chart_scm_cdr (SCM obj)
#define FUNC_NAME "scm_cdr"
{
  SCM_VALIDATE_CONS (SCM_ARG1, obj);
  return SCM_CDR (obj);
}
#undef FUNC_NAME

SCM
chart_scm_caar (SCM obj)
{
  return scm_car (scm_car (obj));
}

#endif /* Guile 1.6 */
