/* Chart guile compatibility bits.

   Copyright 2005, 2006, 2007 Kevin Ryde

   This file is part of Chart.

   Chart is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 3, or (at your option) any later
   version.

   Chart is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
   FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
   details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>. */


#ifndef CHART_COMPAT_H
#define CHART_COMPAT_H 1


/*---------------------------------------------------------------------------*/
/* guile 1.8 C stuff make available in guile 1.6 */

#if SCM_MAJOR_VERSION == 1 && SCM_MINOR_VERSION == 6

#include <string.h>  /* for memchr, memcpy */

#define scm_is_false(obj)    SCM_FALSEP (obj)
#define scm_is_true(obj)     SCM_NFALSEP (obj)
#define scm_is_null(obj)     SCM_NULLP (obj)
#define scm_is_eq(x,y)       SCM_EQ_P (x, y)
#define scm_is_number(obj)   scm_is_true (scm_number_p (obj))
#define scm_is_real(obj)     scm_is_true (scm_real_p (obj))

#define scm_to_int(scm)      scm_num2int (scm, SCM_ARGn, "scm_to_int")
#define scm_to_long(scm)     scm_num2long (scm, SCM_ARGn, "scm_to_long")
#define scm_to_uint(scm)     scm_num2uint (scm, SCM_ARGn, "scm_to_uint")
#define scm_to_double(scm)   scm_num2double (scm, SCM_ARGn, "scm_to_double")
#define scm_from_int(n)      scm_int2num (n)
#define scm_from_long(n)     scm_long2num (n)
#define scm_from_double(x)   scm_double2num (x)

/* strictly speaking these depend on whether size_t is int or long, but we
   don't need the full range for present uses */
#define scm_to_size_t(x)     scm_num2uint (x, SCM_ARGn, "scm_to_size_t")
#define scm_from_size_t(x)   scm_ulong2num (x)

/* as supported by scm_to_locale_stringn() below */
#define scm_is_string(obj)   (SCM_STRINGP (obj) || SCM_SUBSTRP (obj))

#define scm_from_locale_string(str)     scm_makfrom0str(str)
#define scm_from_locale_stringn(s,len)  scm_mem2string(s,len)
#define scm_from_locale_symbol(s)       scm_str2symbol(s)

#define scm_gc_malloc(size, what)     scm_must_malloc (size, what)
#define scm_gc_free(ptr, size, what)    \
  do {                                  \
    scm_must_free (ptr);                \
    scm_done_free (size);               \
  } while (0)

/* fakery designed only for a few strings */
#define scm_dynwind_begin(flags)  SCM __dynwind_keep = SCM_EOL
#define scm_dynwind_free(str)                                           \
  do {                                                                  \
    __dynwind_keep = scm_cons (scm_take0str (str), __dynwind_keep);     \
  } while (0)
#define scm_dynwind_end()   scm_remember_upto_here_1 (__dynwind_keep)


/* compat.c bits */

#define scm_malloc  chart_scm_malloc
#define scm_car     chart_scm_car
#define scm_cdr     chart_scm_cdr
#define scm_caar    chart_scm_caar
#define scm_to_locale_string  chart_scm_to_locale_string
#define scm_to_locale_stringn chart_scm_to_locale_stringn

void *chart_scm_malloc (size_t size);
SCM chart_scm_car (SCM obj);
SCM chart_scm_cdr (SCM obj);
SCM chart_scm_caar (SCM obj);
char *chart_scm_to_locale_string (SCM obj);
char *chart_scm_to_locale_stringn (SCM str, size_t *lenp);

#define scm_cddr(x)     scm_cdr (scm_cdr (x))
#define scm_cadr(x)     scm_car (scm_cdr (x))

/* Copied from guile 1.6 with scm_c_export added.
   It doesn't work to tack another SCM_SNARF_INIT onto the end of
   SCM_DEFINE, because the snarfer doesn't recognise its ^^ marker in the
   middle of a line (after cpp collapses whitespace). */
#define SCM_DEFINE_PUBLIC(FNAME, PRIMNAME, REQ, OPT, VAR, ARGLIST, DOCSTRING) \
SCM_SNARF_HERE(\
static const char s_ ## FNAME [] = PRIMNAME; \
SCM FNAME ARGLIST\
)\
SCM_SNARF_INIT(\
scm_c_define_gsubr (s_ ## FNAME, REQ, OPT, VAR, \
                    (SCM_FUNC_CAST_ARBITRARY_ARGS) FNAME); \
scm_c_export (s_ ## FNAME, NULL); )\
SCM_SNARF_DOCS(primitive, FNAME, PRIMNAME, ARGLIST, REQ, OPT, VAR, DOCSTRING)

#endif /* Guile 1.6 */



#endif /* CHART_COMPAT_H */
