;; Copenhagen Stock Exchange setups.

;; Copyright 2005, 2006, 2007 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(define-module (chart copenhagen)
  #:use-module (srfi srfi-13)
  #:use-module (chart download)
  #:use-module (chart misc)
  #:use-module (chart yahoo))


(define-public (copenhagen-symbol? symbol)
 (string-suffix? ".CO" symbol))

;; ^KFX
(define-public (yahoo-index-symbol-copenhagen? symbol)
  (string-prefix? "^KF" symbol))

(symbol-timezone! copenhagen-symbol? (_ "Copenhagen") timezone-europe-cet)
(symbol-timezone! yahoo-index-symbol-copenhagen? (_ "Copenhagen") timezone-europe-cet)
(yahoo-quote-delay! copenhagen-symbol? 15)

;; http://omxgroup.com/nordicexchange/Abouttrading/tradinginformation/tradinghours/
;; Equities from 9:00 or 9:15, bonds from 8:00 or 8:30
;; Closing 16:50 or with further call to 17:00, bulletin board to 17:30
;;
(yahoo-quote-lock! copenhagen-symbol?
		   #,(hms->seconds 8 0 0) #,(hms->seconds 17 30 0))
(yahoo-quote-lock! yahoo-index-symbol-copenhagen?
		   #,(hms->seconds 8 0 0) #,(hms->seconds 17 0 0))


;; only the home page for now ...
;; redirects to OMX nordic exchange ...
;;
(weblink-handler! (list copenhagen-symbol? yahoo-index-symbol-copenhagen?)
  (lambda (symbol)
    (list (_ "_Copenhagen Stock Exchange Home Page")
	  (_ "Open web browser at the Prague Stock Exchange home page")
	  "http://www.xcse.dk")))
