;; Coppock curve indicator.

;; Copyright 2006, 2007 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(define-module (chart coppock)
  #:use-module (ice-9 and-let-star)
  #:use-module (chart indicator)
  #:use-module (chart misc)
  #:use-module (chart roc)
  #:use-module (chart wma))


(define period1 11)
(define period2 14)
(define smooth  10)

(define warmup (+ smooth -1 (max period1 period2)))


;; Each call (PROC val) enters a new value into the calculation, and the
;; return is the coppock curve on that value, or #f if there hasn't been
;; enough points yet.
;;
(define-public (coppock-calc-proc)
  (let ((roc1-proc (roc-calc-proc period1))
	(roc2-proc (roc-calc-proc period2))
	(wma-proc  (wma-calc-proc smooth)))

    (lambda (x)
      (let ((roc1 (roc1-proc x))
	    (roc2 (roc2-proc x)))
	(and roc1 roc2
	     (wma-proc (+ roc1 roc2)))))))

(define (coppock-fill series new-array lo hi)
  (let* ((parent (series-parent series))
	 (array  (series-array-closes parent lo hi))
	 (cop    (coppock-calc-proc)))

    ;; warmup
    (for-each cop (series-prev-n-closes parent #:before lo #:count warmup))

    ;; points lo to hi inclusive
    (do ((i lo (1+ i)))
	((> i hi))
      (and-let* ((this (array-ref array i)))
	(array-set! new-array (cop this) i)))))

(define-public (coppock-series series)
  (let ((new-series (series-copy series)))
    (series-name!      new-series (_ "Coppock"))
    (series-datatype!  new-series coppock-datatype)
    (series-parent!    new-series series)
    (series-data!      new-series #f)
    (series-require!   new-series (indicator-require-parts coppock-fill
							   warmup))
    (series-array!     new-series #f)
    (series-decimals!  new-series 0)
    (series-dividends! new-series '())
    (series-notes!     new-series '())

    new-series))


(define-public coppock-datatype
  (datatype-new!
   #:key       'coppock
   #:name      (_ "Coppock Curve")
   #:units     'coppock
   #:linestyle 'line
   #:hlines    '(0)
   #:manual    (_ "CONTEXT(manual-node): Coppock Curve")
   #:proc      coppock-series))
