;; Curses terminal control.

;; Copyright 2004, 2005, 2006, 2007 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(define-module (chart curses)
  #:use-module (chart config)
  #:use-module (chart extra-glib))


(load-extension (g-build-filename chart-config-libchartdir "curses.la")
		"chart_curses_init")


;; fallback to running program if no ncurses library
(if (not (defined? 'tput))
    (begin
      (use-modules (ice-9 optargs)
		   (chart misc))

      (define-public tput-available?
	(make-procedure-with-cache-0
	 (lambda ()
	   (eqv? 0 (status:exit-val
		    (system "(tput -V >/dev/null) 2>/dev/null"))))))

      (define*-public (setupterm #:optional term)
	(or (string? term)
	    (error "bad terminal type"))
	(let* ((command (if term
			    (string-append "(tput -T" term
					   " >/dev/null) 2>/dev/null")
			    "(tput >/dev/null) 2>/dev/null"))
	       (status  (system command)))
	  (if (and (status:exit-val status)
		   (<= (status:exit-val status) 1))
	      #f  ;; success
	      "tput program error")))

      (define*-public (tput cap #:optional arg)
	(or (number? arg)
	    (not arg)
	    (error "tput: bad argument type"))
	(eqv? 0 (status:exit-val
		 (system (format #f "(tput ~a ~a) 2>/dev/null"
				 cap (or arg ""))))))))


;;-----------------------------------------------------------------------------

;; write a curses control for COLOUR to the current output port
;; COLOUR should be a symbol per the alists below
;;
;; setaf and setf are both tried, usually an ANSI terminal has just setaf,
;; an older terminal may have setf
;;
(define-public (tput-colour colour)
  (or (tput "setaf"
	    (or (assq-ref '((black   . 0)
			    (red     . 1)
			    (green   . 2)
			    (yellow  . 3)
			    (blue    . 4)
			    (magenta . 5)
			    (cyan    . 6)
			    (white   . 7)) colour)
		(error "tput-colour: unrecognised colour symbol:" colour)))

      (tput "setf"
	    (or (assq-ref '((black   . 0)
			    (blue    . 1)
			    (green   . 2)
			    (cyan    . 3)
			    (red     . 4)
			    (magenta . 5)
			    (yellow  . 6)
			    (white   . 7)) colour)
		(error "tput-colour: unrecognised colour symbol:" colour)))))
