;; Date entry widget.

;; Copyright 2002, 2003, 2004, 2005, 2006, 2007 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(define-module (chart date)
  #:use-module (gtk gtk)
  #:use-module (chart misc)
  #:use-module (chart extra-gtk))


;; This is the y/m/d date entry fields common to div-edit.scm and
;; ann-edit.scm.

(let-toplevel ((rec-type (make-record-type "graph-element"
					   '(year-spin month-spin day-spin))))
  (define-toplevel rec-new  	  (record-constructor rec-type))
  (define-toplevel rec-year-spin  (rec-accessor rec-type 'year-spin))
  (define-toplevel rec-month-spin (rec-accessor rec-type 'month-spin))
  (define-toplevel rec-day-spin   (rec-accessor rec-type 'day-spin)))

(define rec-prop (make-object-property))

;; return adate currently in DW
(define-public (date-widget-adate dw)
  (let ((rec (rec-prop dw)))
    (ymd->adate (gtk-spin-button-get-value-as-int (rec-year-spin rec))
		(gtk-spin-button-get-value-as-int (rec-month-spin rec))
		(gtk-spin-button-get-value-as-int (rec-day-spin rec)))))

;; set DW to TDATE
(define-public (date-widget-tdate! dw tdate)
  (let ((rec (rec-prop dw)))
    (receive-list (year month day)
	(tdate->ymd tdate)
      (gtk-spin-button-set-value (rec-year-spin rec)  year)
      (gtk-spin-button-set-value (rec-month-spin rec) month)
      (gtk-spin-button-set-value (rec-day-spin rec)   day))))

;; return a new date widget
(define-public (date-widget-new)
  (define dw          (gtk-hbox-new #f 4))
  (define year-spin   (gtk-spin-button-new-with-range 1900 2100 1))
  (define month-spin  (gtk-spin-button-new-with-range 1 12 1))
  (define day-spin    (gtk-spin-button-new-with-range 1 31 1))
  (define day-of-week (gtk-label-new ""))

  (define (day-of-week-update)
    (let ((tm (tm-new)))
      (set-tm:wday tm (modulo (1+ (date-widget-adate dw)) 7))
      (gtk-label-set-text day-of-week (strftime-utf8 " %a" tm))))

  (set! (rec-prop dw) (rec-new year-spin month-spin day-spin))

  ;; today, rounded through tdate->ymd to prev trading day
  (receive-list (year month day)
      (tdate->ymd (tdate-today))
    (gtk-spin-button-set-value year-spin  year)
    (gtk-spin-button-set-value month-spin month)
    (gtk-spin-button-set-value day-spin   day))

  (gtk-box-pack-start dw (gtk-label-new (_ "Year")) #f #f 0)
  (c-gtk-widget-set-usize-units year-spin '(digit . 8) '(height . 1.5))
  (gtk-box-pack-start dw year-spin #f #f 0)
  (gtk-signal-connect (gtk-spin-button-get-adjustment year-spin)
      "value-changed" day-of-week-update)

  (gtk-box-pack-start dw (gtk-label-new (_ "Month")) #f #f 0)
  (c-gtk-widget-set-usize-units month-spin '(digit . 6)'(height . 1.5))
  (gtk-box-pack-start dw month-spin #f #f 0)
  (gtk-signal-connect (gtk-spin-button-get-adjustment month-spin)
      "value-changed" day-of-week-update)

  (gtk-box-pack-start dw (gtk-label-new (_ "Day")) #f #f 0)
  (c-gtk-widget-set-usize-units day-spin '(digit . 6) '(height . 1.5))
  (gtk-box-pack-start dw day-spin #f #f 0)
  (gtk-signal-connect (gtk-spin-button-get-adjustment day-spin)
      "value-changed" day-of-week-update)

  (day-of-week-update) ;; initial
  (gtk-box-pack-start dw day-of-week #f #f 0)

  dw)
