;; Delete dialog.

;; Copyright 2004, 2005, 2006, 2007 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(define-module (chart delete)
  #:use-module (ice-9 and-let-star)
  #:use-module (gtk gtk)
  #:use-module (chart database)
  #:use-module (chart extra-gtk)
  #:use-module (chart main)
  #:use-module (chart misc)
  #:use-module (chart symlists)
  #:use-module (chart view))


;;-----------------------------------------------------------------------------
;; delete dialog
;;
;; Created when needed, then re-used.

(define del-symbol  #f)
(define label       #f)
(define dialog      #f)
(define notes-check #f)
(define notes-none  #f)

(define (init)
  (set! dialog (gtk-dialog-new-with-buttons
		(c-make-window-title (_ "Delete"))
		app-widget '()))

  ;; explicitly create the ok button so we can move the focus to it,
  ;; otherwise by default gtk_dialog_map() chooses the first focusable in
  ;; `gtk-dialog-vbox' which means `notes-check'
  ;;
  ;; (note `gtk-dialog-get-response-for-widget' not wrapped in guile-gtk
  ;; 2.0, so can't find response after creation)
  ;;
  (let ((button (gtk-button-new-from-stock "gtk-ok")))
    (gtk-widget-show button)
    (gtk-dialog-add-action-widget dialog button gtk-response-ok)
    (gtk-widget-grab-focus
     (car (gtk-container-children (gtk-dialog-action-area dialog)))))

  (gtk-dialog-add-button dialog "gtk-cancel" gtk-response-cancel)

  (set! notes-check (gtk-check-button-new-with-label
		     (_ "And delete your annotations too")))
  (gtk-toggle-button-set-active notes-check #t)
  (gtk-box-pack-end (gtk-dialog-vbox dialog) notes-check)
  (gtk-widget-show notes-check)

  (set! notes-none (gtk-label-new
		    (_ "(No annotations, just downloaded data.)")))
  (gtk-misc-set-alignment notes-none 0 0.5)
  (gtk-box-pack-end (gtk-dialog-vbox dialog) notes-none)

  (c-gtk-dialog-hide-on-delete dialog)
  (gtk-signal-connect dialog "response"
    (lambda (id)
      (cond ((= id gtk-response-ok)
	     (gtk-widget-hide dialog)
	     ;; if looking at the deleted symbol then step to next
	     (let ((next (if (equal? del-symbol (smarker-entry
						 (view-smarker outer)))
			     chart-next
			     noop)))
	       (database-delete
		#:notes-too (gtk-toggle-button-get-active notes-check)
		del-symbol)
	       (symlist-list historical-symlist)
	       (next)))

	    ((= id gtk-response-cancel)
	     (gtk-widget-hide dialog)))))

  (set! label (gtk-label-new ""))
  (gtk-box-pack-start (gtk-dialog-vbox dialog) label #f #f
		      (c-gtk-widget-units->height label '(height . 1)))
  (gtk-widget-show label)

  ;; run this init once only, and gc the code
  (set! init noop))

(define-public (delete-dialog-popup)
  ;; File/Delete menu entry should be insensitive while no symbol, but check
  ;; just in case
  (and-let* ((symbol (view-symbol outer)))
    (init)
    (set! del-symbol symbol)
    (gtk-label-set-text label (format #f (_ "Delete symbol ~a ?") del-symbol))
    (if (null? (series-notes (database-read-series symbol)))
	(begin
	  (gtk-widget-show notes-none)
	  (gtk-widget-hide notes-check))
	(begin
	  (gtk-widget-show notes-check)
	  (gtk-widget-hide notes-none)))

    (c-gtk-widget-show-or-raise dialog)))
