;; Trace download costs differing from defaults.

;; Copyright 2007 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


;; Usage: Add to your ~/Chart/init.scm,
;;
;;     (load "/my/path/devel/cost-trace.scm")
;;
;; Diagnostics are printed when the various defaults values in the code for
;; to `download-cost-get' differ a lot from the values found from the actual
;; downloads.  (This is done during the downloads, so only those sources and
;; their files which are used are checked.)
;;


(define-module (chart cost-trace)
  #:autoload   (chart down-cost)  (download-cost-get)
  #:use-module (chart misc))

(after-module-load '(chart down-cost)
  (lambda ()
    (define old-download-cost-get download-cost-get)

    (set! download-cost-get
	  (lambda (key default)
	    (let ((got (old-download-cost-get key default)))
	      (if (not (within-range? (* 0.9 default) (* 1.1 default)
				      got))
		  (begin
		    (or (= 0 (port-column (current-output-port)))
			(newline))
		    (format #t "download-cost-get ~a: default ~a got ~a\n"
			    key default got)))
	      got)))))
