;; Trace of interactions between Emacs chart.el and Chart subprocess.

;; Copyright 2007 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


;; Usage: Add to your ~/Chart/init.scm,
;;
;;     (load "/my/path/devel/emacs-trace.scm")
;;
;; This is a hack to print traces of what commands are reaching the
;; emacs.scm module when it's talking to chart.el.  `tty-filename' below is
;; where to send the output.
;;

(define-module (chart emacs-trace)
  #:use-module (chart misc)
  #:autoload   (chart emacs) (emacs-write emacs-run-command))

(define tty-filename "/dev/tty7")



(define (file-can-open? filename)
  (catch #t
    (lambda ()
      (close-port (open-file tty-filename "w0")))
    (lambda args
      #f)))

(define tty-port
  (make-procedure-with-cache-0
   (lambda ()
     (open-file tty-filename "w0"))))

(define (trace-emacs-write proc args)
  (format (tty-port) "send: ~s\n" (cons proc args))
  ;; or this as a short form
  '(format (tty-port) "send: ~s\n" proc))

(define (trace-emacs-run-command form)
  (format (tty-port) "req: ~s\n" form))

(if (and (member "--emacs" (program-arguments))
	 (file-can-open? tty-filename))
    (begin
      (let ((old-emacs-run-command emacs-run-command))
	(set! emacs-run-command
	      (lambda (form)
		(trace-emacs-run-command form)
		(old-emacs-run-command form))))

      (let ((old-emacs-write emacs-write))
	(set! emacs-write
	      (lambda (proc . args)
		(trace-emacs-write proc args)
		(apply old-emacs-write proc args))))

      (let ((old-c-main c-main))
	(set! c-main
	      (lambda ()
		(set-current-output-port (tty-port))
		(set-current-error-port (tty-port))
		(format #t "tracing emacs interaction (subprocess ~a) ...\n" (getpid))
		(old-c-main))))))
