;; Trace any compression seen on http replies.

;; Copyright 2006, 2007 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


;; Usage: Add to your ~/Chart/init.scm,
;;
;;     (load "/my/path/devel/http-trace-compression.scm")
;;
;; Any gzip, deflate or compress responses from http servers are reported.
;; This is mainly for interest, just to see what servers actually use this
;; feature.
;;
;; `seen-hosts-list' is those hosts already reported, so as not to give a
;; message for every single repose.  It starts with the Bendigo Stock
;; Exchange pre-seeded since it's known to use compression (and hence no
;; interest in reporting it again).
;;


(define-module (chart http-trace-compression)
  #:use-module (ice-9 and-let-star)
  #:use-module (ice-9 receive)
  #:use-module (chart misc)
  #:autoload   (chart http)  (http-request))


(after-module-load '(chart http)
  (lambda ()
    (define seen-hosts-list '("www.bsx.com.au"))

    (define (http-trace-compression url headers)
      (and-let* ((encoding (http-headers-get headers "Content-Encoding")))
	(let ((host (http-url-host url)))
	  (or (member host seen-hosts-list)
	      (begin
		(set! seen-hosts-list (cons host seen-hosts-list))
		(or (= 0 (port-column (current-output-port)))
		    (newline))
		(format #t "~a from ~a\n" encoding host))))))

    (let ((old-http-request http-request))
      (set! http-request
	    (lambda (url . rest)
	      (receive (headers body)
		  (apply old-http-request url rest)
		(http-trace-compression url headers)
		(values headers body)))))))
