;; Lookout for http responses which include ETag or Last-Modified.

;; Copyright 2006, 2007 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


;; Usage: Add to your ~/Chart/init.scm,
;;
;;     (load "/my/path/devel/http-trace-etag.scm")
;;
;; This is some code to keep a lookout for servers which generate ETag or
;; Last-Modified headers, and which the data source code doesn't take
;; advantage of yet.  The list of exclusions below are things where tags are
;; already handled.
;;

(define-module (chart http-trace-etag)
  #:use-module (ice-9 receive)
  #:use-module (srfi srfi-1)
  #:use-module (srfi srfi-13)
  #:use-module (chart misc)
  #:autoload   (chart http)  (http-request))

(after-module-load '(chart http)
  (lambda ()
    (define known
      '("http://www.osamex.com/download/data"
	"http://www.lme.co.uk/downloads"
	"http://kanex.or.jp/13history"
	"http://kanex.or.jp/market/price"
	"http://www.tge.or.jp/data/down_load"
	"http://www.ase.gr/content/en/MarketData/Stocks/Prices/Files/txt"
	"http://www.c-com.or.jp/public_html/csv"
	"http://kanex.or.jp/MARKET"
	"http://www.sfe.com.au/Content/reports/EODWebMarketSummary"
	"http://www.sfe.com.au/content/graphs/YTDWebPriceGraph"
	"http://www.ffe.or.jp/home/market/DayData"
	"http://www.tocom.or.jp/data/yakujou/"
	"http://www.sicom.com.sg/publicItems/text"
	"http://ichart.finance.yahoo.com/"
	"http://www.wtb-hannover.de/bilder/charts"
	"http://exchanges.barchart.com/pl/mgex/data.js" ;; names
	))

    (define (trace-http-request-etag url headers)
      (if (not (any (lambda (prefix)
		      (string-prefix? prefix url))
		    known))

	  (let ((etag    (http-headers-get headers "ETag"))
		(lastmod (http-headers-get headers "Last-Modified")))
	    (if (or etag lastmod)
		(format #t "\nfrom ~a:\n" url))
	    (if etag
		(format #t "  ETag ~s\n" etag))
	    (if lastmod
		(format #t "  Last-Modified ~s\n" lastmod)))))

    (let ((old-http-request http-request))
      (set! http-request
	    (lambda (url . rest)
	      (receive (headers body)
		  (apply old-http-request url rest)
		(trace-http-request-etag url headers)
		(values headers body)))))))
