;; Trace http requests.

;; Copyright 2006, 2007 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


;; Usage: Add to your ~/Chart/init.scm,
;;
;;     (load "/my/path/devel/http-trace.scm")
;;
;; Print a trace of all http requests made.  The trace shows the url, and if
;; it's a POST then the data too, plus any If-None-Match and/or
;; If-Modified-Since headers.  This is meant to show what's being actually
;; used (especially to see an exact date or whatever that's resulting in
;; unexpected data).
;;


(define-module (chart http-trace)
  #:use-module (ice-9 and-let-star)
  #:use-module (srfi srfi-1)
  #:use-module (chart misc)
  #:autoload   (chart http)  (http-request))

(if (memq 'debug (debug-options)) ;; do it only under --debug

    (after-module-load '(chart http)
      (lambda ()

	(define (plist-get plist key)
	  (cond ((null? plist)
		 #f)
		((eq? key (first plist))
		 (second plist))
		(else
		 (plist-get (cddr plist) key))))

	(define (trace-http-request url rest)
	  (or (= 0 (port-column (current-output-port)))
	      (newline))
	  (let ((method (plist-get rest #:method)))
	    (format #t "~a ~a\n" url (or method "")))

	  ;; (and-let* ((proxied (http-proxy url)))
	  ;;   (format #t "  proxied through ~a\n" proxied))

	  (and-let* ((val (plist-get rest #:data)))
	    (format #t "  data ~s\n" val))
	  (and-let* ((val (plist-get rest #:if-none-match)))
	    (format #t "  if-none-match: ~s\n" val))
	  (and-let* ((val (plist-get rest #:if-modified-since)))
	    (format #t "  if-modified-since ~s\n" val)))

	(let ((old-http-request http-request))
	  (set! http-request
		(lambda (url . rest)
		  (trace-http-request url rest)
		  (apply old-http-request url rest)))))))
