#!/bin/sh
exec chart --debug -s "$0" "$@"
!#

;; Fetch and dump latest records.

;; Copyright 2006, 2007 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


;; Usage: ./lat.scm symbol...
;;
;; Download the latest quote for each given SYMBOL and print it in a raw
;; format, designed to show what `latest-get' is doing.  Downloads are done
;; with `latest-request-explicit', so it always queries the server (or
;; within trading hours at least), avoiding the 2-minute rule of
;; `latest-request-symbols'.
;;
;; The strings are printed as utf-8 irrespective of what charset the locale
;; says.  The commented out code below can put it through a transliteration
;; if desired.
;;


(use-modules (chart latest)
	     (chart misc))


;; Transliterate through the iconv program:
;;
;; (use-modules (ice-9 popen))
;; (set-current-output-port (open-output-pipe
;; 			  "iconv -f UTF-8 -t LATIN1//TRANSLIT"))
;; (set-current-error-port (current-output-port))


;; return list (HOUR MINUTE SECOND)
(define-public (seconds->hms time)
  (list (quotient time 3600)
	(quotient (remainder time 3600) 60)
	(remainder time 60)))

(define symbol-list (cdr (program-arguments)))

(c-main-enq! latest-request-explicit symbol-list)
(c-main)

(for-each (lambda (symbol)
	    (let ((latest (latest-get symbol)))
	      (if latest
		  (begin
		    (format #t "~a - ~a\n"
			    (latest-symbol latest)
			    (latest-name latest))
		    (format #t "  quote adate=~a ~a time=~a  ~a/~a\n"
			    (latest-quote-adate latest)
			    (and (latest-quote-adate latest)
				 (adate->ymd (latest-quote-adate latest)))
			    (and (latest-quote-time latest)
				 (seconds->hms (latest-quote-time latest)))
			    (latest-bid latest)
			    (latest-offer latest))
		    (format #t "  last ~a ~a ~a  ~a ~a-~a ~a ~a\n"
			    (latest-last-adate latest)
			    (and (latest-last-adate latest)
				 (adate->ymd (latest-last-adate latest)))
			    (and (latest-last-time latest)
				 (seconds->hms (latest-last-time latest)))
			    (latest-open latest)
			    (latest-high latest)
			    (latest-low latest)
			    (latest-last latest)
			    (latest-change latest))
		    (format #t "  decimals ~a   note ~a\n"
			    (latest-decimals latest)
			    (latest-note latest))
		    (format #t "  source ~a\n"
			    (latest-source latest))
		    (format #t "  timezone ~a  current ~a\n"
			    (symbol-timezone symbol)
			    (strftime-utf8 "%H:%M:%S %d-%b-%y"
					   (localtime (current-time)
						      (symbol-timezone symbol))))
		    (format #t "  fetched ~a  ~a local\n"
			    (latest-fetch-time latest)
			    (strftime "%H:%M:%S %d-%b-%Y"
				      (localtime (latest-fetch-time latest))))
		    (format #t "  etag ~s  last-modified ~s\n"
			    (latest-etag latest)
			    (latest-last-modified latest)))
		  (format #t "~a  not found\n" symbol))))
	  symbol-list)

(exit 0)


;; Local variables:
;; mode: scheme
;; End:
