#!/bin/sh
debug=
for i in "$@"; do
  case $i in
    --debug) debug=--debug; break ;;
    --) break ;;
  esac
done
exec s-chart $debug -s "$0" "$@"
!#

;; Print quotes.

;; Copyright 2005, 2006, 2007, 2008 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


;; Usage: ./myshare.scm [--debug] [symbol...]
;;
;; Print a quote line for each given symbol, or for the favourites list if
;; no symbols.  Curses colours show up and down changes in green and red, if
;; the terminal supports that.  The format is similar to the watchlist
;; inside Emacs.
;;


(define-module (myshare)
  #:use-module (ice-9 and-let-star)
  #:use-module (srfi srfi-1)
  #:use-module (srfi srfi-13)
  #:use-module (chart curses)
  #:use-module (chart latest)
  #:use-module (chart misc)
  #:use-module (chart symlists))


;; not enough room in 80 columns for thousands
(set! locale-thousands-sep (lambda () ""))

(define symbol-list (delete "--debug" (cdr (program-arguments))))


(if (null? symbol-list)
    (set! symbol-list (symlist-list favourites-symlist)))


(define (layout width-list str-list)
  (let ((target-right 0)
	(minimum-pad  0))
    (let more ((width-list width-list)
	       (str-list   str-list))
      (if (not (null? width-list))
	  (let ((width (first width-list))
		(str   (first str-list)))
	    (set! target-right (+ target-right (abs width)))
	    ;; track column but don't actually output if only whitespace
	    (if (string-every char-whitespace? str)
		(if (not (string-null? str))
		    (set! minimum-pad 1))
		(begin
		  ;; pad on the left for positives
		  (if (positive? width)
		      (let ((pad (max minimum-pad
				      (- target-right
					 (port-column (current-output-port))
					 (utf8-string-length str)))))
			(display (make-string pad #\space))))
		  (display (utf8->locale str))
		  (set! minimum-pad 0)))
	    (more (cdr width-list) (cdr str-list)))))))
    

(display "Symbol       bid/offer     last  change    low    high    volume    when\n")


(define (one symbol)
  (let ((latest (latest-get symbol)))
    (if latest
	(begin
	  (and-let* ((change (latest-change latest)))
	    (if (positive? change)
		(tput-colour 'green))
	    (if (negative? change)
		(tput-colour 'red)))

	  (set-port-column! (current-output-port) 0)
	  (let ((note (or (latest-note latest)
			  (latest-form-mdate latest))))
	    (layout `(9  ;; symbol
		      7  ;; bid
		      1  ;; slash
		      -7 ;; offer
		      1
		      7  ;; last
		      1
		      6  ;; change
		      1
		      7  ;; low
		      1
		      7  ;; high
		      1
		      8  ;; volume
		      1
		      7  ;; date/time
		      1
		      ,(utf8-string-length note))
		    (list
		     (string-pad-right symbol 9)
		     (latest-form-price latest (latest-bid latest))
		     (latest-form-quote-slash latest)
		     (latest-form-price latest (latest-offer latest))
		     " "
		     (latest-form-price latest (latest-last latest))
		     " "
		     (latest-form-price latest (latest-change latest))
		     " "
		     (latest-form-price latest (latest-low latest))
		     " "
		     (latest-form-price latest (latest-high latest))
		     " "
		     (latest-form-volume-km latest)
		     " "
		     (latest-form-date/time latest)
		     " "
		     note)))
	  (tput "sgr0")
	  (newline))
	(format #t "~a unknown\n" symbol))))


(define got-symbol-list '())

;; display new arrivals progressively
;;
(notify-connect 'latest-update
  (lambda (this-symbol-list)
    (set! this-symbol-list
	  (remove latest-in-progress? this-symbol-list))
    (set! got-symbol-list (append this-symbol-list got-symbol-list))
    (let more ()
      (if (not (null? symbol-list))
	  (let ((symbol (first symbol-list)))
	    (if (member symbol got-symbol-list)
		(begin
		  (set! symbol-list (cdr symbol-list))
		  (one symbol)
		  (more))))))))

(c-main-enq! latest-request-symbols symbol-list)
(c-main)

;; display any left (possibly no new data)
(for-each one symbol-list)

(exit 0)


;; Local variables:
;; mode: scheme
;; End:
