#!/bin/sh
exec chart -s "$0" "$@"
!#

;; Check symbols mentioned in the manual and example files.

;; Copyright 2007 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


;; Usage: chart -s online-symbols.scm [chart.texi]
;;
;; Symbols shown in the textin chart.texi have special identifying comments
;; "@c SYMBOL: FOO".  All the ones there are checked by downloading latest
;; quotes, with a view to ensuring the shares are still listed, or the
;; futures symbols are vaguely up-to-date.
;;


(use-modules (ice-9 and-let-star)
	     (ice-9 rdelim)
	     (ice-9 regex)
	     (srfi srfi-1)
	     (chart latest)
	     (chart misc))

(define good-count 0)


(define loc-list '())

(let ((filename "../doc/chart.texi"))
  (if (length>=? (program-arguments) 2)
      (set! filename (second (program-arguments))))

  (call-with-input-file filename
    (lambda (port)
      (do ((line (read-line port) (read-line port))
	   (lnum 1 (1+ lnum)))
	  ((eof-object? line))
	(and-let* ((m (regexp-exec #,(regexp "^@c +SYMBOL: +(.*)") line)))
	  (let* ((symbol (match:substring m 1))
		 (loc    (format #f "~a:~a: ~a" filename lnum symbol)))
	    (set! loc-list (acons symbol loc loc-list)))))))
  (set! loc-list (reverse! loc-list)))


(let ((symbol-list (delete-duplicates (map car loc-list))))
  (format #t "~a symbols in chart.texi\n" (length symbol-list))
  (c-main-enq! latest-request-symbols symbol-list))
(c-main)


(define limit (- (tdate-today) 5))

(define (loc-ok? symbol)
  (let ((latest (latest-get symbol)))
    (and (latest-last-tdate latest)
	 (>= (latest-last-tdate latest) limit))))

(for-each (lambda (elem)
	    (let ((symbol (car elem))
		  (loc    (cdr elem)))
	      (if (loc-ok? symbol)
		  (set! good-count (1+ good-count))

		  (let ((latest (latest-get symbol)))
		    (format #t "~a\n" loc)
		    (format #t "  last-adate ~a ~a\n"
			    (latest-last-adate latest)
			    (and (latest-last-adate latest)
				 (adate->ymd (latest-last-adate latest))))
		    (format #t "  source     ~a\n"
			    (latest-source latest))))))
	  loc-list)

(format #t "total ~a good\n" good-count)


;; Local variables:
;; mode: scheme
;; compile-command: "chart -s online-symbols.scm"
;; End:
