;; Download dialog.

;; Copyright 2003, 2004, 2005, 2006, 2007 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(define-module (chart down-dialog)
  #:use-module (srfi srfi-1)
  #:use-module (srfi srfi-13)
  #:use-module (gtk gtk)
  #:use-module (gtk gdk)
  #:use-module (chart download)
  #:use-module (chart misc)
  #:use-module (chart extra-gdk)
  #:use-module (chart extra-gtk)
  #:use-module (chart gui-common)
  #:use-module (chart main)
  #:use-module (chart symlists))


;;-----------------------------------------------------------------------------
;; download dialog

(define finished-one-action noop)
(define dialog              #f)
(define textbuf             #f)
(define textview            #f)
(define text-added          #f)
(define progress-title      #f)
(define progress-label      #f)
(define what-title          #f)
(define what-symbol         #f)
(define what-menu           #f)
(define what-option         #f)
(define what-entry          #f)
(define when-title          #f)
(define when-option         #f)
(define job                 #f)

(define (download-start-setups)
  (gtk-widget-set-sensitive when-title #f)
  (gtk-widget-set-sensitive when-option #f)
  (gtk-widget-set-sensitive what-title #f)
  (gtk-widget-set-sensitive what-option #f)
  (gtk-widget-set-sensitive what-entry #f)
  (gtk-text-buffer-set-text textbuf "")
  (gtk-widget-set-sensitive progress-title #t)
  (gtk-dialog-set-response-sensitive dialog 0 #f) ;; start
  (gtk-dialog-set-response-sensitive dialog 1 #t) ;; stop
  (gtk-dialog-set-response-sensitive dialog gtk-response-close #f)
  (gdk-window-set-cursor (gtk-widget-window (gtk-dialog-vbox dialog))
			 (c-gdk-cursor-watch)))

(define (download-stop)
  (gtk-widget-set-sensitive when-title #t)
  (gtk-widget-set-sensitive when-option #t)
  (gtk-widget-set-sensitive what-title #t)
  (gtk-widget-set-sensitive what-option #t)
  (gtk-widget-set-sensitive what-entry (eq? what-symbol
					    (gtk-menu-get-active what-menu)))
  (gtk-widget-set-sensitive progress-title #f)
  (gtk-label-set-text progress-label "")
  (gtk-dialog-set-response-sensitive dialog 0 #t) ;; start
  (gtk-dialog-set-response-sensitive dialog 1 #f) ;; stop
  (gtk-dialog-set-response-sensitive dialog gtk-response-close #t)
  (gdk-window-set-cursor (gtk-widget-window (gtk-dialog-vbox dialog)) #f)

  (if job
      (begin
	(job-stop job)
	(set! job #f))))

(define (download-start-job thunk)
  (download-start-setups)
  (set! job
	(job-start
	 (lambda ()
	   ;; wait for button sensitivity redraws, esp since the first
	   ;; download may load lots of new handler code
	   (c-idle (1+ gtk-priority-redraw))
	   (thunk)
	   (download-stop)))))

(define (init)
  (define accel-group (gtk-accel-group-new))
  (set! dialog (gtk-dialog-new-with-buttons
		(c-make-window-title (_ "Download")) #f '()))
  (gtk-window-add-accel-group dialog accel-group)

  ;; Stock "gtk-stop" is "_Stop" which conflicts with S for start, hence not
  ;; using that.
  (gtk-dialog-add-buttons dialog
			  (_ "_Start") 0
			  (_ "Sto_p")  1
			  "gtk-help"   gtk-response-help)
  (gtk-dialog-add-button dialog "gtk-close" gtk-response-close)

  (set! what-option (gtk-option-menu-new))
  (set! when-option (gtk-option-menu-new))

  (let ((what-favourites (c-gtk-menu-item-new-with-mnemonic
			  what-option accel-group (_ "_Favourites")))
	(what-all        (c-gtk-menu-item-new-with-mnemonic
			  what-option accel-group (_ "_All")))

	(when-menu       (gtk-menu-new))
	(when-backto     (c-gtk-menu-item-new-with-mnemonic
			  when-option accel-group (_ "_Back to")))
	(when-spin       (gtk-spin-button-new-with-range 1 9999 1)))

    (define (what-symbol-list)
      (let ((item (gtk-menu-get-active
		   (gtk-option-menu-get-menu what-option))))
	(cond ((eq? item what-symbol)
	       (list (gtk-entry-get-text what-entry)))
	      ((eq? item what-favourites)
	       ;; only favourites already in the database
	       (lset-intersection string=?
				  (symlist-list favourites-symlist)
				  (symlist-list all-symlist)))
	      ((eq? item what-all)
	       (symlist-list all-symlist)))))

    (c-gtk-dialog-hide-on-delete dialog)
    (gtk-signal-connect dialog "response"
      (lambda (id)
	(cond
	 ((= id 0) ;; start
	  (let ((symbols (what-symbol-list))
		(backto  (and (eq? when-backto (gtk-menu-get-active when-menu))
			      (year->tdate
			       (gtk-spin-button-get-value-as-int when-spin)))))
	    (download-start-job
	     (lambda ()
	       (download #:symbols symbols #:backto backto)))))

	 ((= id 1) ;; stop
	  (download-message "Stopped")
	  (download-stop))

	 ((or (= id gtk-response-close)
	      (= id gtk-response-delete-event))
	  (set! text-added #f)
	  (gtk-widget-hide dialog))

	 ((= id gtk-response-help)
	  (manual-popup (_ "CONTEXT(manual-node): Download") dialog)))))

    (gtk-dialog-set-response-sensitive dialog 1 #f) ;; stop

    (gtk-box-pack-start (gtk-dialog-vbox dialog)
			(gtk-label-new (_ "Download data"))
			#f #f 0)

    ;; what
    (set! what-menu (gtk-menu-new))
    (let ((hbox (gtk-hbox-new #f 4)))
      (gtk-box-pack-start (gtk-dialog-vbox dialog) hbox #f #f 0)
      (set! what-title (gtk-label-new (_ "What")))
      (gtk-box-pack-start hbox what-title #f #f 0)
      (gtk-box-pack-start hbox what-option #f #f 0)
      (set! what-entry (gtk-widget-new 'GtkEntry #:sensitive #f))
      (gtk-box-pack-start hbox what-entry  #f #f 0))

    ;; what-all
    (gtk-widget-show what-all)
    (gtk-menu-append what-menu what-all)
    (gtk-signal-connect what-all "activate"
      (lambda ()
	(gtk-widget-set-sensitive what-entry #f)))

    ;; what-favourites
    (gtk-widget-show what-favourites)
    (gtk-menu-append what-menu what-favourites)
    (gtk-signal-connect what-favourites "activate"
      (lambda ()
	(gtk-widget-set-sensitive what-entry #f)))

    ;; what-symbol
    (set! what-symbol (c-gtk-menu-item-new-with-mnemonic
		       what-option accel-group (_ "S_ymbol")))
    (gtk-widget-show what-symbol)
    (gtk-menu-append what-menu what-symbol)
    (gtk-signal-connect what-symbol "activate"
      (lambda ()
	(gtk-widget-set-sensitive what-entry #t)
	(gtk-widget-grab-focus what-entry)))

    (gtk-option-menu-set-menu what-option what-menu)
    (gtk-option-menu-set-history what-option 0)

    (gtk-signal-connect dialog "map_event"
      (lambda (event)
	(c-gtk-editable-select-region-noclip what-entry 0 -1)))

    ;; when
    (let ((hbox (gtk-hbox-new #f 4)))
      (gtk-box-pack-start (gtk-dialog-vbox dialog) hbox #f #f 0)
      (set! when-title (gtk-label-new (_ "When")))
      (gtk-box-pack-start hbox when-title #f #f 0)
      (gtk-box-pack-start hbox when-option #f #f 0)
      (gtk-box-pack-start hbox when-spin #f #f 0))

    (let ((item (c-gtk-menu-item-new-with-mnemonic
		 when-option accel-group "_Update")))
      (gtk-widget-show item)
      (gtk-menu-append when-menu item)
      (gtk-signal-connect item "activate"
	(lambda()
	  (gtk-widget-set-sensitive when-spin #f))))

    (gtk-widget-show when-backto)
    (gtk-menu-append when-menu when-backto)
    (gtk-signal-connect when-backto "activate"
      (lambda()
	(gtk-widget-set-sensitive when-spin #t)
	(gtk-widget-grab-focus when-spin)))

    (gtk-option-menu-set-menu when-option when-menu)
    (gtk-option-menu-set-history when-option 0)

    (c-gtk-widget-set-usize-units when-spin '(digit . 7) '(height . 1.5))
    (gtk-widget-set-sensitive when-spin #f)
    (gtk-spin-button-set-value when-spin (- (today-year) 10))

    (let* ((scrolled (gtk-scrolled-window-new)))
      (gtk-scrolled-window-set-policy scrolled 'never 'always)

      (set! textbuf (gtk-text-buffer-new #f))
      (set! textview (gtk-text-view-new-with-buffer textbuf))
      (c-gtk-widget-set-usize-units textview '(em . 60) '(height . 9))
      (gtk-text-view-set-editable textview #f)
      (gtk-text-view-set-wrap-mode textview 'word)
      (gtk-container-add scrolled textview)

      (gtk-box-pack-start (gtk-dialog-vbox dialog) scrolled #t #t 0))

    ;; progress
    (let ((progress-hbox (gtk-hbox-new #f 0)))
      (set! progress-title (gtk-widget-new 'GtkLabel
					   #:label (_ "Progress")
					   #:sensitive #f))
      (gtk-box-pack-start progress-hbox progress-title #f #f
			  (c-gtk-widget-units->width progress-title
						     '(em . 1)))

      (set! progress-label (gtk-widget-new 'GtkLabel #:xalign 0))
      (gtk-box-pack-start progress-hbox progress-label #t #t 0)

      (gtk-box-pack-start (gtk-dialog-vbox dialog) progress-hbox #f #f 0))

    ;; only once, and gc this code
    (set! init noop)))

 (set! download-status-output
       (lambda (str)
	 ;; don't display paused message after stopped
	 (if job
	     (begin
	       (gtk-label-set-text progress-label str)
	       ;; try to force output
	       (c-idle)))))

 (set! download-message
       (lambda (str)
	 (c-gtk-text-buffer-append textbuf (string-append str "\n"))
	 (c-gtk-text-view-scroll-to-end textview)
	 (set! text-added #t)))

(define-public (download-dialog-popup)
  (init)
  (set! finished-one-action noop)
  (c-gtk-widget-show-all-or-raise dialog))


(define-public (download-dialog-one-query symbol)
  (symbol-setups symbol)
  (let ((warning-list (filter-map (lambda (proc)
				    (proc (list symbol)))
				  download-now-list)))
    (if (null? warning-list)
	(download-dialog-one symbol)

	(download-now-query
	 symbol
	 (string-join (append warning-list
			      (list "Do you want to download now?"))
		      "\n\n")))))

(define-public (download-dialog-one symbol)
  (init)

  (if (not (download-update? symbol))
      (message "No further data for this symbol")

      (begin
	;; if not already mapped then hide when done, if already mapped then
	;; just raise the main view when done; in both cases only if no
	;; error messages were added, since they need to be left visible
	(cond ((not (gtk-widget-mapped dialog))
	       (set! finished-one-action (lambda ()
					   (if (not text-added)
					       (gtk-widget-hide dialog)))))
	      ((not (gtk-widget-mapped dialog))
	       (set! finished-one-action (lambda ()
					   (if (not text-added)
					       (c-gtk-widget-raise app-widget))))))
	(c-gtk-widget-show-all-or-raise dialog)

	(download-start-job
	 (lambda ()
	   (download #:symbols (list symbol))
	   (finished-one-action))))))


;;-----------------------------------------------------------------------------
;; download now query dialog
;;
;; created when required then destroyed

(define (download-now-query symbol message)
  (let ((dialog (gtk-message-dialog-new app-widget '() 'question 'none
					message)))
    (gtk-dialog-add-buttons dialog
			    "gtk-yes"  gtk-response-yes
			    "gtk-no"   gtk-response-no
			    "gtk-help" gtk-response-help)

    (gtk-signal-connect dialog "response"
      (lambda (id)
	(cond ((= id gtk-response-yes)
	       (gtk-widget-destroy dialog)
	       (download-dialog-one symbol))

	      ((= id gtk-response-no)
	       (gtk-widget-destroy dialog))

	      ((= id gtk-response-help)
	       (manual-popup (_ "CONTEXT(manual-node): Download") dialog)))))

    (gtk-widget-show-all dialog)))
