;; Elder bull power / bear power.

;; Copyright 2007 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(define-module (chart elder-power)
  #:use-module (ice-9 and-let-star)
  #:use-module (chart indicator)
  #:use-module (chart ema)
  #:use-module (chart misc))


;; http://www.incrediblecharts.com/technical/elder_ray_index.htm
;;     Formula, sample All Ords (yahoo ^AORD) from Aug 2001.
;;
;; http://www.prophet.net/analyze/popglossary.jsp?studyid=ELDR
;;     Formula, sample Hilton Hotels (HLT) from 2002/2003, weekly scale.
;;


(define elder-power-warmup-count ema-warmup-count)

(define (elder-power-fill series new-array lo hi)
  (let* ((parent   (series-parent series))
	 (array    (series-array parent lo hi))
	 (count    (series-data series))
	 (ema-proc (ema-calc-proc count)))

    ;; warmup
    (for-each ema-proc
	      (series-prev-n-closes parent
				    #:before lo
				    #:count  (elder-power-warmup-count count)))

    ;; points lo to hi inclusive
    (do ((i lo (1+ i)))
	((> i hi))
      (and-let* ((close (array-ref array i 3))
		 (ema   (ema-proc close)))
	(array-set! new-array (- (or (array-ref array i 1) ;; high
				     close)
				 ema)
		    i 0)
	(array-set! new-array (- (or (array-ref array i 2) ;; low
				     close)
				 ema)
		    i 1)))))

(define-public (elder-power-series series count)
  (ohlcv-required "Elder bull/bear power" series)
  (let ((new-series (series-copy series)))
    (series-name!      new-series (format #f "~a ~a" (_ "Elder Power") count))
    (series-datatype!  new-series elder-power-datatype)
    (series-parent!    new-series series)
    (series-data!      new-series count)
    (series-require!   new-series (indicator-require-parts
				   elder-power-fill
				   (elder-power-warmup-count count)
				   '(0 1)))
    (series-array!     new-series #f)
    (series-dividends! new-series '())
    (series-notes!     new-series '())
    new-series))

(define-public elder-power-datatype
  (datatype-new!
   #:key       'elder-power
   #:name      (_ "Elder Bull/Bear Power")
   #:input     datatype-ohlcv
   #:units     'momentum
   #:linestyle 'dmi
   #:hlines    '(0)
   #:manual    (_ "CONTEXT(manual-node): Elder Bull/Bear Power")
   #:proc      elder-power-series
   #:closes-apply-arg '(0)
   #:parameter-list
   (list (datatype-param-new
	  #:name    (_ "Days")
	  #:key     'elder-power-days
	  #:type    'integer
	  #:minimum 1
	  #:default 13))))
