;; Error dialog for Scheme throws etc.

;; Copyright 2002, 2003, 2004, 2005, 2006, 2007 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(define-module (chart error-dialog)
  #:use-module (gtk gtk)
  #:use-module (chart extra-gtk)
  #:use-module (chart gui-common)
  #:use-module (chart misc))


;; KEY and ARGS and STACK are a caught error
;; STACK can be #f when not run with debugging
;; Details of the error are printed to PORT
;;
(define (c-display-error stack key args port)
  (newline port)
  (or (and (= 4 (length args))
	   (false-if-exception
	    (begin
	      (apply display-error stack port args)
	      #t)))
      ;; if not otherwise displayable
      (format port "~a: ~a\n" key args))

  (cond (stack
	 (c-display-backtrace stack port))
	((memq 'debug (debug-options))
	 (display (_ "No backtrace available.\n") port))
	(else
	 (display (_ "No backtrace available (run with --debug to get one).\n")
		  port))))


(define dialog   #f)
(define textbuf  #f)
(define textview #f)

(define (init)
  (set! dialog (gtk-dialog-new-with-buttons
		(c-make-window-title (_ "Error Trace")) #f '()
		"gtk-close" gtk-response-close
		"gtk-help"  gtk-response-help))

  (c-gtk-dialog-hide-on-delete dialog)
  (gtk-signal-connect dialog "response"
    (lambda (id)
      (cond ((= id gtk-response-close)
	     (gtk-widget-hide dialog))

	    ((= id gtk-response-help)
	     (manual-popup (_ "CONTEXT(manual-node): Debugging") dialog)))))

  (set! textbuf (gtk-text-buffer-new #f))
  (set! textview (gtk-text-view-new-with-buffer textbuf))

  (let ((scrolled (gtk-scrolled-window-new)))
    (gtk-scrolled-window-set-policy scrolled 'never 'always)

    (c-gtk-widget-set-usize-units textview '(em . 70) '(height . 20))
    (gtk-text-view-set-editable textview #f)
    (gtk-text-view-set-wrap-mode textview 'char)

    (gtk-container-add scrolled textview)
    (gtk-widget-show-all scrolled)
    (gtk-box-pack-start (gtk-dialog-vbox dialog) scrolled #t #t 0))

  ;; once only, and gc this code
  (set! init noop))

(define-public (error-dialog-show stack key args)
  ;; to stdout
  (c-display-error stack key args (current-output-port))

  ;; and to dialog
  (init)
  (c-gtk-text-buffer-append textbuf
			    (call-with-output-string
			     (lambda (port)
			       (c-display-error stack key args port))))
  (c-gtk-text-view-scroll-to-end textview)
  (c-gtk-widget-show-or-raise dialog))
