;; Extra glib functions.

;; Copyright 2003, 2004, 2005, 2007 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(define-module (chart extra-glib)
  #:use-module (ice-9 and-let-star)
  #:use-module (srfi srfi-1)
  #:use-module (srfi srfi-13))


;;-----------------------------------------------------------------------------
;; glib miscellaneous utility functions

;; glib 2
(define-public g-dir-separator-s "/")

;; not wrapped
(define-public (g-get-home-dir)
  (let ((result (or (and-let* ((home (getenv "HOME"))
			       (     (not (string-null? home))))
		      home)
		    ;; use uid same as glib does, not euid
		    (and-let* ((pw  (false-if-exception (getpwuid (getuid))))
			       (dir (passwd:dir pw))
			       (    (not (string-null? dir))))
		      dir)
		    (error "chart: no HOME or passwd file home dir"))))
    (set! g-get-home-dir (lambda () result))
    result))

;; glib 2
(define-public (g-build-filename elem . more-elems)
  (apply g-build-path g-dir-separator-s elem more-elems))

;; glib 2
(define-public (g-build-path separator elem . more-elems)
  (string-join (remove string-null? (cons elem more-elems)) separator))

;; not wrapped
(define-public (g-get-tmp-dir)
  (let ((result (string-trim-right ;; no trailing "/"
		 (find (lambda (dir) ;; must exist and be a directory
			 (and dir
			      (not (string-null? dir))
			      (file-exists? dir)
			      (false-if-exception
			       (eq? 'directory (stat:type (stat dir))))))
		       (list (getenv "TMPDIR")
			     (getenv "TMP")
			     (getenv "TEMP")
			     "/tmp"
			     "c:\\"))
		 (string-ref g-dir-separator-s 0))))
    (set! g-get-tmp-dir (lambda () result))
    result))
