;; Hurst exponent shared functions.

;; Copyright 2007 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(define-module (chart hurst)
  #:use-module (ice-9 q)
  #:use-module (srfi srfi-1)
  #:use-module (chart config)
  #:use-module (chart extra-glib)
  #:use-module (chart misc))

(load-extension (g-build-filename chart-config-libchartdir "hurst.la")
		"chart_hurst_init")



;; http://www.bearcave.com/misl/misl_tech/wavelets/hurst/
;;     Good explanation of Hurst exponent calculation on financials etc by
;;     wavelets and R/S method.  Sample brown72.h (see devel/brown72.scm)
;;     calculation, and IBM closing prices 1970 to 2003.
;;
;; http://arxiv.org/pdf/cond-mat/0103621
;;     California electricity prices.  R/S adjusted for the white noise
;;     expected R/S of Anis and Lloyd (and Peters), called R/S-AL.  DFA
;;     description.  GPH periodogram.
;;
;; http://scitation.aip.org/getabs/servlet/GetabsServlet?prog=normal&id=CHAOEH000005000001000082000001&idtype=cvips&gifs=Yes
;;     Peng et al new DFA method.
;;

;; Fractal Dimensional Index:
;;
;; http://www.fractalfinancecubed.com/user/Making%20Sense%20of%20Fractals%20by%20Erik%20Long.pdf
;;     TASC May 2003 fractional dimensional index by Erik Long.  Intended
;;     Hurst calculation not precisely given.
;;
;; http://www.traders.com/Documentation/FEEDbk_docs/Archive/052003/TradersTips/TradersTips.html
;;     TASC Trader's Tips May 2003, code for fractal dimensional index by
;;     Erik Long.  AmiBroker 256 halved down to 8.  Wealth-Lab single R/S on
;;     200 bars.
;;
;; http://www.traders.com/Documentation/FEEDbk_docs/Archive/072003/TradersTips/TradersTips.html
;;     TASC Traders' Tips August 2003, tradestation code for single R/S pass
;;     with default N=30.
;;
;; http://traders.com/Documentation/FEEDbk_docs/Abstracts_new/Panini/panini.html
;;     Start of TASC March 2007 article.  Sample chart of IBM 1997-1999 with
;;     fractal dimensional index, but what size window etc parameters?
;;



;; Return the slope of a linear regression line fitted through a set of X,Y
;; points.  XY-LST is a list of pairs ((X1 . Y1) (X2 . Y2) ... (XN . YN)).
;;
(define-public (linreg-slope xy-lst)
  (let* ((x-lst  (map car xy-lst))
	 (x-mean (apply arithmetic-mean x-lst))
	 (y-lst  (map cdr xy-lst))
	 (y-mean (apply arithmetic-mean y-lst)))
    (set! x-lst (map! (lambda (x)
			(- x x-mean))
		      x-lst))
    (set! y-lst (map! (lambda (y)
			(- y y-mean))
		      y-lst))
    (let ((den (apply + (map square x-lst))))
      (if (zero? den)
	  0
	  (/ (apply + (map * x-lst y-lst))
	     den)))))

;; return a new queue consisting of the elements of LST
;; LST is incorporated into the queue and will be modified by subsequent
;; queue operations
(define-public (list->q! lst)
  (sync-q! (cons lst #f)))
