;; National Stock Exchange of India (NSE) setups.

;; Copyright 2005, 2006, 2007 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(define-module (chart india)
  #:use-module (srfi srfi-13)
  #:use-module (chart misc)
  #:use-module (chart yahoo))


(define-public (india-symbol? symbol)
  (string-suffix? ".NS" symbol))

(define-public timezone-calcutta
  (make-procedure-with-cache-0
   (lambda ()
     (timezone-choose (_ "Calcutta")
		      '("Asia/Calcutta"
			;; no daylight savings
			"IST-5:30")
		      "IST-5:30"))))

(symbol-timezone! india-symbol?  (_ "Mumbai") timezone-calcutta)
(yahoo-quote-delay! india-symbol? 15)


;;-----------------------------------------------------------------------------
;; web link - NSE company info
;;
;; Eg., and normally shown in a frame,
;;
;;     http://www.nseindia.com/marketinfo/companyinfo/eod/address.jsp?symbol=SESAGOA
;;

(weblink-handler! india-symbol?
  (lambda (symbol)
    (list (_ "NSE _Company Information")
	  (_ "Open web browser at the National Stock Exchange of India page for this company")
	  ;; there's links on this page to other info, including a current
	  ;; quote
	  (string-append "http://www.nseindia.com/marketinfo/companyinfo/eod/address.jsp?symbol="
			 (chart-symbol-sans-dot symbol)))))
