;; is this right?



;; Bandini.

;; Copyright 2007, 2008 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(define-module (chart bandini)
  #:use-module (ice-9 and-let-star)
  #:use-module (srfi srfi-1)
  #:use-module (chart indicator)
  #:use-module (chart misc)
  #:use-module (chart sma))



;; http://trader.online.pl/MSZ/e-w-Trending_Bandini.html
;;     Formula
;;
;; Asking that repeated SMA[2] smoothings remain above each other.


(define bandini-lines-count 10)
(define bandini-warmup-count 1000)

(define (bandini-fill series new-array lo hi)
  (let* ((parent      (series-parent series))
	 (array       (series-array-closes parent lo hi))
	 (proc-list   (map sma-calc-proc (make-list bandini-lines-count 2)))
	 (column-list (iota bandini-lines-count)))

    ;; warmup
    (for-each (lambda (close)
		(fold (lambda (proc val)
			(and val
			     (proc val)))
		      close proc-list))
	      (series-prev-n-closes
	       parent #:before lo
	       #:count bandini-warmup-count))

    ;; points lo to hi inclusive
    (do ((i lo (1+ i)))
	((> i hi))
      (and-let* ((val (array-ref array i)))
	(for-each (lambda (proc column)
		    (set! val (and val (proc val)))
		    (array-set! new-array val i column))
		  proc-list column-list)))))

(define-public (bandini-series series)
  (let ((new-series (series-copy series)))
    (series-name!     new-series (format #f "~a ~a"
					 (_ "Bandini") bandini-lines-count))
    (series-datatype! new-series bandini-datatype)
    (series-parent!   new-series series)
    (series-require!  new-series
		      (indicator-require-parts
		       bandini-fill
		       bandini-warmup-count
		       (list 0 (1- bandini-lines-count))))
    (series-array!    new-series #f)

    new-series))

(define-public bandini-datatype
  (datatype-new!
   #:key    'bandini
   #:name   (_ "Bandini")
   #:units  #f
   #:manual (_ "CONTEXT(manual-node): Bandini")
   #:proc   bandini-series
   #:closes-apply-arg '(0)))
