;; ???


;; Balance of power.

;; Copyright 2007 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(define-module (chart bop)
  #:use-module (chart indicator)
  #:use-module (chart misc))


;; http://www.linnsoft.com/tour/techind/bop.htm
;;     Formula, sample Adobe (ADBE) 2000-2001.
;;


(define-public (bop-calc open high low close)
  (let ((range (- (or high close) (or low close))))
    (if (zero? range)
	0
	(/ (- close (or open close))
	   range))))

(define (bop-fill series new-array lo hi)
  (let* ((parent (series-parent series))
	 (array  (series-array parent lo hi)))

    ;; points lo to hi inclusive
    (do ((i lo (1+ i)))
	((> i hi))
      (if (array-ref array i 3)
	  (array-set! new-array (bop-calc (array-ref array i 0)  ;; open
					  (array-ref array i 1)  ;; high
					  (array-ref array i 2)  ;; low
					  (array-ref array i 3)) ;; close
		      i)))))

(define-public (bop-series series)
  (ohlcv-required "Balance of Power" series)
  (let ((new-series (series-copy series)))
    (series-name!      new-series (_ "BOP"))
    (series-datatype!  new-series bop-datatype)
    (series-parent!    new-series series)
    (series-data!      new-series #f)
    (series-require!   new-series (indicator-require-parts bop-fill 0))
    (series-array!     new-series #f)
    (series-decimals!  new-series 0)
    (series-dividends! new-series '())
    (series-notes!     new-series '())
    new-series))

(define-public bop-datatype
  (datatype-new!
   #:key     'bop
   #:input   datatype-ohlcv
   #:units   'plus-or-minus-one
   #:minimum -1
   #:maximum 1
   #:hlines  '(0)
   #:manual  (_ "CONTEXT(manual-node): Balance of Power")
   #:proc    bop-series))
