;; Ehler's zany quadrature etc





;; Instantaneous trend line.

;; Copyright 2007, 2008 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(define-module (chart inst-trend)
  #:use-module (ice-9 and-let-star)
  #:use-module (chart indicator)
  #:use-module (chart midpoint)
  #:use-module (chart misc)
  #:use-module (chart momentum)
  #:use-module (chart sma)
  #:use-module (chart tma))


;; http://share.esignal.com/download.jsp?groupid=84&amp;folder=Stocks_Commodities&amp;file=InstantaneousTrendLine.efs
;;     Sample code.
;;


(define (inst-trend-calc-proc count)
  (let ((mom-proc  (momentum-calc-proc count))
	(sma-proc  (sma-calc-proc count))
	(tma4-proc (tma-calc-proc 4)))

    (lambda (x)
      (let ((sma (sma-proc x)))
	(and-let* ((slope        (mom-proc x))
		   (smooth-slope (tma4-proc slope))
		   (             sma))
	  (+ sma (* 0.5 smooth-slope)))))))

;; COUNT-1 for the sma
;; COUNT-1 + 4 for momentum+TMA = COUNT+3
;;
(define (inst-trend-warmup-count count)
  (+ 3 count))


(define (inst-trend-fill series new-array lo hi)
  (let* ((parent (series-parent series))
	 (count  (series-data series))
	 (warmup (inst-trend-warmup-count count))
	 (array  (series-array-closes parent lo hi))
	 (proc   (inst-trend-calc-proc count)))

    (for-each proc (series-prev-n-closes parent #:before lo
					 #:count warmup))

    ;; points lo to hi inclusive
    (do ((i lo (1+ i)))
	((> i hi))
      (if (array-ref array i)
	  (array-set! new-array (proc (array-ref array i)) i)))))

(define-public (inst-trend-series series count)
  (if (eq? datatype-ohlcv (series-datatype series))
      (set! series (midpoint-series series)))

  (let ((new-series (series-copy series)))
    (series-name!      new-series (format #f "~a ~a" (_ "Inst Trend") count))
    (series-datatype!  new-series inst-trend-datatype)
    (series-parent!    new-series series)
    (series-data!      new-series count)
    (series-require!   new-series (indicator-require-parts
				   inst-trend-fill
				   (inst-trend-warmup-count count)))
    (series-array!     new-series #f)

    new-series))

(define-public inst-trend-datatype
  (datatype-new!
   #:key     'inst-trend
   #:manual  (_ "CONTEXT(manual-node): Instantaneous Trend")
   #:proc    inst-trend-series
   #:parameter-list
   (list (datatype-param-new
	  #:name    (_ "Days")
	  #:key     'inst-trend-days
	  #:type    'integer
	  #:minimum 1
	  #:default 20))))
