;; up day is a ratio, but down day is plain TR ... ???



;; Range indicator by Jack Weinberg.

;; Copyright 2007 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.



;; http://www.paritech.com/education/technical/indicators/momentum/range.asp#chart
;;     Sample of Harvey Norman (symbol HVN.AX) from 2003/4.
;;
;; http://www.traderslog.com/range-indicator.htm
;;     Sample chart of Whole Foods Market (symbol WFMI) from Oct/Nov ?.
;;
;; http://www.fmlabs.com/reference/RangeIndicator.htm
;;     Formula.
;;
;; http://store.traders.com/-v13-c06-rangein-pdf.html
;;     Start of TASC article June 1995.
;;
;; http://www.amibroker.com/library/detail.php?id=126
;;     Code.
;;
;; http://www.tradesignalonline.com/wsn/print.asp?id=2044
;; http://translate.google.com/translate?hl=en&sl=de&u=http://www.tradesignalonline.com/wsn/print.asp%3Fid%3D2044&sa=X&oi=translate&resnum=1&ct=result&prev=/search%3Fq%3Dweinberg%2B%2522range%2Bindicator%2522%26start%3D10%26hl%3Den%26sa%3DN
;;     Description and formula in German, google translation comes out
;;     slightly strange.  Sample chart of Adidas (symbol ADSG.DE) Jul to Nov
;;     2004.
;;
;; http://ta-glossary.netfirms.com/taglossary-R.htm
;;     Description, sample BDY from 2004.


(define-module (chart range)
  #:use-module (ice-9 and-let-star)
  #:use-module (chart ema)
  #:use-module (chart indicator)
  #:use-module (chart misc)
  #:use-module (chart williams-r))


;; Return a procedure which calculates a Williams %R over a window of COUNT
;; many points.
;;
;; Each call (PROC high low close) enters a new point into the window, and
;; the return is the %R value up to (and including) that point, or #f if not
;; enough points yet.  CLOSE must be a number, but HIGH and/or LOW can be
;; #f, in which case the CLOSE is used for those.
;;
;; To prime the window initially, call PROC with COUNT-1 many points
;; preceding the first desired.
;;
(define-public (range-indicator-calc-proc range-count smooth-count)
  (define williams-proc (williams-%R-calc-proc range-count))
  (define ema-proc      (ema-calc-proc smooth-count))
  (define prev-proc     (prevnth-calc-proc 1))

  (lambda (high low close)
    (set! high (or high close))
    (set! low  (or low  close))

    (and-let* ((prev-close (prev-proc close))
	       (tr         (and prev-close
				(- (max high prev-close)
				   (min low  prev-close))))
	       (ratio      (if (> close prev-close)
			       (/ tr
				  (abs (- close prev-close)))
			       tr))
	       (wil        (williams-proc #f #f ratio)))
      (ema-proc (+ 100 wil)))))

(define-public (range-indicator-warmup-count range-count smooth-count)
  (+ range-count
     (ema-warmup-count smooth-count)))


(define (range-indicator-fill series new-array lo hi)
  (receive-list (range-count smooth-count)
      (series-data series)
    (let* ((parent (series-parent series))
	   (start  (series-prev-n-closes-pos
		    parent #:before lo
		    #:count (range-indicator-warmup-count range-count
							  smooth-count)))
	   (array  (series-array parent start hi))
	   (proc   (range-indicator-calc-proc range-count smooth-count)))

      ;; warmup start (inclusive) to lo (exclusive)
      (do ((i start (1+ i)))
	  ((>= i lo))
	(if (array-ref array i 3)
	    (proc (array-ref array i 1)    ;; high
		  (array-ref array i 2)    ;; low
		  (array-ref array i 3)))) ;; close

      ;; points lo to hi inclusive
      (do ((i lo (1+ i)))
	  ((> i hi))
	(if (array-ref array i 3)
	    (array-set! new-array (proc (array-ref array i 1)  ;; high
					(array-ref array i 2)  ;; low
					(array-ref array i 3)) ;; close
			i))))))

(define-public (range-indicator-series series range-count smooth-count)
  (ohlcv-required "Range Indicator" series)
  (let ((new-series (series-copy series)))
    (series-name!     new-series (format #f (_ "Range Ind ~a, ~a")
					 range-count smooth-count))
    (series-datatype!  new-series range-indicator-datatype)
    (series-parent!    new-series series)
    (series-data!      new-series (list range-count smooth-count))
    (series-require!   new-series
		       (indicator-require-parts
			range-indicator-fill
			(range-indicator-warmup-count range-count
						      smooth-count)))
    (series-array!     new-series #f)
    (series-decimals!  new-series 0)
    (series-dividends! new-series '())
    (series-notes!     new-series '())
    new-series))

(define-public range-indicator-datatype
  (datatype-new!
   #:key     'range-indicator
   #:name    (_ "Range Indicator")
   #:units   'percentage
   #:input   datatype-ohlcv
   #:hlines  '(20 70)
   #:minimum 0
   #:maximum 100
   #:manual  (_ "CONTEXT(manual-node): Range Indicator")
   #:proc    range-indicator-series
   #:parameter-list
   (list (datatype-param-new
	  #:key     'range-indicator-range-days
	  #:name    (_ "Range Days")
	  #:type    'integer
	  #:minimum 1
	  #:default 10)
	 (datatype-param-new
	  #:key     'range-indicator-smooth-days
	  #:name    (_ "Smooth Days")
	  #:type    'integer
	  #:minimum 1
	  #:default 3))))
