;; Kirshenbaum bands.

;; Copyright 2007 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(define-module (chart kirshenbaum)
  #:use-module (ice-9 and-let-star)
  #:use-module (chart ema)
  #:use-module (chart indicator)
  #:use-module (chart linreg-stderr)
  #:use-module (chart misc))



(define-public (kirshenbaum-warmup count)
  (max (ema-warmup-count count)
       (1- count)))

(define (kirshenbaum-fill series new-array lo hi)
  (receive-list (count stderr-factor)
      (series-data series)
    (let* ((parent      (series-parent series))
	   (warmup      (kirshenbaum-warmup count))
	   (start       (series-prev-n-closes-pos parent
						  #:before lo #:count warmup))
	   (array       (series-array-closes parent start hi))
	   (ema-proc    (ema-calc-proc count))
	   (stderr-proc (linreg-stderr-calc-proc count)))

      ;; warmup
      (do ((i start (1+ i)))
	  ((>= i lo))
	(and-let* ((close (array-ref array i)))
	  (ema-proc close)
	  (stderr-proc close)))

      ;; points lo to hi inclusive
      (do ((i lo (1+ i)))
	  ((> i hi))
	(and-let* ((close (array-ref array i)))
	  (let ((ema    (ema-proc close))
		(stderr (stderr-proc close)))
	    (if ema
		(begin
		  (array-set! new-array ema                  i 1)
		  (if stderr
		      (begin
			(set! stderr (* stderr-factor stderr))
			(array-set! new-array (- ema stderr) i 0)
			(array-set! new-array (+ ema stderr) i 2)))))))))))

(define-public (kirshenbaum-series series count stderr-factor)
  (let ((new-series (series-copy series)))
    (series-name!     new-series (format #f (_ "Kirshenbaum ~a, ~a")
					 count (locale-number-min-decimals
						stderr-factor 1)))
    (series-datatype! new-series kirshenbaum-datatype)
    (series-parent!   new-series series)
    (series-data!     new-series (list count stderr-factor))
    (series-require!  new-series (indicator-require-parts kirshenbaum-fill
							  (1- count)
							  '(0 2)))
    (series-array!    new-series #f)

    new-series))

(define-public kirshenbaum-datatype
  (datatype-new!
   #:key       'kirshenbaum
   #:name      (_ "Kirshenbaum Bands")
   #:units     #f
   #:linestyle 'bands
   #:manual    (_ "CONTEXT(manual-node): Kirshenbaum Bands")
   #:proc      kirshenbaum-series
   #:closes-apply-arg '(1)  ;; middle MA line
   #:parameter-list
   (list (datatype-param-new
	  #:name     (_ "Days")
	  #:key      'kirshenbaum-days
	  #:type     'integer
	  #:default  20
	  #:minimum  1)
	 (datatype-param-new
	  #:name     (_ "Stderrs")
	  #:key      'kirshenbaum-stderrs
	  #:type     'float
	  #:decimals 2
	  #:default  1.75
	  #:step     0.25
	  #:minimum  0
	  #:maximum  9))))
