;; Korea Stock Exchange and KOSDAQ setups.

;; Copyright 2005, 2006, 2007 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(define-module (chart korea)
  #:use-module (srfi srfi-13)
  #:use-module (chart download)
  #:use-module (chart misc)
  #:use-module (chart yahoo))


(define (korea-symbol? symbol)
  (string-suffix? ".KS" symbol))

(define (kosdaq-symbol? symbol)
  (string-suffix? ".KQ" symbol))

;; ^KS11
;;
;; avoid clash with ^KSE in karachi.scm
;;
(define (yahoo-index-symbol-korea? symbol) 
  (string-prefix? "^KS1" symbol))

(define-public timezone-seoul
  (make-procedure-with-cache-0
   (lambda ()
     (timezone-choose (_ "Seoul")
		      '("Asia/Seoul"
			;; no daylight savings post 1988
			"KST-9")
		      "KST-9"))))

(symbol-timezone! korea-symbol?             (_ "Seoul") timezone-seoul)
(symbol-timezone! kosdaq-symbol?            (_ "Seoul") timezone-seoul)
(symbol-timezone! yahoo-index-symbol-korea? (_ "Seoul") timezone-seoul)
(yahoo-quote-delay! korea-symbol? 20)
(yahoo-quote-delay! kosdaq-symbol? 20)


;;-----------------------------------------------------------------------------
;; web link - KSE
;;
;; Only the home page for now.  The symbol lookup at the web site doesn't
;; seem to work as of March 2007.
;;
(weblink-handler! (list korea-symbol? yahoo-index-symbol-korea?)
  (lambda (symbol)
    (list (_ "_Korea Stock Exchange Home Page")
	  (_ "Open web browser at the Korea Stock Exchange home page")
	  (lang-select '(("ko" "http://www.kse.or.kr/index.html")
			 ("en" "http://eng.krx.co.kr/index.html"))))))


;;-----------------------------------------------------------------------------
;; web link - KOSDAQ company info
;;
;; Full link is
;;     http://english.kosdaq.com/enterprise/center/center.jsp?code=052300&codeInFocus=052300&selFocus=enterprise%2Fcenter%2Fcenter.jsp&x=0&y=0
;;
;; but it's enough to give
;;     http://english.kosdaq.com/enterprise/center/center.jsp?code=052300
;;

(weblink-handler! kosdaq-symbol?
  (lambda (symbol)
    (list (_ "KOSDAQ _Company Information")
	  (_ "Open web browser at the KOSDAQ Exchange page for this company")
	  (string-append "http://"
			 (lang-select '(("ko" "www")
					("en" "english")))
			 ".kosdaq.com/enterprise/center/center.jsp?code="
			 (chart-symbol-sans-dot symbol)))))
