;; MACD momentum advance/decline.

;; Copyright 2004, 2005, 2006, 2007 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(define-module (chart macd)
  #:use-module (ice-9 and-let-star)
  #:use-module (chart indicator)
  #:use-module (chart ema)
  #:use-module (chart misc))


(define (macd-fill series new-array lo hi)
  (receive-list (fast-count slow-count smooth-count histogram)
      (series-data series)
    (let* ((parent      (series-parent series))
	   (warmup      (+ (ema-warmup-count (max fast-count slow-count))
			   (ema-warmup-count smooth-count)))
	   (array       (series-array-closes parent lo hi))
	   (fast-proc   (ema-calc-proc fast-count))
	   (slow-proc   (ema-calc-proc slow-count))
	   (smooth-proc (ema-calc-proc smooth-count)))

      ;; warmup
      (for-each (lambda (this)
		  (let* ((fast (fast-proc this))
			 (slow (slow-proc this))
			 (macd (and fast slow (- fast slow))))
		    (and macd (smooth-proc macd))))
		(series-prev-n-closes parent #:before lo #:count warmup))

      ;; points lo to hi inclusive
      (do ((i lo (1+ i)))
	  ((> i hi))
	(and-let* ((this (array-ref array i)))
	  (let* ((fast   (fast-proc this))
		 (slow   (slow-proc this))
		 (macd   (and fast slow (- fast slow)))
		 (smooth (and macd (smooth-proc macd))))
	    (array-set! new-array macd i 0)
	    (array-set! new-array smooth i 1)
	    (if histogram
		(array-set! new-array (and macd smooth (- macd smooth))
			    i 2))))))))

(define-public (macd-series series fast-count slow-count smooth-count histogram)
  (let ((new-series (series-copy series)))
    (series-name!      new-series (format #f (_ "MACD ~a,~a,~a")
					  fast-count slow-count smooth-count))
    (series-datatype!  new-series macd-datatype)
    (series-parent!    new-series series)
    (series-data!      new-series (list fast-count slow-count smooth-count
					histogram))
    (series-require!   new-series
		       (indicator-require-parts
			macd-fill
			(+ (ema-warmup-count slow-count)
			   (ema-warmup-count smooth-count))
			(if histogram
			    '(0 2)
			    '(0 1))))
    (series-array!     new-series #f)
    (series-dividends! new-series '())
    (series-notes!     new-series '())

    new-series))


(define-public macd-datatype
  (datatype-new!
   #:key       'macd
   #:name      (_ "MACD")
   #:units     'macd
   #:linestyle 'macd
   #:hlines    '(0)
   #:manual    (_ "CONTEXT(manual-node): MACD")
   #:proc      macd-series
   #:closes-apply-arg '(0)
   #:parameter-list
   (list (datatype-param-new
	  #:name    (_ "Fast days")
	  #:key     'macd-fast-days
	  #:type    'integer
	  #:default 12
	  #:minimum 1)
	 (datatype-param-new
	  #:name    (_ "Slow days")
	  #:key     'macd-slow-days
	  #:default 26
	  #:type    'integer
	  #:minimum 1)
	 (datatype-param-new
	  #:name    (_ "Smooth")
	  #:key     'macd-smooth-days
	  #:default 9
	  #:type    'integer
	  #:minimum 1)
	 (datatype-param-new
	  #:name    (_ "Histogram")
	  #:key     'macd-histogram
	  #:type    'boolean
	  #:default #t))))
