;; Midpoint of high/low in daily data.

;; Copyright 2006, 2007, 2009 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(define-module (chart midpoint)
  #:use-module (ice-9 and-let-star)
  #:use-module (chart indicator)
  #:use-module (chart misc))


(define (midpoint-fill series new-array lo hi)
  (let* ((parent (series-parent series))
	 (array  (series-array parent lo hi)))

    ;; points lo to hi inclusive
    (do ((i lo (1+ i)))
	((> i hi))
      (and-let* ((close (array-ref array i 3)))
	(array-set! new-array (arithmetic-mean
			       (or (array-ref array i 1) close)  ;; high
			       (or (array-ref array i 2) close)) ;; low
		    i)))))

(define-public (midpoint-series series)
  (if (eq? datatype-ohlcv (series-datatype series))
      (let ((new-series (series-copy series)))
	(series-name!     new-series (_ "Midpoint"))
	(series-datatype! new-series midpoint-datatype)
	(series-parent!   new-series series)
	(series-require!  new-series (indicator-require-parts midpoint-fill 0))
	(series-array!    new-series #f)
	new-series)

      series)) ;; not ohlcv, unchanged

(define-public midpoint-datatype
  (datatype-new! #:key      'midpoint
		 #:name     (_ "Midpoint (H+L)/2")
		 #:priority -1000
		 #:proc     midpoint-series))
