;; Negative Volume Indicator (NVI).

;; Copyright 2006, 2007, 2009 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(define-module (chart neg-volume)
  #:use-module (chart indicator)
  #:use-module (chart misc))


;; http://www.incrediblecharts.com/technical/negative_volume_index.htm
;; http://www.incrediblecharts.com/technical/positive_volume_index.htm
;; http://www.incrediblecharts.com/technical/nvi_construction.htm
;;
;; http://www.fosback.com/
;;     Norman Fosback's web site (nothing on NVI).
;;


;;-----------------------------------------------------------------------------
;; NVI

(define-public (negative-volume-func array prev this)
  (if (and (< (or (array-ref array this 4) 0)  ;; volume
	      (or (array-ref array prev 4) 0)) ;; prev volume
	   (not (zero? (array-ref array prev 3))))
      (/ (array-ref array this 3)  ;; close
	 (array-ref array prev 3)) ;; prev close
      1))

(define-public (negative-volume-series series)
  (accumulation-series series (_ "Neg Vol")
		       negative-volume-datatype negative-volume-func
		       #:multiplicative 100))

(define-public negative-volume-datatype
  (datatype-new!
   #:key    'negative-volume
   #:name   (_ "NVI - Negative Volume Index")
   #:units  'negative-volume
   #:manual (_ "CONTEXT(manual-node): Negative Volume Index")
   #:proc   negative-volume-series))


;;-----------------------------------------------------------------------------
;; PVI

(define-public (positive-volume-func array prev this)
  (if (and (> (or (array-ref array this 4) 0)  ;; volume
	      (or (array-ref array prev 4) 0)) ;; prev volume
	   (not (zero? (array-ref array prev 3))))
      (/ (array-ref array this 3)  ;; close
	 (array-ref array prev 3)) ;; prev close
      1))


(define-public (positive-volume-series series)
  (accumulation-series series (_ "Pos Vol")
		       positive-volume-datatype positive-volume-func
		       #:multiplicative 100))

(define-public positive-volume-datatype
  (datatype-new!
   #:key    'positive-volume
   #:name   (_ "PVI - Positive Volume Index")
   #:units  'positive-volume
   #:manual (_ "CONTEXT(manual-node): Negative Volume Index")
   #:proc   positive-volume-series))
