;; New York Board of Trade (NYBOT) setups.

;; Copyright 2005, 2006, 2007 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(define-module (chart nybot)
  #:use-module (srfi srfi-13)
  #:use-module (chart barchart)
  #:use-module (chart download)
  #:use-module (chart misc)
  #:use-module (chart yahoo))


(define-public (nybot-symbol? symbol)
  (string-suffix? ".NYBOT" symbol))

(symbol-timezone! nybot-symbol? (_ "New York") timezone-newyork)


;;-----------------------------------------------------------------------------
;; yahoo for quotes

(set! yahoo-suffix-transform-alist
      (cons '(".NYBOT" . ".NYB")
	    yahoo-suffix-transform-alist))
(set! yahoo-futures-symbol-predlist
      (cons nybot-symbol?
	    yahoo-futures-symbol-predlist))
(yahoo-quote-delay! nybot-symbol? 30)


;;-----------------------------------------------------------------------------
;; barchart for database and for intraday graphs

(set! barchart-download-symbol-predlist
      (cons nybot-symbol? barchart-download-symbol-predlist))

(set! barchart-suffix-delay-alist (acons ".NYBOT" 30
					 barchart-suffix-delay-alist))

(set! barchart-commodity-mung-alist
      (acons nybot-symbol?
	     '(;; "CC"       ;; COCOA
	       ;; "CT"       ;; COTTON NO. 2
	       ;; "MK"       ;; MINI COFFEE C
	       ;; "XA"       ;; ETHANOL
	       ;; "SE"       ;; SUGAR 14
	       ;; "OJ"       ;; FCOJ
	       ;; "OD"       ;; FCOJ DIFF: PAR=100
	       ("OB" . #f)   ;; FCOJ-B -- seems to be missing from barchart
	       ("P" . "PU")  ;; PULP
	       ;; "KC"       ;; COFFEE "C"
	       ;; "SB"       ;; SUGAR 11

	       ;; "AR"       ;; Aus. Dollar - N.Z. Dollar
	       ;; "AS"       ;; Australian Dollar - Canadian $
	       ;; "YA"       ;; Australian Dollar - Jap.Yen
	       ("MP" . "OG") ;; Small British Pd. - U.S. Dollar
	       ;; "SS"       ;; British Pd. - Swiss Franc
	       ;; "SY"       ;; British Pd. - Jap. Yen
	       ;; "HY"       ;; Canadian Dollar - Jap. Yen
	       ;; "EO"       ;; Euro - U.S. Dollar Regular (barchart "SMALL")
	       ;; "EP"       ;; Euro - Canadian Dollar
	       ;; "GB"       ;; Euro - Sterling
	       ;; "EJ"       ;; Euro - Jap. Yen
	       ;; "OL"       ;; Euro - Norw. Krone
	       ;; "RK"       ;; Euro - Swed. Krona
	       ("RA" . "EQ") ;; Euro - Australian Dollar
	       ;; "RZ"       ;; Euro - Swiss Franc
	       ;; "EU"       ;; Euro - U.S.Dollar Large
	       ("HR" . "GY") ;; Euro - Hungarian forint
	       ("EZ" . "GZ") ;; Euro - Czech koruna
	       ;; "ZY"       ;; Swiss Franc-Jap. Yen
	       ;; "DX"       ;; U.S. Dollar Index
	       ;; "AU"       ;; Aus. Dollar - U.S. Dollar
	       ;; "KU"       ;; U.S. Dollar - Swed. Krona
	       ;; "NS"       ;; U.S. Dollar - Norw. Krone
	       ;; "YF"       ;; U.S. Dollar - Swiss Franc
	       ("MF" . "OF") ;; Small U.S. Dollar - Swiss Franc
	       ;; "YP"       ;; U.S. Dollar - British Pound
	       ;; "YY"       ;; U.S. Dollar - Japanese Yen
	       ;; "SN"       ;; Small U.S. Dollar - Japanese Yen
	       ("ZR" . "OR") ;; U.S. Dollar - S.African Rand
	       ;; "ZX"       ;; U.S. Dollar - New Z Dollar
	       ;; "YD"       ;; U.S. Dollar - Canadian Dollar
	       ("SV" . "OV") ;; Small U.S. Dollar - Canadian Dollar
	       ("UF" . "UY") ;; U.S. Dollar - Hungarian forint
	       ;; "UZ"       ;; U.S. Dollar - Czech koruna
	       ("NJ" . "EK") ;; Norwegian Krone / Swedish krona

	       ;; "CR"       ;; Reuters CRB Index
	       ("R" . "RX")  ;; Russell 1000
	       ("RM" . "RJ") ;; Russell 1000 Mini
	       ("GG" . "VU") ;; Russell 1000 Growth Index
	       ;; "VV"       ;; Russell 1000 Value Index
	       ;; "TO"       ;; Russell 2000
	       ("GH" . "RG") ;; Russell 2000 Growth Index
	       ("VB" . "RV") ;; Russell 2000 Value Index
	       ;; "TH"       ;; Russell 3000
	       ("YU" . "YV") ;; Revised NYSE Comp. Index
	       ;; "MU"       ;; Revised NYSE Small Comp. Index
	       )
	     barchart-commodity-mung-alist))
