;; New York Mercantile Exchange (NYMEX+COMEX) setups.

;; Copyright 2006, 2007 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(define-module (chart nymex)
  #:use-module (srfi srfi-13)
  #:use-module (chart barchart)
  #:use-module (chart download)
  #:use-module (chart misc)
  #:use-module (chart yahoo))


(define-public (comex-symbol? symbol)
  (string-suffix? ".COMEX" symbol))

(define-public (nymex-symbol? symbol)
  (string-suffix? ".NYMEX" symbol))

(symbol-timezone! comex-symbol? (_ "New York")  timezone-newyork)
(symbol-timezone! nymex-symbol? (_ "New York")  timezone-newyork)


;;-----------------------------------------------------------------------------
;; yahoo for quotes

(set! yahoo-suffix-transform-alist
      (cons* '(".COMEX" . ".CMX")
	     '(".NYMEX" . ".NYM")
	     yahoo-suffix-transform-alist))
(set! yahoo-futures-symbol-predlist
      (cons* comex-symbol?
	     nymex-symbol?
	     yahoo-futures-symbol-predlist))

(yahoo-quote-delay! comex-symbol? 30)
(yahoo-quote-delay! nymex-symbol? 30)


;;-----------------------------------------------------------------------------
;; barchart for database and for intraday graphs

(set! barchart-download-symbol-predlist
      (cons* nymex-symbol?
	     comex-symbol?
	     barchart-download-symbol-predlist))

(set! barchart-suffix-delay-alist (cons* '(".COMEX" . 30)
					 '(".NYMEX" . 30)
					 barchart-suffix-delay-alist))

;; barchart-commodity-mung-alist
;;
;; COMEX - no transforms
;; "HG" high grade copper
;; "GC" gold
;; "SI" silver
;; "AL" aluminium
;;
;; NYMEX - no transformations
;; "CL" light sweet crude
;; "HU" gasoline
;; "HO" heating oil
;; "NG" henry hub natural gas
;; "PA" palladium
;; "PL" platinum
;; "PN" propane
;; "QG" e-miNY henry hub natural gas
;; "QL" CAPP central appalacian coal
;; "QM" e-miNY light sweet crude
;; "SC" brent crude - dublin
;;
;; "GR" north west gasoil - dublin
;; "SA" brent calendar swap futures - clearport new york
;;
;; don't know what barchart "F0" heating oil / crude and "F5"
;; unleaded / crude are really


;;-----------------------------------------------------------------------------
;; This was some bits to make the timezone come out as London (and
;; previously Dublin) for some relevant symbols.  But www.nymex.com doesn't
;; make it clear (as of March 2007) what's where, so leave it all as New
;; York for now.

;; NYMEX symbols trading in London
;;
;; http://www.nymex.com/press_releas.aspx?id=pr20050825c
;;     Aug 05 press release - brent crude + northwest gasoil
;;
;; (define (nymex-london-symbol? symbol)
;;   (and (nymex-symbol? symbol)
;;        (member (chart-symbol-commodity symbol)
;; 	       '("GR"  ;; gasoil
;; 		 "BB"  ;; brent crude
;; 		 ))))
;; (symbol-timezone! nymex-london-symbol?  (_ "London")    timezone-london)

;; Contracts trading open-outcry in London (currently Brent Crude @samp{SC} and
;; Northwest Europe Gasoil @samp{GR}) have that as their home timezone for the
;; watchlist etc, everything else is New York.
;; @c xSYMBOL: SC.NYMEX
;; @c xSYMBOL: GR.NYMEX


;;-----------------------------------------------------------------------------
;; web link - contract specs

(weblink-handler! (list comex-symbol? nymex-symbol?)
  (lambda (symbol)
    (list (_ "NYMEX _Contract Specifications")
	  (_ "Open web browser at the NYMEX/COMEX contract specifications for this commodity")
	  (string-append "http://www.nymex.com/"
			 (chart-symbol-commodity symbol)
			 "_spec.aspx"))))
