;; Open interest series.

;; Copyright 2005, 2006, 2007 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(define-module (chart open-interest)
  #:use-module (chart indicator)
  #:use-module (chart misc)
  #:use-module (chart volume))


(define (open-interest-require series lo hi)
  (make-shared-array-column (series-array (series-parent series) lo hi) 5))

(define (open-interest-series series)
  (ohlcv-required "Open Interest" series)

  (let ((new-series (series-copy series)))
    (series-parent!    new-series series)
    (series-datatype!  new-series open-interest-datatype)
    (series-name!      new-series (_ "Open interest"))
    (series-decimals!  new-series 0)
    (series-dividends! new-series '())
    (series-notes!     new-series '())

    (if (ohlcv-open-interest? series)
	(begin
	  (series-array!   new-series #f)
	  (series-require! new-series open-interest-require))

	(begin
	  (series-lo! new-series (series-lo series))
	  (series-hi! new-series (series-lo series))
	  (series-array! new-series
			 (make-array #f (list (series-lo series)
					      (series-lo series))))
	  (series-require! new-series #f)))

    new-series))

(define-public open-interest-datatype
  (datatype-new!
   #:key       'open-interest
   #:units     'open-interest
   #:name      (_ "Open Interest")
   #:input     datatype-ohlcv
   #:linestyle 'bar
   #:manual    (_ "CONTEXT(manual-node): Volume and Open Interest")
   #:proc      open-interest-series
   #:initial-page-proc volume-initial-page))
