;; Points drawing.

;; Copyright 2003, 2004, 2005, 2006, 2007 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(define-module (chart points)
  #:use-module (ice-9 and-let-star)
  #:use-module (gtk gdk)
  #:use-module (gtk gtk)
  #:use-module (chart extra-gdk)
  #:use-module (chart graph)
  #:use-module (chart misc)
  #:use-module (chart view))


;; convert an X pixel distance to a Y pixel distance, according to the
;; aspect ratio of the screen
;;
(define c-gdk-x-pixels->y-pixels
  (let ((factor (exact->inexact
		 (* (/ (gdk-screen-width-mm) (gdk-screen-width))       ;; x->mm
		    (/ (gdk-screen-height) (gdk-screen-height-mm)))))) ;; mm->y
    (lambda (x)
      (round-exact (* x factor)))))

(define-public (points-draw graph gc series)
  (let* ((lo         (graph-draw-x-lo graph series))
	 (hi         (graph-draw-x-hi graph series))
	 (array      (series-array series lo hi))
	 (win        (gtk-widget-window graph))
	 (height     (c-gdk-drawable-get-height win))
	 (scale-y    (graph-scale-y-proc graph))

	 ;; points 1/2 of step wide, but at least 1 pixel
	 (x-diameter (max 1 (round-exact (* 0.5 scale-x-step))))

	 ;; points centred in step, round down so in 2 pixels we're at the left
	 (x-offset   (quotient (- scale-x-step x-diameter) 2))

	 (y-diameter (max 1 (c-gdk-x-pixels->y-pixels x-diameter)))
	 (y-offset   (- (quotient y-diameter 2)))

	 (win-p-hi   (graph-y->price graph (- y-diameter)))
	 (win-p-lo   (graph-y->price graph (+ height y-diameter))))

    (define (draw-column . col)
      (do ((i lo (1+ i))
	   (x (+ x-offset (graph-scale-x graph lo)) (+ x scale-x-step)))
	  ((> i hi))
	(and-let* ((p (apply array-ref array i col))
		   (  (within-range? win-p-lo win-p-hi p)))
	  (gdk-draw-arc win gc #t
			x (+ y-offset (scale-y p))
			x-diameter y-diameter
			0 23040)))) ;; 360*64 for circle

    (cond ((eq? 'ohlcv (datatype-key (series-datatype series)))
	   ;; closes for ohlcv
	   (draw-column 3))

	  ((= 1 (array-rank array))
	   ;; only one column
	   (draw-column))

	  (else
	   ;; draw lines for all columns
	   (do ((i (array-first-column array) (1+ i))
		(col-hi (array-last-column array)))
	       ((> i col-hi))
	     (draw-column i))))))
