;; Price Volume Trend (PVT) indicator.

;; Copyright 2006, 2007 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(define-module (chart pvt)
  #:use-module (chart indicator)
  #:use-module (chart misc))


;; http://www.incrediblecharts.com/technical/price_and_volume_trend.htm
;; http://www.incrediblecharts.com/technical/pvt_construction.htm
;;     Including sample chart of ANZ.AX from Oct 2000.
;;


(define (pvt-func array prev this)
  (let ((prev-close (array-ref array prev 3))
	(close      (array-ref array this 3))
	(volume     (or (array-ref array this 4) 0)))
    (if (= 0 prev-close)
	0
	(exact->inexact (* volume
			   (/ (- close prev-close)
			      prev-close))))))

(define-public (pvt-series series)
  (accumulation-series series (_ "PVT") pvt-datatype pvt-func))

(define-public pvt-datatype
  (datatype-new!
   #:key    'pvt
   #:name   (_ "PVT - Price Volume Trend")
   #:units  'pvt
   #:manual (_ "CONTEXT(manual-node): Price and Volume Trend")
   #:proc   pvt-series))
