;; RAVI indicator.

;; Copyright 2006, 2007 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(define-module (chart ravi)
  #:use-module (ice-9 and-let-star)
  #:use-module (chart indicator)
  #:use-module (chart misc)
  #:use-module (chart sma))


(define (ravi-fill series new-array lo hi)
  (receive-list (short long)
      (series-data series)
    (let* ((parent     (series-parent series))
	   (lst        (series-prev-n-closes parent
					     #:before lo #:count (1- long)))
	   (array      (series-array-closes parent lo hi))
	   (short-proc (sma-calc-proc short lst))
	   (long-proc  (sma-calc-proc long lst)))

      ;; points lo to hi inclusive
      (do ((i lo (1+ i)))
	  ((> i hi))
	(and-let* ((this (array-ref array i)))
	  (let ((short (short-proc this))
		(long  (long-proc this)))
	    (and short
		 long
		 (not (zero? long))
		 (array-set! new-array (* 100.0 (/ (abs (- short long)) long))
			     i))))))))

(define-public (ravi-series series short long)
  (let ((new-series (series-copy series)))
    (series-name!      new-series (format #f (_ "RAVI ~a,~a") short long))
    (series-datatype!  new-series ravi-datatype)
    (series-parent!    new-series series)
    (series-data!      new-series (list short long))
    (series-require!   new-series (indicator-require-parts ravi-fill
							   (1- long)))
    (series-array!     new-series #f)
    (series-decimals!  new-series 0)
    (series-dividends! new-series '())
    (series-notes!     new-series '())

    new-series))

(define-public ravi-datatype
  (datatype-new!
   #:key     'ravi
   #:name    (_ "RAVI")
   #:units   'ravi
   #:minimum 0
   #:hlines  '(3)
   #:manual  (_ "CONTEXT(manual-node): RAVI")
   #:proc    ravi-series
   #:parameter-list
   (list (datatype-param-new
	  #:key     'ravi-short-days
	  #:name    (_ "Short Days")
	  #:type    'integer
	  #:minimum 1
	  #:default 7)
	 (datatype-param-new
	  #:key     'ravi-short-days
	  #:name    (_ "Long Days")
	  #:type    'integer
	  #:minimum 1
	  #:default 65))))
