;; Standard and Poors web links.

;; Copyright 2007 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(define-module (chart sandp)
  #:use-module (chart download)
  #:use-module (chart misc))


;; an alist of (SYMBOL . URL)
;;
(define standard-and-poors-weblink-alist
  '(
    ;; Australia
    ("^AORD" . "http://www2.standardandpoors.com/portal/site/sp/~a/page.topic/indices_asxallo/2,3,2,8,0,0,0,0,0,0,0,0,0,0,0,0.html")
    ("^ATLI" . "http://www2.standardandpoors.com/portal/site/sp/~a/page.topic/indices_asx20/2,3,2,8,0,0,0,0,0,0,0,0,0,0,0,0.html")
    ("^AFLI" . "http://www2.standardandpoors.com/portal/site/sp/~a/page.topic/indices_asx50/2,3,2,8,0,0,0,0,0,0,0,0,0,0,0,0.html")
    ("^ATOI" . "http://www2.standardandpoors.com/portal/site/sp/~a/page.topic/indices_asx100/2,3,2,8,0,0,0,0,0,0,0,0,0,0,0,0.html")
    ("^AXJO" . "http://www2.standardandpoors.com/portal/site/sp/~a/page.topic/indices_asx200/2,3,2,8,0,0,0,0,0,0,0,0,0,0,0,0.html")
    ("^AXKO" . "http://www2.standardandpoors.com/portal/site/sp/~a/page.topic/indices_asx300/2,3,2,8,0,0,0,0,0,0,0,0,0,0,0,0.html")
    ("^AXMD" . "http://www2.standardandpoors.com/portal/site/sp/~a/page.topic/indices_asxmc50/2,3,2,8,0,0,0,0,0,0,0,0,0,0,0,0.html")
    ("^AXSO" . "http://www2.standardandpoors.com/portal/site/sp/~a/page.topic/indices_asxsmo/2,3,2,8,0,0,0,0,0,0,0,0,0,0,0,0.html")
    ;;
    ;; sectors, only one shared page
    ("^AXEJ" ;; Energy
     . "http://www2.standardandpoors.com/portal/site/sp/~a/page.topic/indices_asxsec/2,3,2,8,0,0,0,0,0,0,0,0,0,0,0,0.html")
    ("^AXMJ" ;; Materials
     . "http://www2.standardandpoors.com/portal/site/sp/~a/page.topic/indices_asxsec/2,3,2,8,0,0,0,0,0,0,0,0,0,0,0,0.html")
    ("^AXNJ" ;; S&P ASX 200 Industrials
     . "http://www2.standardandpoors.com/portal/site/sp/~a/page.topic/indices_asxsec/2,3,2,8,0,0,0,0,0,0,0,0,0,0,0,0.html")
    ("^AXDJ" ;; S&P ASX 200 Consumer Discretionary
     . "http://www2.standardandpoors.com/portal/site/sp/~a/page.topic/indices_asxsec/2,3,2,8,0,0,0,0,0,0,0,0,0,0,0,0.html")
    ("^AXSJ" ;; S&P ASX 200 Consumer Staples
     . "http://www2.standardandpoors.com/portal/site/sp/~a/page.topic/indices_asxsec/2,3,2,8,0,0,0,0,0,0,0,0,0,0,0,0.html")
    ("^AXHJ" ;; S&P ASX 200 Health Care
     . "http://www2.standardandpoors.com/portal/site/sp/~a/page.topic/indices_asxsec/2,3,2,8,0,0,0,0,0,0,0,0,0,0,0,0.html")
    ("^AXFJ" ;; S&P ASX 200 Financials
     . "http://www2.standardandpoors.com/portal/site/sp/~a/page.topic/indices_asxsec/2,3,2,8,0,0,0,0,0,0,0,0,0,0,0,0.html")
    ("^AXIJ" ;; S&P ASX 200 Information Technology
     . "http://www2.standardandpoors.com/portal/site/sp/~a/page.topic/indices_asxsec/2,3,2,8,0,0,0,0,0,0,0,0,0,0,0,0.html")
    ("^AXTJ" ;; S&P ASX 200 Telecommunication Services
     . "http://www2.standardandpoors.com/portal/site/sp/~a/page.topic/indices_asxsec/2,3,2,8,0,0,0,0,0,0,0,0,0,0,0,0.html")
    ("^AXUJ" ;; S&P ASX 200 Utilities
     . "http://www2.standardandpoors.com/portal/site/sp/~a/page.topic/indices_asxsec/2,3,2,8,0,0,0,0,0,0,0,0,0,0,0,0.html")
    ("^AXPJ" ;; S&P ASX 200 Property Trusts
     . "http://www2.standardandpoors.com/portal/site/sp/~a/page.topic/indices_asxsec/2,3,2,8,0,0,0,0,0,0,0,0,0,0,0,0.html")
    ("^AXXJ" ;; S&P ASX 200 Financial-x-Property Trusts
     . "http://www2.standardandpoors.com/portal/site/sp/~a/page.topic/indices_asxsec/2,3,2,8,0,0,0,0,0,0,0,0,0,0,0,0.html")

    ;; Canada
    ("^GSPTSE"   . "http://www2.standardandpoors.com/portal/site/sp/~a/page.topic/indices_tsx/2,3,2,3,0,0,0,0,0,0,0,0,0,0,0,0.html")
    ("^SPTSECP"  . "http://www2.standardandpoors.com/portal/site/sp/~a/page.topic/indices_tsx60cap/2,3,2,3,0,0,0,0,0,0,0,0,0,0,0,0.html")
    ("^SPTSECP3" . #f) ;; S&P TSX 300 Capped
    ("^SPTSEM"   . "http://www2.standardandpoors.com/portal/site/sp/~a/page.topic/indices_tsxmid/2,3,2,3,0,0,0,0,0,0,0,0,0,0,0,0.html")
    ("^SPTSES"   . "http://www2.standardandpoors.com/portal/site/sp/~a/page.topic/indices_tsxsml/2,3,2,3,0,0,0,0,0,0,0,0,0,0,0,0.html")
    ;;
    ;; sectors, one shared page
    ("^SPTTEN"  ;; Energy
     . "http://www2.standardandpoors.com/portal/site/sp/~a/page.topic/indices_tsxsec/2,3,2,3,0,0,0,0,0,0,0,0,0,0,0,0.html")
    ("^SPTTFS"  ;; Financials
     . "http://www2.standardandpoors.com/portal/site/sp/~a/page.topic/indices_tsxsec/2,3,2,3,0,0,0,0,0,0,0,0,0,0,0,0.html")
    ("^SPTTGD"  ;; Gold
     . "http://www2.standardandpoors.com/portal/site/sp/~a/page.topic/indices_tsxsec/2,3,2,3,0,0,0,0,0,0,0,0,0,0,0,0.html")
    ("^SPTTTK"  ;; Information Technology
     . "http://www2.standardandpoors.com/portal/site/sp/~a/page.topic/indices_tsxsec/2,3,2,3,0,0,0,0,0,0,0,0,0,0,0,0.html")
    ("^GSPTTCD" ;; Consumer Discretionary
     . "http://www2.standardandpoors.com/portal/site/sp/~a/page.topic/indices_tsxsec/2,3,2,3,0,0,0,0,0,0,0,0,0,0,0,0.html")
    ("^GSPTTCS" ;; Consumer Staples
     . "http://www2.standardandpoors.com/portal/site/sp/~a/page.topic/indices_tsxsec/2,3,2,3,0,0,0,0,0,0,0,0,0,0,0,0.html")
    ("^GSPTTHC" ;; Health Care
     . "http://www2.standardandpoors.com/portal/site/sp/~a/page.topic/indices_tsxsec/2,3,2,3,0,0,0,0,0,0,0,0,0,0,0,0.html")
    ("^GSPTTIN" ;; Industrials
     . "http://www2.standardandpoors.com/portal/site/sp/~a/page.topic/indices_tsxsec/2,3,2,3,0,0,0,0,0,0,0,0,0,0,0,0.html")
    ("^GSPTTMN" ;; Diversified Metals & Mining
     . "http://www2.standardandpoors.com/portal/site/sp/~a/page.topic/indices_tsxsec/2,3,2,3,0,0,0,0,0,0,0,0,0,0,0,0.html")
    ("^GSPTTMT" ;; Materials
     . "http://www2.standardandpoors.com/portal/site/sp/~a/page.topic/indices_tsxsec/2,3,2,3,0,0,0,0,0,0,0,0,0,0,0,0.html")
    ("^GSPTTRE" ;; Real Estate
     . "http://www2.standardandpoors.com/portal/site/sp/~a/page.topic/indices_tsxsec/2,3,2,3,0,0,0,0,0,0,0,0,0,0,0,0.html")
    ("^GSPTTTS" ;; Telecommunications Services
     . "http://www2.standardandpoors.com/portal/site/sp/~a/page.topic/indices_tsxsec/2,3,2,3,0,0,0,0,0,0,0,0,0,0,0,0.html")
    ("^GSPTTUT" ;; Utilities
     . "http://www2.standardandpoors.com/portal/site/sp/~a/page.topic/indices_tsxsec/2,3,2,3,0,0,0,0,0,0,0,0,0,0,0,0.html")
    ;;
    ;; TSX Venture
    ("^SPCDNX"   . "http://www2.standardandpoors.com/portal/site/sp/~a/page.topic/indices_tsxven/2,3,2,3,0,0,0,0,0,0,0,0,0,0,0,0.html")

    ;; USA
    ("^GSPC" . "http://www2.standardandpoors.com/portal/site/sp/~a/page.topic/indices_500/2,3,2,2,0,0,0,0,0,0,0,0,0,0,0,0.html")
    ("^OEX"  . "http://www2.standardandpoors.com/portal/site/sp/~a/page.topic/indices_100/2,3,2,2,0,0,0,0,0,0,0,0,0,0,0,0.html")

    ))

(define-public (standard-and-poors-weblink! symbol url)
  (set! standard-and-poors-weblink-alist 
	(acons symbol url standard-and-poors-weblink-alist)))

(define (standard-and-poors-weblink-get symbol)
  (assoc-ref standard-and-poors-weblink-alist symbol))

(weblink-handler! standard-and-poors-weblink-get
  (lambda (symbol)
    (list (_ "Standard and _Poors index information")
	  (_ "Open web browser at the Standard and Poors web site page for this index")
	  (lambda ()
	    ;; language choices at page
	    ;; http://www2.standardandpoors.com/portal/site/sp/en/au/page.siteselection/site_selection/0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0.html
	    ;; the selections considered here are the ones that follow the
	    ;; basic pattern
	    ;;
	    (let ((location (lang-select '(("en" "en/us"
					    "es" "es/la"
					    "pt" "pt/la"))))
		  (fmt      (standard-and-poors-weblink-get symbol)))
	      (format #f fmt location))))))
