;; Scrolling by dragging.

;; Copyright 2005, 2006, 2007 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(define-module (chart scroll-drag)
  #:use-module (gtk gtk)
  #:use-module (gtk gdk)
  #:use-module (srfi srfi-1)
  #:autoload   (chart cross)       (cross-disable)
  #:use-module (chart extra-gtk)
  #:use-module (chart graph)
  #:use-module (chart keymap)
  #:use-module (chart misc)
  #:use-module (chart view))


;;-----------------------------------------------------------------------------
;; cross state

(let-toplevel ((state #f))
  (notify-connect 'cross-update (lambda (new-state)
				  (set! state new-state)))
  ;; do a `cross-disable', but avoid loading the cross module if it isn't
  ;; already in use
  (define-toplevel (scroll-drag-cross-disable)
    (if state
	(cross-disable))))


;;-----------------------------------------------------------------------------
;; scroll drag

(define scroll-drag-cursor (gdk-cursor-new 'hand2))

;; graph widget, or #f
(define scroll-drag-graph  #f)

(define hadj-initial #f)
(define vadj-initial #f)
(define x-initial #f)
(define y-initial #f)

(define (scroll-drag-move x y)
  (if scroll-drag-graph
      (begin
	;; must confine values to adj range

	(c-gtk-adjustment-set-value-inpage
	 (graph-hadjustment scroll-drag-graph)
	 (- hadj-initial (/ (- x x-initial) scale-x-step)))

	(c-gtk-adjustment-set-value-inpage
	 (graph-vadjustment scroll-drag-graph)
	 (+ vadj-initial (/ (- y y-initial)
			    (graph-scale-y-factor scroll-drag-graph)))))))

(define (scroll-drag-done)
  (if scroll-drag-graph
      (begin
	(gdk-window-set-cursor (gtk-widget-window scroll-drag-graph) #f)
	(set! scroll-drag-graph #f))))

(define done-init-list '())

(define (handle-motion-notify event)
  (let ((values (gdk-window-get-pointer (gdk-event-window event))))
    (scroll-drag-move (inexact->exact (second values))
		      (inexact->exact (third  values)))))

(define (handle-button-release event)
  (scroll-drag-move (inexact->exact (gdk-event-x event))
		    (inexact->exact (gdk-event-y event)))
  (scroll-drag-done))

(define (init graph)
  (if (not (memq graph done-init-list))
      (begin
	(set! done-init-list (cons graph done-init-list))
	(gtk-signal-connect graph "motion_notify_event"
	  handle-motion-notify)
	(gtk-signal-connect graph "button_release_event"
	  handle-button-release))))

;; initiate a drag with a button press from `keymap-dispatch-button'
(define-public (scroll-drag-start-button)
  (set! scroll-drag-graph keymap-widget)

  (init scroll-drag-graph)
  (scroll-drag-cross-disable)
  (gdk-window-set-cursor (gtk-widget-window scroll-drag-graph)
			 scroll-drag-cursor)

  (set! hadj-initial
	(gtk-adjustment-value (graph-hadjustment scroll-drag-graph)))
  (set! x-initial (inexact->exact (gdk-event-x keymap-event)))

  (set! vadj-initial
	(gtk-adjustment-value (graph-vadjustment scroll-drag-graph)))
  (set! y-initial (inexact->exact (gdk-event-y keymap-event))))
