;; Shanghai Stock Exchange (SSE) setups.

;; Copyright 2007 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(define-module (chart shanghai)
  #:use-module (srfi srfi-13)
  #:use-module (chart misc)
  #:use-module (chart yahoo))


(define-public (shanghai-symbol? symbol)   
  (string-suffix? ".SS" symbol))

;; http://au.finance.yahoo.com/intlindices
;; ^SSEC shanghai composite
;;
;; except it doesn't work with the quotes.csv, it redirects to 000001.SS by
;; some magic, so the latter should be used apparently
;;
;; and note not hitting ^SSMI (switzerland.scm)
;;
(define-public (yahoo-index-symbol-shanghai? symbol)   
  (string-prefix? "^SSE" symbol))

(define-public timezone-shanghai
  (make-procedure-with-cache-0
   (lambda ()
     (timezone-choose (_ "Shanghai")
		      '("Asia/Shanghai"
			;; no daylight savings
			"CST-8")
		      "CST-8"))))

(symbol-timezone! shanghai-symbol? (_ "Shanghai") timezone-shanghai)
(symbol-timezone! yahoo-index-symbol-shanghai? (_ "Shanghai") timezone-shanghai)
(yahoo-quote-delay! shanghai-symbol? 0) ;; real-time

;; http://www.sse.com.cn/sseportal/en_us/ps/about/tcs.shtml
;; 9:15 to 9:25 pricing, 9:30-11:30 and 13:00-15:00 trading
;;
(yahoo-quote-lock! shanghai-symbol?
		   #,(hms->seconds 9 15 0) #,(hms->seconds 15 0 0))


;;-----------------------------------------------------------------------------
;; web links - indices

;; not sure about this, but think all indices begin with 0
(define (shanghai-index-symbol? symbol)
  (and (shanghai-symbol? symbol)
       (string-prefix? "0" symbol)))

(weblink-handler! shanghai-index-symbol?
  (lambda (symbol)
    (list (_ "SSE _Index Information")
	  (_ "Open web browser at the Shanghai Stock Exchange page for this index")
	  (string-append "http://www.sse.com.cn/sseportal/en_us/ps/ggxx/zsjbxx.jsp?indexName=&indexCode=" (chart-symbol-sans-dot symbol)))))
