;; Singapore International Money Exchange (SIMEX) setups.

;; Copyright 2005, 2006, 2007 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(define-module (chart simex)
  #:use-module (srfi srfi-13)
  #:use-module (chart barchart)
  #:use-module (chart download)
  #:use-module (chart misc))


(define-public (simex-symbol? symbol)
  (string-suffix? ".SIMEX" symbol))

(symbol-timezone! simex-symbol? (_ "Singapore") timezone-singapore)


;;-----------------------------------------------------------------------------
;; barchart for database and for intraday graphs

(set! barchart-latest-symbol-predlist
      (cons simex-symbol? barchart-latest-symbol-predlist))
(set! barchart-download-symbol-predlist
      (cons simex-symbol? barchart-download-symbol-predlist))

;; FIXME: dunno what delay for simex, www.sgx.com says its quotes are
;; delayed, but doesn't seem to say be how much
;;
;; (set! barchart-suffix-delay-alist (acons ".SIMEX" 20
;; 					 barchart-suffix-delay-alist))

(set! barchart-commodity-mung-alist
      (acons simex-symbol?

	     ;; "JP" MSCI Japan seems to be missing from barchart, it looks
	     ;; like that contract has had no volume for a long time, so it
	     ;; doesn't matter
	     ;;
	     '(;; equity index futures
	       ("NK" . "NX")  ;; nikkei 225
	       ("N3" . "VN")  ;; nikkei 300
	       ("JP" . #f)    ;; msci japan
	       ("TW" . "TI")  ;; msci taiwan
	       ("SG" . "SV")  ;; msci singapore
	       ("ST" . "VS")  ;; straits times
	       ("HK" . "VK")  ;; msci hong kong
	       ("IN" . "NH")  ;; cnx nifty

	       ;; interest rate futures
	       ("ED" . "DS")  ;; eurodollar
	       ("EY" . "WT")  ;; euroyen tibor
	       ("EL" . "SL")  ;; euroyen libor
	       ("JG" . "JV")  ;; japan 10-yr bond
	       ("JB" . "JX")  ;; japan 10-yr bond mini
	       ("SD" . "VD")  ;; singapore dollar interest rate
	       ("SB" . "JZ")  ;; singapore 5-yr bond
	       )
	     barchart-commodity-mung-alist))


;;-----------------------------------------------------------------------------
;; web link - home page only for now
;;

(weblink-handler! simex-symbol?
  (lambda (symbol)
    (list (_ "SGX _Home Page")
	  (_ "Open web browser at the Singapore Stock Exchange home page")
	  "http://www.sgx.com")))
