;; Solid line drawing.

;; Copyright 2003, 2004, 2005, 2006, 2007 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(define-module (chart solid)
  #:use-module (ice-9 optargs)
  #:use-module (gtk gdk)
  #:use-module (gtk gtk)
  #:use-module (chart extra-gdk)
  #:use-module (chart graph)
  #:use-module (chart misc)
  #:use-module (chart view))


;; ENHANCE-ME: could draw #f gaps in the data in grey or stipple
;;
(define*-public (solid-draw graph gc series #:key column (colour "white"))
  (let* ((lo      (graph-draw-x-lo graph series -1))
	 (hi      (graph-draw-x-hi graph series))
	 (array   (series-array series lo hi))
	 (win     (gtk-widget-window graph))
	 (scale-x (graph-scale-x-proc graph))
	 (scale-y (graph-scale-y-proc graph))
	 ;; centre in step
	 (x-first (+ (scale-x lo) (quotient scale-x-step 2)))
	 (poly    (list (cons x-first (scale-y 0))))
	 (gc      (c-gdk-gc-get win #:foreground colour)))

    (case (datatype-key (series-datatype series))
      ((ohlcv)
       (set! array (make-shared-array-column array 3))))

    ;; given column, and do nothing if no such column (as for MACD when
    ;; histogram disabled)
    (if column
	(set! array (and (<= column (array-last-column array))
			 (make-shared-array-column array column))))

    (if array
	(begin
	  (do ((i lo (1+ i))
	       (x x-first (+ x scale-x-step)))
	      ((> i hi))
	    (if (array-ref array i)
		(set! poly (acons x (scale-y (array-ref array i))
				  poly))))

	  (set! poly (cons (cons (scale-x hi) (scale-y 0)) poly))
	  (gdk-draw-polygon win gc #t poly)))))
