;; Stop-loss levels drawing.

;; Copyright 2006, 2007 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(define-module (chart stops)
  #:use-module (gtk gdk)
  #:use-module (gtk gtk)
  #:use-module (chart extra-gdk)
  #:use-module (chart graph)
  #:use-module (chart misc)
  #:use-module (chart view))

(define (set-cadar! lst val)
  (set-car! (cdar lst) val))

;; SERIES is a set of stop-loss values to be drawn as horizontal lines, for
;; parabolic SAR or for Darvas boxes.
;; There can be #f values for gaps (nothing drawn).
;; There can be adjacent values the same, or rounding to the same, and the
;; line segments are collapsed together for that.
;;
(define-public (stops-draw graph gc series)
  (let* ((lo       (graph-draw-x-lo graph series))
	 (hi       (graph-draw-x-hi graph series))
	 (array    (series-array series lo hi))
	 (win      (gtk-widget-window graph))
	 (height   (c-gdk-drawable-get-height win))
	 (win-p-lo (graph-y->price graph height))
	 (win-p-hi (graph-y->price graph -1))
	 (scale-y  (graph-scale-y-proc graph)))

    (define (draw-column . col)
      (let ((seg-list '()) ;; list of line segments ((x1 . y1) . (x2 . y2))
	    (seg-y    #f)) ;; y from last of seg-list, if joinable
	(do ((i lo (1+ i))
	     (x (graph-scale-x graph lo) (+ x scale-x-step)))
	    ((> i hi))
	  (let ((p (apply array-ref array i col)))
	    (if (and p
		     (within-range? win-p-lo win-p-hi p))
		(let ((y (scale-y p)))
		  (if (eqv? y seg-y)
		      ;; extend last seg-list entry
		      (set-cadar! seg-list (+ x scale-x-step -1))
		      ;; new seg-list entry
		      (begin
			(set! seg-list (cons (cons (cons x y)
						   (cons (+ x scale-x-step -1)
							 y))
					     seg-list))
			(set! seg-y y))))

		;; no price, or outside window, segment ends
		(set! seg-y #f))))

	(gdk-draw-segments win gc seg-list)))

    (cond ((= 1 (array-rank array))
	   ;; only one column
	   (draw-column))

	  (else
	   ;; draw stops lines for all columns
	   (do ((col (array-first-column array) (1+ col)))
	       ((> col (array-last-column array)))
	     (draw-column col))))))
