;; Exercise adjust.scm.

;; Copyright 2006, 2007 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(load "tests.scm")
(use-modules (chart adjust))


;;
;; elem-selector-proc
;;

(define elem-selector-proc (@@ (chart adjust) elem-selector-proc))

;;                                               split div rollover
(let ((lst '(( (123 . "something")                  #f   #f  #f)
	     ( (123 . (split 5 2))                  #t   #f  #f)
	     ( (123 . (amount 456))                 #f   #t  #f)
	     ( (123 . (amount 456 (decimals 2)))    #f   #t  #f)
	     ( (123 . (amount unknown))             #f   #f  #f)
	     ( (123 . (amount to-be-advised))       #f   #f  #f)
	     ( (123 . (month 456))                  #f   #f  #f)
	     ( (123 . (month 456 "20"))             #f   #f  #t)
	     ( (123 . (month 456 "2.0"))            #f   #f  #t))))
  (define split-proc    (elem-selector-proc #t #f #f))
  (define div-proc      (elem-selector-proc #f #t #f))
  (define rollover-proc (elem-selector-proc #f #f #t))

  (for-each (lambda (rec)
	      (let* ((elem (first rec))
		     (want (second rec))
		     (got  (split-proc elem)))
		(test "elem-selector-proc - splits" elem want got))

	      (let* ((elem (first rec))
		     (want (third rec))
		     (got  (div-proc elem)))
		(test "elem-selector-proc - dividends" elem want got))

	      (let* ((elem (first rec))
		     (want (fourth rec))
		     (got  (rollover-proc elem)))
		(test "elem-selector-proc - rollovers" elem want got)))
	    lst))


;;
;; adjust-series
;;

(let* ((series (series-new #:decimals 0
			   #:datatype datatype-ohlcv
			   #:array (list->array 2 '((42 44 38 40 100)
						    (19 23 17 18 45)))
			   #:dividends '((1 split 2 1))))
       (adj    (adjust-series series #t #t #t #t))
       (want   '((21.0 22.0 19.0 20.0 200)
		 (19 23 17 18 45)))
       (got    (array->list (series-array adj 0 1))))
  (test "adjust-series" series want got))

(let* ((series (series-new #:decimals 0
			   #:datatype datatype-ohlcv
			   #:array (list->array 2 '((110 110 110 110 2000)
						    (19 23 17 18 45)))
			   #:dividends '((1 amount 10))))
       (adj    (adjust-series series #t #t #t #t))
       (want   '((100.0 100.0 100.0 100.0 2200.0)
		 (19 23 17 18 45)))
       (got    (array->list (series-array adj 0 1))))
  (test "adjust-series" series want got))

;; different decimals in the div elem
(let* ((series (series-new #:decimals 0
			   #:datatype datatype-ohlcv
			   #:array (list->array 2 '((110 110 110 110 2000)
						    (19 23 17 18 45)))
			   #:dividends '((1 amount 100 (decimals . 1)))))
       (adj    (adjust-series series #t #t #t #t))
       (want   '((100.0 100.0 100.0 100.0 2200.0)
		 (19 23 17 18 45)))
       (got    (array->list (series-array adj 0 1))))
  (test "adjust-series" series want got))

(let* ((series (series-new #:decimals 0
			   #:datatype datatype-ohlcv
			   #:array (list->array 2 '((110 110 110 110 2000)
						    (19 23 17 18 45)))
			   #:dividends '((1 month 3 "-10"))))
       (adj    (adjust-series series #t #t #t #t))
       (want   '((100.0 100.0 100.0 100.0 2200.0)
		 (19 23 17 18 45)))
       (got    (array->list (series-array adj 0 1))))
  (test "adjust-series" series want got))

(let* ((series (series-new #:decimals 0
			   #:datatype datatype-ohlcv
			   #:array (list->array 2 '((42 44 38 40 100)
						    (19 23 17 18 45)))
			   #:dividends '((1 split 2 1)
					 (0 split 2 1)  ;; out of range ...
					 (-1 split 2 1)
					 (6 split 2 1))))
       (adj    (adjust-series series #t #t #t #t))
       (want   '((21.0 22.0 19.0 20.0 200)
		 (19 23 17 18 45)))
       (got    (array->list (series-array adj 0 1))))
  (test "adjust-series" series want got))

(tests-end)
