;; Test athens.scm functions.

;; Copyright 2005, 2006, 2007 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(load "tests.scm")
(use-modules (chart athens))


;;
;; athens-available-tdate
;;

(define athens-available-tdate (@@ (chart athens) athens-available-tdate))

(let ((ymdhm  '(2005 1 6  17 30))) ;; Thursday
  (apply fake-localtime-ymdhm! (timezone-athens) ymdhm)
  (let ((want (ymd->tdate 2005 1 5))
	(got  (athens-available-tdate)))
    (test "athens-available-tdate" ymdhm want got)))

(let ((ymdhm  '(2005 1 6  18 30))) ;; Thursday
  (apply fake-localtime-ymdhm! (timezone-athens) ymdhm)
  (let ((want (ymd->tdate 2005 1 6))
	(got  (athens-available-tdate)))
    (test "athens-available-tdate" ymdhm want got)))

(let ((ymdhm  '(2005 1 7  17 30))) ;; Friday
  (apply fake-localtime-ymdhm! (timezone-athens) ymdhm)
  (let ((want (ymd->tdate 2005 1 6))
	(got  (athens-available-tdate)))
    (test "athens-available-tdate" ymdhm want got)))

(let ((ymdhm  '(2005 1 7  18 30))) ;; Friday
  (apply fake-localtime-ymdhm! (timezone-athens) ymdhm)
  (let ((want (ymd->tdate 2005 1 7))
	(got  (athens-available-tdate)))
    (test "athens-available-tdate" ymdhm want got)))

(let ((ymdhm  '(2005 1 8  17 30))) ;; Saturday
  (apply fake-localtime-ymdhm! (timezone-athens) ymdhm)
  (let ((want (ymd->tdate 2005 1 7))
	(got  (athens-available-tdate)))
    (test "athens-available-tdate" ymdhm want got)))

(let ((ymdhm  '(2005 1 8  18 30))) ;; Saturday
  (apply fake-localtime-ymdhm! (timezone-athens) ymdhm)
  (let ((want (ymd->tdate 2005 1 7))
	(got  (athens-available-tdate)))
    (test "athens-available-tdate" ymdhm want got)))

(let ((ymdhm  '(2005 1 9  17 30))) ;; Sunday
  (apply fake-localtime-ymdhm! (timezone-athens) ymdhm)
  (let ((want (ymd->tdate 2005 1 7))
	(got  (athens-available-tdate)))
    (test "athens-available-tdate" ymdhm want got)))

(let ((ymdhm  '(2005 1 9  18 30))) ;; Sunday
  (apply fake-localtime-ymdhm! (timezone-athens) ymdhm)
  (let ((want (ymd->tdate 2005 1 7))
	(got  (athens-available-tdate)))
    (test "athens-available-tdate" ymdhm want got)))

(let ((ymdhm  '(2005 1 10  17 30))) ;; Monday
  (apply fake-localtime-ymdhm! (timezone-athens) ymdhm)
  (let ((want (ymd->tdate 2005 1 7))
	(got  (athens-available-tdate)))
    (test "athens-available-tdate" ymdhm want got)))

(let ((ymdhm  '(2005 1 10  18 30))) ;; Monday
  (apply fake-localtime-ymdhm! (timezone-athens) ymdhm)
  (let ((want (ymd->tdate 2005 1 10))
	(got  (athens-available-tdate)))
    (test "athens-available-tdate" ymdhm want got)))

(let ((ymdhm  '(2005 1 11  17 30))) ;; Tuesday
  (apply fake-localtime-ymdhm! (timezone-athens) ymdhm)
  (let ((want (ymd->tdate 2005 1 10))
	(got  (athens-available-tdate)))
    (test "athens-available-tdate" ymdhm want got)))

(let ((ymdhm  '(2005 1 11  18 30))) ;; Tuesday
  (apply fake-localtime-ymdhm! (timezone-athens) ymdhm)
  (let ((want (ymd->tdate 2005 1 11))
	(got  (athens-available-tdate)))
    (test "athens-available-tdate" ymdhm want got)))

(let ((ymdhm  '(2005 1 12  17 30))) ;; Wednesday
  (apply fake-localtime-ymdhm! (timezone-athens) ymdhm)
  (let ((want (ymd->tdate 2005 1 11))
	(got  (athens-available-tdate)))
    (test "athens-available-tdate" ymdhm want got)))

(let ((ymdhm  '(2005 1 12  18 30))) ;; Wednesday
  (apply fake-localtime-ymdhm! (timezone-athens) ymdhm)
  (let ((want (ymd->tdate 2005 1 12))
	(got  (athens-available-tdate)))
    (test "athens-available-tdate" ymdhm want got)))

;;
;; athens-dividend-parse
;;

(define athens-dividend-parse (@@ (chart athens) athens-dividend-parse))

(let* ((str  "something")
       (want "something")
       (got  (athens-dividend-parse str)))
  (test "athens-dividend-parse" str want got))

(let* ((str  ".")
       (want ".")
       (got  (athens-dividend-parse str)))
  (test "athens-dividend-parse" str want got))

(let* ((str  "..")
       (want "..")
       (got  (athens-dividend-parse str)))
  (test "athens-dividend-parse" str want got))

(let* ((str  "1.23")
       (want '(amount 123 (decimals . 2)))
       (got  (athens-dividend-parse str)))
  (test "athens-dividend-parse" str want got))

(let* ((str  "1.")
       (want '(amount 1 (decimals . 0)))
       (got  (athens-dividend-parse str)))
  (test "athens-dividend-parse" str want got))

(let* ((str  ".50")
       (want '(amount 50 (decimals . 2)))
       (got  (athens-dividend-parse str)))
  (test "athens-dividend-parse" str want got))

(let* ((str  "1.23.45")
       (want "1.23.45")
       (got  (athens-dividend-parse str)))
  (test "athens-dividend-parse" str want got))

(tests-end)
