;; Test database functions.

;; Copyright 2003, 2004, 2005, 2006, 2007 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(load "tests.scm")
(use-modules (chart database)
	     (chart extra-glib))


(system "rm -rf database-temp")
(set! database-directory    (g-build-filename (getcwd) "database-temp"))
(set! data-directory        (g-build-filename database-directory "data"))
(set! cache-directory       (g-build-filename database-directory "cache"))
(set! user-notes-directory  (g-build-filename database-directory "notes"))


;;
;; database-read-series, database-empty-series?
;;

(define database-empty-series? (@@ (chart download) database-empty-series?))

(let* ((symbol "NOSUCH")
       (want   #f)
       (got    (database-read-series symbol)))
  (test "database-read-series" symbol want got))

(database-new "EMPTY")
(let* ((symbol "EMPTY")
       (want   #t)
       (got    (->bool (database-read-series symbol))))
  (test "database-read-series" symbol want got))

(let* ((symbol "EMPTY")
       (want   #t)
       (got    (->bool (database-empty-series?
			(database-read-series symbol)))))
  (test "database-empty-series?" symbol want got))

(let ((today (tdate-today)))
  (database-write-series (series-new #:symbol "ONEDAY"
				     #:array  (make-array #f (list today today)
							  '(0 4))
				     #:datatype datatype-ohlcv))

  (series-merge-ohlcv (database-read-series "EMPTY")
		      (database-read-series "ONEDAY"))

  (series-merge-ohlcv (database-read-series "ONEDAY")
		      (database-read-series "EMPTY")))

;;
;; dividend-months-flatten
;;

(define dividend-months-flatten (@@ (chart database) dividend-months-flatten))

(let* ((dividends '((1 . (month 1))))
       (want      '((1 . (month 1))))
       (got       (sort (dividend-months-flatten dividends)
			car<)))
  (test "dividend-months-flatten" dividends want got))

(let* ((dividends '((1 . (month 1))
		    (2 . (month 2))))
       (want      '((1 . (month 1))
		    (2 . (month 2))))
       (got       (sort (dividend-months-flatten dividends)
			car<)))
  (test "dividend-months-flatten" dividends want got))

(let* ((dividends '((2 . (month 2))
		    (1 . (month 1))))
       (want      '((1 . (month 1))
		    (2 . (month 2))))
       (got       (sort (dividend-months-flatten dividends)
			car<)))
  (test "dividend-months-flatten" dividends want got))

(let* ((dividends '((1 . (month 1))
		    (2 . (month 1))))
       (want      '((1 . (month 1))))
       (got       (sort (dividend-months-flatten dividends)
			car<)))
  (test "dividend-months-flatten" dividends want got))

(let* ((dividends '((1 . (month 1))
		    (2 . (month 2))))
       (want      '((1 . (month 1))
		    (2 . (month 2))))
       (got       (sort (dividend-months-flatten dividends)
			car<)))
  (test "dividend-months-flatten" dividends want got))

(let* ((want      '((1 . (month 1))
		    (2 . "foo")))
       (dividends '((1 . (month 1))
		    (2 . "foo")
		    (3 . (month 1))))
       (got       (sort (dividend-months-flatten dividends)
			car<)))
  (test "dividend-months-flatten" dividends want got))

;;
;; read-last
;;

(define read-last (@@ (chart database) read-last))

(let* ((str      "")
       (want     eof-object)
       (got      (read-last (open-input-string str))))
  (test "read-last" str want got))

(let* ((str      "1")
       (want     1)
       (got      (read-last (open-input-string str))))
  (test "read-last" str want got))

(let* ((str      "1 2")
       (want     2)
       (got      (read-last (open-input-string str))))
  (test "read-last" str want got))

(let* ((str      "1 2 3")
       (want     3)
       (got      (read-last (open-input-string str))))
  (test "read-last" str want got))


(system "rm -rf database-temp")
(tests-end)
