;; Exercise dema.scm.

;; Copyright 2006, 2007 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(load "tests.scm")
(use-modules (chart dema)
	     (chart ema))

;; return the coefficient of the f^k term in an EMA
(define (ref:ema-coef f k)
  (- 1 f))

;; return the coefficient of the f^k term in an EMAofEMA
(define (ref:ema-2-coef f k)
  (* (- 1 f)
     (- 1 f)
     (1+ k)))

;; return the coefficient of the f^k term in a DEMA
(define (ref:dema-coef f k)
  (- (* 2 (ref:ema-coef f k))
     (ref:ema-2-coef f k)))

(define (ref:dema-total-abs-weight count f)
  (1+ (* 2 (expt f (1+ count)))))

(define (ref:dema-omitted count f k)
  (do ((k          (1+ k) (1+ k))
       (stop       #f)
       (total      0.0))
      (stop
       (/ total (ref:dema-total-abs-weight count f)))
    (let ((weight (* (expt f k)
		     (ref:dema-coef f k))))
      (if (and (> k (* 5 count))
	       (< (/ (abs weight) total) 0.0000001))
	  (set! stop #t))
      (set! weight (abs weight))
      (set! total (+ total weight)))))

;;
;; ref:dema-coef
;;

(let* ((count 3)
       (f     (ema-count->f count))
       (k     count)
       (want  0.0)
       (got   (ref:dema-coef f k)))
  (test "ref:dema-coef" (list count k) want got))

(let* ((count 3)
       (f     (ema-count->f count))
       (k     (1- count))
       (want  #t)
       (got   (positive? (ref:dema-coef f k))))
  (test "ref:dema-coef" (list count k) want got))

(let* ((count 3)
       (f     (ema-count->f count))
       (k     (1+ count))
       (want  #t)
       (got   (negative? (ref:dema-coef f k))))
  (test "ref:dema-coef" (list count k) want got))

(let* ((count 7)
       (f     (ema-count->f count))
       (k     count)
       (want  0.0)
       (got   (ref:dema-coef f k)))
  (test "ref:dema-coef" (list count k) want got))

(let* ((count 7)
       (f     (ema-count->f count))
       (k     (1- count))
       (want  #t)
       (got   (positive? (ref:dema-coef f k))))
  (test "ref:dema-coef" (list count k) want got))

(let* ((count 7)
       (f     (ema-count->f count))
       (k     (1+ count))
       (want  #t)
       (got   (negative? (ref:dema-coef f k))))
  (test "ref:dema-coef" (list count k) want got))

;;
;; ref:dema-omitted
;;

(let* ((count 7)
       (f     (ema-count->f count))
       (want  1.0)
       (got   (ref:dema-omitted count f -1)))
  (test "ref:dema-omitted" count want got
	#:compare =-close?))



;;
;; dema-omitted
;;

(for-each
 (lambda (count)
   (define f (ema-count->f count))
   (for-each (lambda (k)
	       (let* ((want  (ref:dema-omitted count f k))
		      (got   (dema-omitted count f k)))
		 (test "dema-omitted" (list count k) want got
		       #:compare =-close?)))
	     '(1 5 10 20 50 100 200)))
 '(3 7 20 40))

(for-each
 (lambda (count)
   (let* ((k    -1)
	  (f    (ema-count->f count))
	  (want 1.0)
	  (got  (dema-omitted count f k)))
     (test "dema-omitted" (list count k) want got)))
 '(3 7 20 40))

;;
;; dema-warmup-count
;;

(let* ((count 1)
       (want  0)
       (got   (dema-warmup-count count)))
  (test "dema-warmup-count" count want got))

(let* ((count 2)
       (want  #f)
       (got   (negative? (dema-warmup-count count))))
  (test "dema-warmup-count positive" count want got))



(tests-end)
