;; Exercise div-edit.scm.

;; Copyright 2006, 2007 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(load "tests.scm")
(need-gtk)


;;
;; value->text
;;

(define value->text (@@ (chart div-edit) value->text))

(let* ((value "something")
       (want  "something")
       (got   (value->text value)))
  (test "value->text" value want got))

(let* ((value '(month 0))
       (want  (strftime-utf8 "month: %b %Y" (mdate->tm 0)))
       (got   (value->text value)))
  (test "value->text" value want got))

(let* ((value '(month 0))
       (want  (strftime-utf8 (_ "month: %b %Y") (mdate->tm 0)))
       (got   (value->text value)))
  (test "value->text" value want got))

(let* ((value '(split 5 2))
       (want  (format #f (_ "split ~a for ~a") 5 2))
       (got   (value->text value)))
  (test "value->text" value want got))

(let* ((value '(amount unknown))
       (want  (_ "unknown"))
       (got   (value->text value)))
  (test "value->text" value want got))

(let* ((value '(amount to-be-advised))
       (want  (_ "to-be-advised"))
       (got   (value->text value)))
  (test "value->text" value want got))

(let* ((value '(amount 123 (decimals . 0)))
       (want  "123")
       (got   (value->text value)))
  (test "value->text" value want got))

(let* ((value '(amount 123 (decimals . 0) (estimated . #t)))
       (want  (string-append (_ "estimated ") "123"))
       (got   (value->text value)))
  (test "value->text" value want got))

(let* ((value '(amount 123 (decimals . 2)))
       (want  (string-append "1" (locale-decimal-point) "23"))
       (got   (value->text value)))
  (test "value->text" value want got))


;;
;; text->value
;;

(define text->value (@@ (chart div-edit) text->value))

(let* ((text  "something")
       (want  "something")
       (got   (text->value text)))
  (test "text->value" text want got))

(let* ((text  (strftime (_ "month: %b %Y") (mdate->tm 0)))
       (want  '(month 0))
       (got   (text->value text)))
  (test "text->value" text want got))

(let* ((text  (format #f (_ "split ~a for ~a") 5 2))
       (want  '(split 5 2))
       (got   (text->value text)))
  (test "text->value" text want got))

(let* ((text  (_ "unknown"))
       (want  '(amount unknown))
       (got   (text->value text)))
  (test "text->value" text want got))

(let* ((text  (_ "to-be-advised"))
       (want  '(amount to-be-advised))
       (got   (text->value text)))
  (test "text->value" text want got))

(let* ((text  "123")
       (want  '(amount 123 (decimals . 0)))
       (got   (text->value text)))
  (test "text->value" text want got))

(let* ((text  (string-append (_ "estimated ") "123"))
       (want  '(amount 123 (decimals . 0) (estimated . #t)))
       (got   (text->value text)))
  (test "text->value" text want got))

(let* ((text  (string-append "1" (locale-decimal-point) "23"))
       (want  '(amount 123 (decimals . 2)))
       (got   (text->value text)))
  (test "text->value" text want got))

(let* ((text  (string-append "" (locale-decimal-point) "23"))
       (want  '(amount 23 (decimals . 2)))
       (got   (text->value text)))
  (test "text->value" text want got))

(let* ((text  (string-append "1" (locale-decimal-point) ""))
       (want  '(amount 1 (decimals . 0)))
       (got   (text->value text)))
  (test "text->value" text want got))

(let* ((text  (locale-decimal-point)) ;; just decimal point is not a number
       (want  (locale-decimal-point))
       (got   (text->value text)))
  (test "text->value" text want got))



(tests-end)
