;; Exercise ema.scm.

;; Copyright 2006, 2007 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(load "tests.scm")
(use-modules (chart ema))


;;
;; ema-alpha->count
;;

(let* ((alpha 1.0)
       (want  1.0)
       (got   (ema-alpha->count alpha)))
  (test "ema-alpha->count" alpha want got))

(let* ((alpha 0.5)
       (want  3.0)
       (got   (ema-alpha->count alpha)))
  (test "ema-alpha->count" alpha want got))

(let* ((alpha 0.25)
       (want  7.0)
       (got   (ema-alpha->count alpha)))
  (test "ema-alpha->count" alpha want got))

;;
;; ema-count->alpha
;;

(let* ((count 1)
       (want  1.0)
       (got   (ema-count->alpha count)))
  (test "ema-count->alpha" count want got))

(let* ((count 3)
       (want  0.5)
       (got   (ema-count->alpha count)))
  (test "ema-count->alpha" count want got))

(let* ((count 7)
       (want  0.25)
       (got   (ema-count->alpha count)))
  (test "ema-count->alpha" count want got))

;;
;; ema-count->f
;;

(let* ((count 1)
       (want  0.0)
       (got   (ema-count->f count)))
  (test "ema-count->f" count want got))

(let* ((count 3)
       (want  0.5)
       (got   (ema-count->f count)))
  (test "ema-count->f" count want got))

(let* ((count 7)
       (want  0.75)
       (got   (ema-count->f count)))
  (test "ema-count->f" count want got))

;;
;; ema-warmup-count
;;

(let* ((count 0)
       (want  0)
       (got   (ema-warmup-count count)))
  (test "ema-warmup-count" count want got))

(let* ((count 1)
       (want  0)
       (got   (ema-warmup-count count)))
  (test "ema-warmup-count" count want got))

(let* ((count 2)
       (want  #f)
       (got   (negative? (ema-warmup-count count))))
  (test "ema-warmup-count positive" count want got))

;;
;; ema-calc-proc
;;

(let* ((count 3)
       (want  '(128 64 32 16 8 4))
       (proc  (ema-calc-proc count))
       (got   (begin
		(for-each proc (make-list 100 0))
		(map round-exact
		     (map proc (cons #x100 (make-list 5 0)))))))
  (test "ema-calc-proc" count want got))

;; f==0.75
(let* ((count 7)
       (want  '(1024 768 576 432 324))
       (proc  (ema-calc-proc count))
       (got   (begin
		(for-each proc (make-list 500 0))
		(map round-exact
		     (map proc (cons #x1000 (make-list 4 0)))))))
  (test "ema-calc-proc" count want got))

;; f==0.5, so f+f^2==0.75 is the first with enough weight
(let* ((count 3)
       (want  '(#f 1.0 1.0 1.0 1.0))
       (proc  (ema-calc-proc count))
       (got   (map proc (make-list 5 1.0))))
  (test "ema-calc-proc" count want got))

;; f==0, so get just the current point
(let* ((count 1)
       (want  '(1.0 9.0 7.0 6.0 -123.0))
       (proc  (ema-calc-proc count))
       (got   (map proc want)))
  (test "ema-calc-proc" count want got))



(tests-end)
