;; Test emacs.scm.

;; Copyright 2006, 2007 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(load "tests.scm")
(use-modules (ice-9 rdelim))


;; extract the HAVE_ICONV setting from config.h, as #t or #f
;;
(define HAVE_ICONV
  (call-with-input-file "../config.h"
    (lambda (port)
      (let more ()
	(let ((line (read-line port)))
	  (cond ((eof-object? line)
		 #t)
		((string=? line "/* #undef HAVE_ICONV */")
		 #f)
		(else
		 (more))))))))


;;
;; display-string-escape-dquote
;;

(define display-string-escape-dquote
  (@@ (chart emacs) display-string-escape-dquote))

(malloced-steady
 (lambda ()
   (let* ((str    "")
	  (want   "")
	  (got    (call-with-output-string
		   (lambda (port)
		     (display-string-escape-dquote str port)))))
     (test "display-string-escape-dquote" str want got))))

(malloced-steady
 (lambda ()
   (let* ((str    "hello")
	  (want   "hello")
	  (got    (call-with-output-string
		   (lambda (port)
		     (display-string-escape-dquote str port)))))
     (test "display-string-escape-dquote" str want got))))

(malloced-steady
 (lambda ()
   (let* ((str    "\"hello")
	  (want   "\\\"hello")
	  (got    (call-with-output-string
		   (lambda (port)
		     (display-string-escape-dquote str port)))))
     (test "display-string-escape-dquote" str want got))))

(malloced-steady
 (lambda ()
   (let* ((str    "\"he\\llo\\")
	  (want   "\\\"he\\\\llo\\\\")
	  (got    (call-with-output-string
		   (lambda (port)
		     (display-string-escape-dquote str port)))))
     (test "display-string-escape-dquote" str want got))))

;;
;; el-write
;;

(define el-write
  (@@ (chart emacs) el-write))

(malloced-steady
 (lambda ()
   (let* ((obj  'foo)
	  (want "foo")
	  (got  (call-with-output-string
		 (lambda (port)
		   (el-write obj port)))))
     (test "el-write" obj want got))))

(malloced-steady
 (lambda ()
   (let* ((obj  123)
	  (want "123")
	  (got  (call-with-output-string
		 (lambda (port)
		   (el-write obj port)))))
     (test "el-write" obj want got))))

(malloced-steady
 (lambda ()
   (let* ((obj  "")
	  (want "\"\"")
	  (got  (call-with-output-string
		 (lambda (port)
		   (el-write obj port)))))
     (test "el-write" obj want got))))

(malloced-steady
 (lambda ()
   (let* ((obj  "\\")
	  (want "\"\\\\\"")
	  (got  (call-with-output-string
		 (lambda (port)
		   (el-write obj port)))))
     (test "el-write" obj want got))))

(malloced-steady
 (lambda ()
   (let* ((obj  '(1))
	  (want "(1)")
	  (got  (call-with-output-string
		 (lambda (port)
		   (el-write obj port)))))
     (test "el-write" obj want got))))

(malloced-steady
 (lambda ()
   (let* ((obj  '(1 2))
	  (want "(1 2)")
	  (got  (call-with-output-string
		 (lambda (port)
		   (el-write obj port)))))
     (test "el-write" obj want got))))

(malloced-steady
 (lambda ()
   (let* ((obj  '(1 2 3))
	  (want "(1 2 3)")
	  (got  (call-with-output-string
		 (lambda (port)
		   (el-write obj port)))))
     (test "el-write" obj want got))))

(malloced-steady
 (lambda ()
   (let* ((obj  '(1 2 3 4))
	  (want "(1 2 3 4)")
	  (got  (call-with-output-string
		 (lambda (port)
		   (el-write obj port)))))
     (test "el-write" obj want got))))

(malloced-steady
 (lambda ()
   (let* ((obj  '(1))
	  (want "(1)")
	  (got  (call-with-output-string
		 (lambda (port)
		   (el-write obj port)))))
     (test "el-write" obj want got))))

(malloced-steady
 (lambda ()
   (let* ((obj  '(1 . 2))
	  (want "(1 . 2)")
	  (got  (call-with-output-string
		 (lambda (port)
		   (el-write obj port)))))
     (test "el-write" obj want got))))

(malloced-steady
 (lambda ()
   (let* ((obj  '(1 2 . 3))
	  (want "(1 2 . 3)")
	  (got  (call-with-output-string
		 (lambda (port)
		   (el-write obj port)))))
     (test "el-write" obj want got))))

(malloced-steady
 (lambda ()
   (let* ((obj  '(1 2 3 . 4))
	  (want "(1 2 3 . 4)")
	  (got  (call-with-output-string
		 (lambda (port)
		   (el-write obj port)))))
     (test "el-write" obj want got))))

(malloced-steady
 (lambda ()
   (let* ((obj  '("\\"))
	  (want "(\"\\\\\")")
	  (got  (call-with-output-string
		 (lambda (port)
		   (el-write obj port)))))
     (test "el-write" obj want got))))

(malloced-steady
 (lambda ()
   (let* ((obj  '(123 "\\"))
	  (want "(123 \"\\\\\")")
	  (got  (call-with-output-string
		 (lambda (port)
		   (el-write obj port)))))
     (test "el-write" obj want got))))

(malloced-steady
 (lambda ()
   (let* ((obj  '(123 . "\\"))
	  (want "(123 . \"\\\\\")")
	  (got  (call-with-output-string
		 (lambda (port)
		   (el-write obj port)))))
     (test "el-write" obj want got))))

(malloced-steady
 (lambda ()
   (let* ((obj  '(123 zzz . "\\"))
	  (want "(123 zzz . \"\\\\\")")
	  (got  (call-with-output-string
		 (lambda (port)
		   (el-write obj port)))))
     (test "el-write" obj want got))))

;;
;; utf8-string-pad-right
;;

(define utf8-string-pad-right (@@ (chart emacs) utf8-string-pad-right))

(let* ((str    "hello")
       (target 3)
       (want   "hel")
       (got    (utf8-string-pad-right str target)))
  (test "utf8-string-pad-right" (list str target) want got))

(let* ((str    "hello")
       (target 5)
       (want   "hello")
       (got    (utf8-string-pad-right str target)))
  (test "utf8-string-pad-right" (list str target) want got))

(let* ((str    "hello")
       (target 9)
       (want   "hello    ")
       (got    (utf8-string-pad-right str target)))
  (test "utf8-string-pad-right" (list str target) want got))

(if HAVE_ICONV
    (let* ((str    (string-ints #xE6 #x9C #x88)) ;; japanese month char
	   (target 2)
	   (want   (string-ints #xE6 #x9C #x88 #x20))
	   (got    (utf8-string-pad-right str target)))
      (test "utf8-string-length" str want got)))


(tests-end)
