;; Check FUNC_NAME #defines in .c files.

;; Copyright 2007 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


;; This program looks at all the .c files in "$srcdir/..", ie. the top-level
;; directory, to see that the #define FUNC_NAME defines match the
;; function/name-string symbol given in the preceding SCM_DEFINE.
;;
;; The libguile/guile-func-name-check script in the guile sources does a
;; similar thing.


(load "tests.scm")
(use-modules (ice-9 rdelim)
	     (ice-9 regex))

(define srcdir (getenv "srcdir"))  ;; from the Makefile

(define result 0)

(define c-files (directory-files (string-append srcdir "/..") #t
		  (lambda (filename)
		    (string-suffix? ".c" filename))))

(define-public (string-trim-str str part)
  (if (string-prefix? part str)
      (string-drop str (string-length part))
      str))

(define (check-one filename)
  (call-with-input-file filename
    (lambda (port)
      (define want-name #f)
      (do ((line (read-line port) (read-line port)))
	  ((eof-object? line))
	(cond ((regexp-exec #,(regexp "^SCM_DEFINE[^(]*\\([ \t]*([^, \t]+)")
			    line)
	       => (lambda (m)
		    (set! want-name (match:substring m 1))))
	      ((regexp-exec #,(regexp "^#[ \t]*define[ \t]+FUNC_NAME[ \t]+([a-zA-Z0-9_]+)")
			    line)
	       => (lambda (m)
		    (let ((got-name (string-trim-str (match:substring m 1)
						     "s_")))
		      (or (equal? want-name got-name)
			  (begin
			    (format #t "~a: ~a: ~a wrong\n"
				    filename (port-line port) want-name)
			    (format #t "    ~a\n" line)
			    (set! result 1)))))))))))

(for-each check-one c-files)
(format #t "checked ~a files\n" (length c-files))
(exit result)
