;; Test futures.scm functions.

;; Copyright 2006, 2007 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(load "tests.scm")
(use-modules (chart futures))

;;
;; m/y-str->tdate
;;

(for-each (lambda (elem)
	    (let* ((str  (first elem))
		   (want (second elem))
		   (got  (catch #t
			   (lambda ()
			     (m/y-str->mdate str))
			   (lambda args
			     'error))))
	      (test "m/y-str->tdate" str want got)))

	  `(("Jan 1970"      0)
	    ("January 1970"  0)
	    ("1/1970"        0)
	    ("01/70"         0)
	    ("1.1970"        0)
	    ("Foo 1970"      error)
	    ("30-1970"       error)
	    ("1970/5"        error)))

;;
;; partition-equal
;;

(let* ((lst  '())
       (want '())
       (got  (partition-equal (lambda (str) (string-ref str 0)) lst)))
  (test "partition-equal" lst want got))

(let* ((lst  '("a"))
       (want '(("a")))
       (got  (partition-equal (lambda (str) (string-ref str 0)) lst)))
  (test "partition-equal" lst want got))

(let* ((lst  '("a" "b"))
       (want '(("a") ("b")))
       (got  (partition-equal (lambda (str) (string-ref str 0)) lst)))
  (test "partition-equal" lst want got))

(let* ((lst  '("a1" "b" "a2"))
       (want '(("a1" "a2") ("b")))
       (got  (partition-equal (lambda (str) (string-ref str 0)) lst)))
  (test "partition-equal" lst want got))

(let* ((lst  '("a1" "b1" "a2" "b2"))
       (want '(("a1" "a2") ("b1" "b2")))
       (got  (partition-equal (lambda (str) (string-ref str 0)) lst)))
  (test "partition-equal" lst want got))

(let* ((lst  '("a1" "b1" "a2" "c1" "b2" "c2"))
       (want '(("a1" "a2") ("b1" "b2") ("c1" "c2")))
       (got  (partition-equal (lambda (str) (string-ref str 0)) lst)))
  (test "partition-equal" lst want got))


(tests-end)
