;; Test gnome.scm.

;; Copyright 2006, 2007 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(load "tests.scm")
(need-gtk)
(use-modules (chart gnome))


;;
;; gconf-client-get-string
;;

(define gconf-client-get-string (@@ (chart gnome) gconf-client-get-string))

(let* ((client #f)
       (key    "totallyboguskey")
       (want   'g-error)
       (got    (catch 'g-error
		 (lambda ()
		   (gconf-client-get-string client key))
		 (lambda args
		   'g-error))))
  (test "gconf-client-get-string" (list client key) want got))

;;
;; ghelp-url->file-url
;;

(define ghelp-url->file-url (@@ (chart gnome) ghelp-url->file-url))

(let* ((url  "ghelp:/foo/bar.html")
       (want "file:///foo/bar.html")
       (got  (ghelp-url->file-url url)))
  (test "ghelp-url->file-url" url want got))

(let* ((url  "ghelp:/foo/bar.html?section")
       (want "file:///foo/bar.html#section")
       (got  (ghelp-url->file-url url)))
  (test "ghelp-url->file-url" url want got))

;;
;; gnome-execute-async
;;

(define (positive-exact-integer? obj)
  (and (number? obj)
       (exact? obj)
       (integer? obj)))

(let* ((args '(#f "nosuchprogram"))
       (want -1)
       (got  (apply gnome-execute-async args)))
  (test "gnome-execute-async" args want got))

(let* ((args '(#f "exit" "0"))
       (want #t)
       (got  (positive-exact-integer? (apply gnome-execute-async args))))
  (test "gnome-execute-async" args want got))

(let* ((args '("/tmp" "nosuchprogram"))
       (want -1)
       (got  (apply gnome-execute-async args)))
  (test "gnome-execute-async" args want got))

(let* ((args '("/tmp" "exit" "0"))
       (want #t)
       (got  (positive-exact-integer? (apply gnome-execute-async args))))
  (test "gnome-execute-async" args want got))

(false-if-exception (delete-file "gnome.tmp"))
(let* ((args '(#f "touch" "gnome.tmp"))
       (pid  (apply gnome-execute-async args))
       (want #t)
       (got  (and (positive-exact-integer? pid)
		  (begin
		    (sleep 2)
		    (file-exists? "gnome.tmp")))))
  (test "gnome-execute-async" args want got))
(delete-file-maybe "gnome.tmp")


;;
;; string-substitute-%s
;;

(define string-substitute-%s (@@ (chart gnome) string-substitute-%s))

(let* ((str  "foo %s bar")
       (repl "quux")
       (want "foo quux bar")
       (got  (string-substitute-%s str repl)))
  (test "string-substitute-%s" (list str repl) want got))

(let* ((str  "foo bar")
       (repl "quux")
       (want "foo bar")
       (got  (string-substitute-%s str repl)))
  (test "string-substitute-%s" (list str repl) want got))

;;
;; url-scheme
;;

(define url-scheme (@@ (chart gnome) url-scheme))

(let* ((url  "http://foo.com/index.html")
       (want "http")
       (got  (url-scheme url)))
  (test "url-scheme" url want got))

(let* ((url  "file:///")
       (want "file")
       (got  (url-scheme url)))
  (test "url-scheme" url want got))

(let* ((url  "f&&&ile:///")
       (want #f)
       (got  (url-scheme url)))
  (test "url-scheme" url want got))

(let* ((url  "gconf/something")
       (want #f)
       (got  (url-scheme url)))
  (test "url-scheme" url want got))



(tests-end)
