;; Exercise graph.scm.

;; Copyright 2005, 2006, 2007 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(load "tests.scm")
(need-gtk)
(use-modules (chart graph))


;;
;; array-contents->list
;;

(define array-contents->list (@@ (chart graph) array-contents->list))

(let* ((array  #(1 2 3))
       (want   '(1 2 3))
       (got    (array-contents->list array)))
  (test "array-contents->list" array want got))

(let* ((array  #2((1 2) (3 4)))
       (want   '(1 2 3 4))
       (got    (array-contents->list array)))
  (test "array-contents->list" array want got))

;;
;; dividend->string
;;

(let* ((div  '(amount 123 (decimals . 2)))
       (want (string-append "1" (locale-decimal-point) "23"))
       (got  (dividend->string div)))
  (test "dividend->string" div want got))

;; not sure about displaying imputation
;; (let* ((div  '(amount 123 (decimals . 2) (imputation . "zz")))
;;        (want (string-append "1" (locale-decimal-point) "23 + zz"))
;;        (got  (dividend->string div)))
;;   (test "dividend->string" div want got))

(let* ((div  '(amount unknown))
       (want (_ "Unknown"))
       (got  (dividend->string div)))
  (test "dividend->string" div want got))

(let* ((div  '(amount to-be-advised))
       (want (_ "To be advised"))
       (got  (dividend->string div)))
  (test "dividend->string" div want got))

;;
;; interpolate
;;

(define interpolate (@@ (chart line) interpolate))

(for-each (lambda (elem)
	    (let* ((args (first elem))
		   (want (second elem))
		   (got  (apply interpolate args)))
	      (test "interpolate" args want got)))
	  '(((0 0   10 10  5) 5)
	    ((10 10   0 0  5) 5)
	    ((0 0   20 10  5) 10)
	    ((20 10   0 0  5) 10)

	    ((0 6  800 -2  -1) 700)))

;;
;; overlap-exclusive?
;;

(define overlap-exclusive? (@@ (chart graph) overlap-exclusive?))

(for-each (lambda (elem)
	    (let* ((args (first elem))
		   (want (second elem))
		   (got  (->bool (apply overlap-exclusive? args))))
	      (test "overlap-exclusive?" args want got)))
	  '(((5 6 8 9) #f)
	    ((5 6 6 9) #f)
	    ((5 7 6 9) #t)
	    ((7 8 6 9) #t)
	    ((7 10 6 9) #t)
	    ((10 10 6 9) #f)))


(tests-end)
