;; Test gzip.scm functions.

;; Copyright 2006, 2007 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(load "tests.scm")
(use-modules (chart gzip))

(define (string-or-f? obj)
  (or (string? obj)
      (eq? #f obj)))

;;
;; gzip-version
;;

(let* ((want #t)
       (got  (string-or-f? (gzip-version))))
  (test "gzip-version" "string-or-f?" want got))

;;
;; gunzip-string
;;

(if (gzip-version)
    (begin
      (let* ((str  (list->string
		    (map integer->char
			 '(31 139 8 8 227 193 248 65 2 3 120 0 3
			      0 0 0 0 0 0 0 0 0))))
	     (want "")
	     (got  (gunzip-string str)))
	(test "gunzip-string" str want got))

      (let* ((str  (list->string
		    (map integer->char
			 '(31 139 8 8 231 194 248 65 2 3 120 0 203 72 205 201
			      201 231 2 0 32 48 58 54 6 0 0 0))))
	     (want "hello\n")
	     (got  (gunzip-string str)))
	(test "gunzip-string" str want got))))

;;
;; program-filter-string
;;

(let* ((str     "hello")
       (command "cat")
       (want    "hello")
       (got     (program-filter-string command str)))
  (test "program-filter-string" (list command str) want got))

(let* ((str     "hello")
       (command "cat")
       (want    "hello")
       (got     (program-filter-string command str #:ext ".txt")))
  (test "program-filter-string" (list command str) want got))

(let* ((str     "hello")
       (command "echo")
       (want    #t)
       (got     (string-suffix? ".txt\n"
				(program-filter-string command str
						       #:ext ".txt"))))
  (test "program-filter-string" (list command str) want got))

