;; Test html.scm functions.

;; Copyright 2004, 2005, 2006, 2007 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(load "tests.scm")
(use-modules (chart html))


;;
;; html-charset
;;

(let* ((headers "")
       (str     "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=iso-8859-1\">")
       (want    "iso-8859-1")
       (got     (html-charset headers str)))
  (test "html-charset" (list headers str) want got))

(let* ((headers "")
       (str     "<meta http-equiv='content-Type' content='text/html; charset=iso-8859-1'/>")
       (want    "iso-8859-1")
       (got     (html-charset headers str)))
  (test "html-charset" (list headers str) want got))

(let* ((headers "HTTP/1.0 200 Ok\r\nContent-Type: text/html; charset=utf-8\r\n\r\n")
       (str     "<html></html>")
       (want    "utf-8")
       (got     (html-charset headers str)))
  (test "html-charset" (list headers str) want got))

(let* ((headers "HTTP/1.0 200 Ok\r\nContent-Type: text/html; charset=utf-8; foo=bar\r\n\r\n")
       (str     "<html></html>")
       (want    "utf-8")
       (got     (html-charset headers str)))
  (test "html-charset" (list headers str) want got))

;;
;; html-skip-to-</table>
;;

(define html-skip-to-</table> (@@ (chart html) html-skip-to-</table>))

(let* ((str  "blah</table>x")
       (want (string-index str #\x))
       (got  (html-skip-to-</table> str 0)))
  (test "html-skip-to-</table>" str want got))

(let* ((str  "blah<table>blah</table></table>x")
       (want (string-index str #\x))
       (got  (html-skip-to-</table> str 0)))
  (test "html-skip-to-</table>" str want got))

(let* ((str  "blah<table>blah</TABLE></table>x")
       (want (string-index str #\x))
       (got  (html-skip-to-</table> str 0)))
  (test "html-skip-to-</table>" str want got))

(let* ((str  "blah<table>blah</table>")
       (want (string-length str))
       (got  (html-skip-to-</table> str 0)))
  (test "html-skip-to-</table>" str want got))

(let* ((str  "blah<table>blah")
       (want (string-length str))
       (got  (html-skip-to-</table> str 0)))
  (test "html-skip-to-</table>" str want got))

;;
;; html-table-rows-raw
;;

(let* ((str  "<table>")
       (want '())
       (got  (html-table-rows-raw str)))
  (test "html-table-rows-raw" str want got))

(let* ((str  "<table></table>")
       (want '())
       (got  (html-table-rows-raw str)))
  (test "html-table-rows-raw" str want got))

(let* ((str  "<table><tr>")
       (want '(()))
       (got  (html-table-rows-raw str)))
  (test "html-table-rows-raw" str want got))

(let* ((str  "<table><tr></table>")
       (want '(()))
       (got  (html-table-rows-raw str)))
  (test "html-table-rows-raw" str want got))

(let* ((str  "<table><td>foo")
       (want '(("<td>foo")))
       (got  (html-table-rows-raw str)))
  (test "html-table-rows-raw" str want got))

(let* ((str  "<table><tr><td>foo")
       (want '(("<td>foo")))
       (got  (html-table-rows-raw str)))
  (test "html-table-rows-raw" str want got))

(let* ((str  "<table><tr><th>foo")
       (want '(("<th>foo")))
       (got  (html-table-rows-raw str)))
  (test "html-table-rows-raw" str want got))

(let* ((str  "<table><tr><th>foo</th>")
       (want '(("<th>foo")))
       (got  (html-table-rows-raw str)))
  (test "html-table-rows-raw" str want got))

(let* ((str  "<table><tr><th>foo</tr>")
       (want '(("<th>foo")))
       (got  (html-table-rows-raw str)))
  (test "html-table-rows-raw" str want got))

(let* ((str  "<table><tr><th>foo</table>")
       (want '(("<th>foo")))
       (got  (html-table-rows-raw str)))
  (test "html-table-rows-raw" str want got))

(let* ((str  "<table><tr><th>foo</table> <tr><td>bar</td></tr>")
       (want '(("<th>foo")))
       (got  (html-table-rows-raw str)))
  (test "html-table-rows-raw" str want got))

(let* ((str  "<table><tr><th>foo</table> <table><tr><td>bar</td></tr>")
       (want '(("<th>foo")))
       (got  (html-table-rows-raw str)))
  (test "html-table-rows-raw" str want got))

(let* ((str  "<table><td>foo<td>bar")
       (want '(("<td>foo" "<td>bar")))
       (got  (html-table-rows-raw str)))
  (test "html-table-rows-raw" str want got))

(let* ((str  "<table><td>foo</td><td>bar")
       (want '(("<td>foo" "<td>bar")))
       (got  (html-table-rows-raw str)))
  (test "html-table-rows-raw" str want got))

(let* ((str  "<table><td>foo</td><td>bar</td>")
       (want '(("<td>foo" "<td>bar")))
       (got  (html-table-rows-raw str)))
  (test "html-table-rows-raw" str want got))

(let* ((str  "<table><td>foo<tr><td>bar")
       (want '(("<td>foo") ("<td>bar")))
       (got  (html-table-rows-raw str)))
  (test "html-table-rows-raw" str want got))

(let* ((str  "<table><td>foo<table></table> <tr><td>bar")
       (want '(("<td>foo<table></table> ") ("<td>bar")))
       (got  (html-table-rows-raw str)))
  (test "html-table-rows-raw" str want got))

(let* ((str  "<table><tr><td>foo</td></tr> <!-- <tr><td>foo</td></tr> --> </table>")
       (want '(("<td>foo")))
       (got  (html-table-rows-raw str)))
  (test "html-table-rows-raw" str want got))

(let* ((str  "<table><!-- <tr><td>foo</td></tr> <tr><td>foo</td></tr> </table>")
       (want '())
       (got  (html-table-rows-raw str)))
  (test "html-table-rows-raw" str want got))

(let* ((str  "<!-- blah -->   <table><tr><td>foo</td></tr> <tr><td>bar</td></tr> </table>")
       (want '(("<td>foo") ("<td>bar")))
       (got  (html-table-rows-raw str)))
  (test "html-table-rows-raw" str want got))

(let* ((str  "<table><tr><td>foo <!-- blah --> </td></tr> <tr><td>bar</td></tr> </table>")
       (want '(("<td>foo <!-- blah --> ") ("<td>bar")))
       (got  (html-table-rows-raw str)))
  (test "html-table-rows-raw" str want got))

;;
;; html-text-extract
;;

(let* ((str  "")
       (want "")
       (got  (html-text-extract str)))
  (test "html-text-extract" str want got))

(let* ((str  "<foo>")
       (want "")
       (got  (html-text-extract str)))
  (test "html-text-extract" str want got))

(let* ((str  "x<foo>")
       (want "x")
       (got  (html-text-extract str)))
  (test "html-text-extract" str want got))

(let* ((str  "<foo>x")
       (want "x")
       (got  (html-text-extract str)))
  (test "html-text-extract" str want got))

(let* ((str  "<foo>x</foo>")
       (want "x")
       (got  (html-text-extract str)))
  (test "html-text-extract" str want got))

(let* ((str  "a<foo>b</foo>c")
       (want "abc")
       (got  (html-text-extract str)))
  (test "html-text-extract" str want got))

(let* ((str  "<foo")
       (want "")
       (got  (html-text-extract str)))
  (test "html-text-extract" str want got))

(let* ((str  "a<foo")
       (want "a")
       (got  (html-text-extract str)))
  (test "html-text-extract" str want got))

(let* ((str  "a<foo></foo")
       (want "a")
       (got  (html-text-extract str)))
  (test "html-text-extract" str want got))

(let* ((str  "&nbsp;")
       (want " ")
       (got  (html-text-extract str)))
  (test "html-text-extract" str want got))

(let* ((str  "XC&nbsp;XD")
       (want "XC XD")
       (got  (html-text-extract str)))
  (test "html-text-extract" str want got))

(let* ((str  "&ndash;X")
       (want "-X")
       (got  (html-text-extract str)))
  (test "html-text-extract" str want got))

(let* ((str  "&nbsp")
       (want " ")
       (got  (html-text-extract str)))
  (test "html-text-extract" str want got))

(let* ((str  "x&nbsp")
       (want "x ")
       (got  (html-text-extract str)))
  (test "html-text-extract" str want got))

(let* ((str  "XC&nbsp<FONT>")
       (want "XC ")
       (got  (html-text-extract str)))
  (test "html-text-extract" str want got))

(let* ((str  "&ndash;+")
       (want "-+")
       (got  (html-text-extract str)))
  (test "html-text-extract" str want got))

(let* ((str  "&ndashity")
       (want "&ndashity")
       (got  (html-text-extract str)))
  (test "html-text-extract" str want got))

;; rfc1866 example
(let* ((str  "abc &ndash def")
       (want "abc - def")
       (got  (html-text-extract str)))
  (test "html-text-extract" str want got))

;; rfc1866 example
(let* ((str  "abc & ndash def")
       (want "abc & ndash def")
       (got  (html-text-extract str)))
  (test "html-text-extract" str want got))


(tests-end)
