;; Exercise indicator.scm.

;; Copyright 2006, 2007 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(load "tests.scm")
(use-modules (chart indicator))


;;
;; bsearch-first-true
;;

(do ((len 1 (1+ len)))
    ((> len 8))

  (let ((data (make-vector len #t)))
    (define (pred i)
      (if (< i len)
	  (vector-ref data i)
	  #t))

    (do ((want 0 (1+ want)))
	((>= want len))

      (let ((got (bsearch-first-true pred 3)))
	(test "bsearch-first-true" data want got))

      (vector-set! data want #f))))

;;
;; list-ref-fraction
;;

(let* ((lst  '(1 2 3))
       (f    0)
       (want 1)
       (got  (list-ref-fraction lst f)))
  (test "list-ref-fraction" (list f lst) want got))

(let* ((lst  '(1 2 3))
       (f    0.1)
       (want 1)
       (got  (list-ref-fraction lst f)))
  (test "list-ref-fraction" (list f lst) want got))

(let* ((lst  '(1 2 3))
       (f    0.5)
       (want 2)
       (got  (list-ref-fraction lst f)))
  (test "list-ref-fraction" (list f lst) want got))

(let* ((lst  '(1 2 3))
       (f    0.9)
       (want 3)
       (got  (list-ref-fraction lst f)))
  (test "list-ref-fraction" (list f lst) want got))

(let* ((lst  '(1 2 3))
       (f    1.0)
       (want 3)
       (got  (list-ref-fraction lst f)))
  (test "list-ref-fraction" (list f lst) want got))

;;
;; locale-number-min-decimals
;;

(let* ((x        1.0)
       (decimals 0)
       (want     "1")
       (got      (locale-number-min-decimals x decimals)))
  (test "locale-number-min-decimals" (list x decimals) want got))

(let* ((x        1.0)
       (decimals 1)
       (want     "1.0")
       (got      (locale-number-min-decimals x decimals)))
  (test "locale-number-min-decimals" (list x decimals) want got))

(let* ((x        1.0)
       (decimals 2)
       (want     "1.00")
       (got      (locale-number-min-decimals x decimals)))
  (test "locale-number-min-decimals" (list x decimals) want got))

(let* ((x        1.25)
       (decimals 0)
       (want     "1.25")
       (got      (locale-number-min-decimals x decimals)))
  (test "locale-number-min-decimals" (list x decimals) want got))

(let* ((x        1.25)
       (decimals 1)
       (want     "1.25")
       (got      (locale-number-min-decimals x decimals)))
  (test "locale-number-min-decimals" (list x decimals) want got))

(let* ((x        1.25)
       (decimals 2)
       (want     "1.25")
       (got      (locale-number-min-decimals x decimals)))
  (test "locale-number-min-decimals" (list x decimals) want got))

;;
;; prevnth-calc-proc
;;

(let* ((count  0)
       (proc   (prevnth-calc-proc count))
       (data   '(1 2 3 4 5))
       (want   '(1 2 3 4 5))
       (got    (map-in-order proc data)))
  (test "prevnth-calc-proc" (list count data) want got))

(let* ((count  1)
       (proc   (prevnth-calc-proc count))
       (data   '(   1 2 3 4 5))
       (want   '(#f 1 2 3 4))
       (got    (map-in-order proc data)))
  (test "prevnth-calc-proc" (list count data) want got))

(let* ((count  2)
       (proc   (prevnth-calc-proc count))
       (data   '(      1 2 3 4 5))
       (want   '(#f #f 1 2 3))
       (got    (map-in-order proc data)))
  (test "prevnth-calc-proc" (list count data) want got))

;;
;; series-array-closes
;;

(let* ((series (series-new #:array (list->array 2 '((1 2 3 4 5)
						    (6 7 8 9 10)))
			   #:datatype datatype-ohlcv))
       (want   '(4 9))
       (got    (array->list (series-array-closes series 0 1))))
  (test "series-array-closes" series want got))

;;
;; series-find-close
;;

(let* ((data       '(0 10 20 30 40))
       (t          2)
       (want       2)
       (got        (series-find-close (series-new #:array (list->array 1 data)) t)))
  (test "series-find-close" (list t data) want got))

(let* ((data       '(0 10 #f 30 40))
       (t          2)
       (want       3)
       (got        (series-find-close (series-new #:array (list->array 1 data)) t)))
  (test "series-find-close" (list t data) want got))

(let* ((data       '(0 10 #f #f #f))
       (t          2)
       (want       1)
       (got        (series-find-close (series-new #:array (list->array 1 data)) t)))
  (test "series-find-close" (list t data) want got))

;;
;; series-prev-n-closes
;;

(let* ((data   (array-from 0  10 30))
       (series (series-new #:array data))
       (want   '(10))
       (got    (series-prev-n-closes series #:before 1 #:count 1)))
  (test "series-prev-n-closes" data want got))

(let* ((data   (array-from 0  10 20 30 40))
       (series (series-new #:array data))
       (want   '(10 20))
       (got    (series-prev-n-closes series #:before 2 #:count 2)))
  (test "series-prev-n-closes" data want got))

(let* ((data   (array-from 0  10 20 30 40))
       (series (series-new #:array data))
       (want   '(30 40))
       (got    (series-prev-n-closes series #:before 1000 #:count 2)))
  (test "series-prev-n-closes" data want got))

(let* ((data   (array-from 0  10 20 30 40))
       (series (series-new #:array data))
       (want   '())
       (got    (series-prev-n-closes series #:before -999 #:count 2)))
  (test "series-prev-n-closes" data want got))


;;
;; series-prev-n-closes-pos
;;

(let* ((data   (array-from 0  10 30))
       (series (series-new #:array data))
       (args   '(#:before 1 #:count 1))
       (want   0)
       (got    (apply series-prev-n-closes-pos series args)))
  (test "series-prev-n-closes-pos" (list data args) want got))

(let* ((data   (array-from 0  12 22 32 42))
       (series (series-new #:array data))
       (want   0)
       (got    (series-prev-n-closes-pos series #:before 2 #:count 2)))
  (test "series-prev-n-closes-pos" data want got))

(let* ((data   (array-from 0  13 23 33 43))
       (series (series-new #:array data))
       (want   1)
       (got    (series-prev-n-closes-pos series #:before 2 #:count 1)))
  (test "series-prev-n-closes-pos" data want got))

(let* ((data   (array-from 0  14 24 34 44))
       (series (series-new #:array data))
       (want   2)
       (got    (series-prev-n-closes-pos series #:before 1000 #:count 2)))
  (test "series-prev-n-closes-pos" data want got))

(let* ((data   (array-from 0  15 25 35 45))
       (series (series-new #:array data))
       (want   0)
       (got    (series-prev-n-closes-pos series #:before -999 #:count 2)))
  (test "series-prev-n-closes-pos" data want got))

(let* ((data   (array-from 0  7 8 9))
       (series (series-new #:array data))
       (want   0)
       (got    (series-prev-n-closes-pos series #:before -999 #:count 0)))
  (test "series-prev-n-closes-pos" data want got))

(let* ((data   (array-from 0  9 8 7))
       (series (series-new #:array data))
       (want   2)
       (got    (series-prev-n-closes-pos series #:before 100 #:count 0)))
  (test "series-prev-n-closes-pos" data want got))

(let* ((data   (array-from 0  3 2 1))
       (args   '(#:before 1 #:count 0))
       (series (series-new #:array data))
       (want   1)
       (got    (apply series-prev-n-closes-pos series args)))
  (test "series-prev-n-closes-pos" (list data args) want got))

(let* ((data   (array-from 0  1 #f 2 #f #f #f #f #f #f #f #f #f #f #f #f 1))
       (args   '(#:before 100 #:count 2))
       (series (series-new #:array data))
       (want   2)
       (got    (apply series-prev-n-closes-pos series args)))
  (test "series-prev-n-closes-pos" (list data args) want got))

(let* ((data   (apply array-from 0  123 #f 456 #f 789 (make-list 1000 #f)))
       (args   '(#:before 100 #:count 2))
       (series (series-new #:array data))
       (want   2)
       (got    (apply series-prev-n-closes-pos series args)))
  (test "series-prev-n-closes-pos" (list data args) want got))


(tests-end)
