;; Test LME functions.

;; Copyright 2005, 2007 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(load "tests.scm")


;;
;; daily-available-tdate
;;

(define daily-available-tdate (@@ (chart lme) daily-available-tdate))

(let ((ymdhm  '(2005 1 6  18 0))) ;; Thursday late
  (apply fake-localtime-ymdhm! (timezone-london) ymdhm)
  (let ((want (ymd->tdate 2005 1 6))
	(got  (daily-available-tdate 'metals)))
    (test "daily-available-tdate" ymdhm want got)))

(let ((ymdhm  '(2005 1 7  12 0)))  ;; Friday early
  (apply fake-localtime-ymdhm! (timezone-london) ymdhm)
  (let ((want (ymd->tdate 2005 1 6))
	(got  (daily-available-tdate 'metals)))
    (test "daily-available-tdate" ymdhm want got)))

(let ((ymdhm  '(2005 1 7  18 0))) ;; Friday late
  (apply fake-localtime-ymdhm! (timezone-london) ymdhm)
  (let ((want (ymd->tdate 2005 1 7))
	(got  (daily-available-tdate 'metals)))
    (test "daily-available-tdate" ymdhm want got)))

(let ((ymdhm  '(2005 1 8  12 0)))  ;; Saturday early
  (apply fake-localtime-ymdhm! (timezone-london) ymdhm)
  (let ((want (ymd->tdate 2005 1 7))
	(got  (daily-available-tdate 'metals)))
    (test "daily-available-tdate" ymdhm want got)))

(let ((ymdhm  '(2005 1 8  18 0))) ;; Saturday late
  (apply fake-localtime-ymdhm! (timezone-london) ymdhm)
  (let ((want (ymd->tdate 2005 1 7))
	(got  (daily-available-tdate 'metals)))
    (test "daily-available-tdate" ymdhm want got)))

(let ((ymdhm  '(2005 1 9  12 0)))  ;; Sunday early
  (apply fake-localtime-ymdhm! (timezone-london) ymdhm)
  (let ((want (ymd->tdate 2005 1 7))
	(got  (daily-available-tdate 'metals)))
    (test "daily-available-tdate" ymdhm want got)))

(let ((ymdhm  '(2005 1 9  18 0))) ;; Sunday late
  (apply fake-localtime-ymdhm! (timezone-london) ymdhm)
  (let ((want (ymd->tdate 2005 1 7))
	(got  (daily-available-tdate 'metals)))
    (test "daily-available-tdate" ymdhm want got)))

(let ((ymdhm  '(2005 1 10  12 0)))  ;; Monday early
  (apply fake-localtime-ymdhm! (timezone-london) ymdhm)
  (let ((want (ymd->tdate 2005 1 7))
	(got  (daily-available-tdate 'metals)))
    (test "daily-available-tdate" ymdhm want got)))

(let ((ymdhm  '(2005 1 10  18 0))) ;; Monday late
  (apply fake-localtime-ymdhm! (timezone-london) ymdhm)
  (let ((want (ymd->tdate 2005 1 10))
	(got  (daily-available-tdate 'metals)))
    (test "daily-available-tdate" ymdhm want got)))

(let ((ymdhm  '(2005 1 11  12 0))) ;; Tuesday early
  (apply fake-localtime-ymdhm! (timezone-london) ymdhm)
  (let ((want (ymd->tdate 2005 1 10))
	(got  (daily-available-tdate 'metals)))
    (test "daily-available-tdate" ymdhm want got)))

(let ((ymdhm  '(2005 1 11  18 0))) ;; Tuesday late
  (apply fake-localtime-ymdhm! (timezone-london) ymdhm)
  (let ((want (ymd->tdate 2005 1 11))
	(got  (daily-available-tdate 'metals)))
    (test "daily-available-tdate" ymdhm want got)))

(let ((ymdhm  '(2005 1 12  12 0))) ;; Wednesday early
  (apply fake-localtime-ymdhm! (timezone-london) ymdhm)
  (let ((want (ymd->tdate 2005 1 11))
	(got  (daily-available-tdate 'metals)))
    (test "daily-available-tdate" ymdhm want got)))

(let ((ymdhm  '(2005 1 12  18 0))) ;; Wednesday late
  (apply fake-localtime-ymdhm! (timezone-london) ymdhm)
  (let ((want (ymd->tdate 2005 1 12))
	(got  (daily-available-tdate 'metals)))
    (test "daily-available-tdate" ymdhm want got)))

(let ((ymdhm  '(2005 1 13  12 0))) ;; Thursday early
  (apply fake-localtime-ymdhm! (timezone-london) ymdhm)
  (let ((want (ymd->tdate 2005 1 12))
	(got  (daily-available-tdate 'metals)))
    (test "daily-available-tdate" ymdhm want got)))

;;
;; daily-heading->commodity+name
;;

(define daily-heading->commodity+name
  (@@ (chart lme) daily-heading->commodity+name))

(for-each (lambda (elem)
	    (let* ((heading (first elem))
		   (want    (second elem))
		   (got     (daily-heading->commodity+name heading)))
	      (test "daily-heading->commodity+name" heading want got)))
	  '(("COPPER"  ("COPPER" #f))
	    ("ALUMINIUM ALLOY"  ("ALUMINIUM ALLOY" #f))
	    ("PP  Global"                 ("PP" "Global")) ;; eg. 2 Jul 07
	    ("PP\r\n\t\t\t\r\n\t\tGlobal" ("PP" "Global")) ;; eg. 13 Jul 07
	    ("PA\r\n\t\t\t\r\n\t\tAsia"   ("PA" "Asia"))   ;; eg. 30 Jul 07
	    ))

;;
;; volumexls-url->mdate
;;

(define volumexls-url->mdate (@@ (chart lme) volumexls-url->mdate))

(let* ((url  "http://www.lme.co.uk/downloads/volumes_Jan_07.xls")
       (want (ymd->mdate 2007 1 1))
       (got  (volumexls-url->mdate url)))
  (test "volumexls-url->mdate" url want got))


(tests-end)
