;; Test manual references in the sources.

;; Copyright 2005, 2006, 2007 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(load "tests.scm")
(need-gtk)
(use-modules (ice-9 rdelim)
	     (ice-9 regex)
	     (chart extra-glib))

(define srcdir (getenv "srcdir"))  ;; from the Makefile
(define result 0)


;;
;; "@node Foo" lines and "@anchor{Foo}" directives from chart.texi
;;

(define manual-node-list '())

(with-input-from-file (g-build-filename srcdir "../doc/chart.texi")
  (lambda ()
    (do ((line (read-line) (read-line)))
	((eof-object? line))
      (and-let* ((m (regexp-exec #,(regexp "^@node ([^,]+)|@anchor\\{([^}]+)")
				 line)))
	(set! manual-node-list (cons (or (match:substring m 1)
					 (match:substring m 2))
				     manual-node-list))))))

;;
;; "<a name="Foo">" anchors from chart.html
;;

(define html-anchor-list '())

;; chart.html is in the builddir when developing, but in srcdir in a dist
(define chart-html-filename
  (find file-exists?
	(list "../doc/chart.html"
	      (g-build-filename (getenv "srcdir") "../doc/chart.html"))))

(with-input-from-file chart-html-filename
  (lambda ()
    (do ((line (read-line) (read-line)))
	((eof-object? line))
      (for-each (lambda (m)
		  (set! html-anchor-list (cons (match:substring m 1)
					       html-anchor-list)))
		(list-matches #,(regexp "<a name=\"([^\"]+)\">") line)))))


;;
;; "CONTEXT(manual-node): Foo" strings from sources
;;

(define texinfo-node->html-anchor
  (@@ (chart gui-common) texinfo-node->html-anchor))

(define source-references-count 0)

(define (process-node name)
  (set! source-references-count (1+ source-references-count))
  (if (not (member name manual-node-list))
      (begin
	(set! result 1)
	(format #t "~a: no such texinfo node: ~a\n"
		(port-filename (current-input-port)) name)))

  (let ((anchor (texinfo-node->html-anchor name)))
    (if (not (member anchor html-anchor-list))
	(begin
	  (set! result 1)
	  (format #t "~a: no such HTML anchor: ~a\n"
		  (port-filename (current-input-port)) anchor)))))

(define (process-expr expr)
  (cond ((string? expr)
	 (if (string-prefix? "CONTEXT(manual-node):" expr)
	     (process-node (string-drop expr 22))))
	((list? expr)
	 (for-each process-expr expr))))

(define (process-file filename)
  (with-input-from-file filename
    (lambda ()
      (lazy-catch #t
	(lambda ()
	  (do ((obj (read) (read)))
	      ((eof-object? obj))
	    (process-expr obj)))
	(lambda args
	  (format #t "Error processing ~a\n" filename)
	  (apply throw args))))))

(for-each process-file
	  (directory-files (g-build-filename srcdir "..") #t
			   (lambda (path)
			     (string-suffix? ".scm" path))))
(process-file (g-build-filename srcdir ".." "chart.in"))


;;
;; datatypes without #:manual
;;

(for-each (lambda (datatype)
	    (or (datatype-manual datatype)              ;; got a manual link
		(eq? 'none (datatype-key datatype))     ;; nothing for "None"
		(<= (datatype-priority datatype) -1000) ;; low priority stuff
		(format #t "datatype ~a: no manual link\n"
			(datatype-key datatype))))
	  (datatype-list-all))


;;
;;
;;

(format #t "chart.texi ~a nodes, source code ~a references\n"
	(length manual-node-list) source-references-count)
(exit result)
