;; Test MGEX functions.

;; Copyright 2005, 2007 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(load "tests.scm")


;;
;; aquote-symbol?
;;

(define aquote-symbol? (@@ (chart mgex) aquote-symbol?))

(let* ((symbol "MW.MGEX")
       (want   #f)
       (got    (->bool (aquote-symbol? symbol))))
  (test "aquote-symbol?" symbol want got))

(let* ((symbol "ICK06.MGEX")
       (want   #t)
       (got    (->bool (aquote-symbol? symbol))))
  (test "aquote-symbol?" symbol want got))

;;
;; mgex-front-mdate
;;

(define mgex-front-mdate (@@ (chart mgex) mgex-front-mdate))

;; January 2007

(let ((ymd '(2007 1 14)))  ;; Sunday
  (fake-localtime-ymdhm! (timezone-chicago)
			 (first ymd) (second ymd) (third ymd) 12 0)
  (let* ((want (ymd->mdate 2007 1 1))
	 (got  (mgex-front-mdate)))
    (test "mgex-front-mdate" ymd want got)))

(let ((ymd '(2007 1 15)))  ;; Monday
  (fake-localtime-ymdhm! (timezone-chicago)
			 (first ymd) (second ymd) (third ymd) 12 0)
  (let* ((want (ymd->mdate 2007 2 1))
	 (got  (mgex-front-mdate)))
    (test "mgex-front-mdate" ymd want got)))

;; September 2007

(let ((ymd '(2007 9 14)))  ;; Friday
  (fake-localtime-ymdhm! (timezone-chicago)
			 (first ymd) (second ymd) (third ymd) 12 0)
  (let* ((want (ymd->mdate 2007 9 1))
	 (got  (mgex-front-mdate)))
    (test "mgex-front-mdate" ymd want got)))

(let ((ymd '(2007 9 15)))  ;; Saturday
  (fake-localtime-ymdhm! (timezone-chicago)
			 (first ymd) (second ymd) (third ymd) 12 0)
  (let* ((want (ymd->mdate 2007 9 1))
	 (got  (mgex-front-mdate)))
    (test "mgex-front-mdate" ymd want got)))

(let ((ymd '(2007 9 16)))  ;; Sunday
  (fake-localtime-ymdhm! (timezone-chicago)
			 (first ymd) (second ymd) (third ymd) 12 0)
  (let* ((want (ymd->mdate 2007 9 1))
	 (got  (mgex-front-mdate)))
    (test "mgex-front-mdate" ymd want got)))

(let ((ymd '(2007 9 17)))  ;; Monday
  (fake-localtime-ymdhm! (timezone-chicago)
			 (first ymd) (second ymd) (third ymd) 12 0)
  (let* ((want (ymd->mdate 2007 10 1))
	 (got  (mgex-front-mdate)))
    (test "mgex-front-mdate" ymd want got)))

(fake-time-end)

;;
;; mgex-quote-adate-time
;;

(define mgex-quote-adate-time (@@ (chart mgex) mgex-quote-adate-time))

(let ((ymdhms '(2005 1 7  6 0 0)))  ;; Friday 6:00:00
  (apply fake-localtime-ymdhm! (timezone-chicago) ymdhms)
  (let* ((symbol "MW.MGEX")
	 (want   (list (ymd->adate 2005 1 6)
		       (hms->seconds 13 15 0)))
	 (got    (mgex-quote-adate-time symbol)))
    (test "mgex-quote-adate-time" (list symbol ymdhms) want got)))

(let ((ymdhms '(2005 1 7  12 0 0)))  ;; Friday 12:00:00
  (apply fake-localtime-ymdhm! (timezone-chicago) ymdhms)
  (let* ((symbol "MW.MGEX")
	 (want   (list (ymd->adate 2005 1 7)
		       (hms->seconds 11 50 0)))
	 (got    (mgex-quote-adate-time symbol)))
    (test "mgex-quote-adate-time" (list symbol ymdhms) want got)))

(let ((ymdhms '(2005 1 7  18 0 0)))  ;; Friday 18:00:00
  (apply fake-localtime-ymdhm! (timezone-chicago) ymdhms)
  (let* ((symbol "MW.MGEX")
	 (want   (list (ymd->adate 2005 1 7)
		       (hms->seconds 13 15 0)))
	 (got    (mgex-quote-adate-time symbol)))
    (test "mgex-quote-adate-time" (list symbol ymdhms) want got)))

(let ((ymdhms '(2005 1 8  12 0 0)))  ;; Saturday 12:00:00
  (apply fake-localtime-ymdhm! (timezone-chicago) ymdhms)
  (let* ((symbol "MW.MGEX")
	 (want   (list (ymd->adate 2005 1 7)
		       (hms->seconds 13 15 0)))
	 (got    (mgex-quote-adate-time symbol)))
    (test "mgex-quote-adate-time" (list symbol ymdhms) want got)))

(let ((ymdhms '(2005 1 9  12 0 0)))  ;; Sunday 12:00:00
  (apply fake-localtime-ymdhm! (timezone-chicago) ymdhms)
  (let* ((symbol "MW.MGEX")
	 (want   (list (ymd->adate 2005 1 7)
		       (hms->seconds 13 15 0)))
	 (got    (mgex-quote-adate-time symbol)))
    (test "mgex-quote-adate-time" (list symbol ymdhms) want got)))

(let ((ymdhms '(2005 1 10  6 0 0)))  ;; Monday 6:00:00
  (apply fake-localtime-ymdhm! (timezone-chicago) ymdhms)
  (let* ((symbol "MW.MGEX")
	 (want   (list (ymd->adate 2005 1 7)
		       (hms->seconds 13 15 0)))
	 (got    (mgex-quote-adate-time symbol)))
    (test "mgex-quote-adate-time" (list symbol ymdhms) want got)))

(let ((ymdhms '(2005 1 10  12 0 0)))  ;; Monday 12:00:00
  (apply fake-localtime-ymdhm! (timezone-chicago) ymdhms)
  (let* ((symbol "MW.MGEX")
	 (want   (list (ymd->adate 2005 1 10)
		       (hms->seconds 11 50 0)))
	 (got    (mgex-quote-adate-time symbol)))
    (test "mgex-quote-adate-time" (list symbol ymdhms) want got)))

;; MWE

(let ((ymdhms '(2005 1 7  5 0 0)))  ;; Friday 5:00:00
  (apply fake-localtime-ymdhm! (timezone-chicago) ymdhms)
  (let* ((symbol "MWE.MGEX")
	 (want   (list (ymd->adate 2005 1 7)
		       (hms->seconds 4 50 0)))
	 (got    (mgex-quote-adate-time symbol)))
    (test "mgex-quote-adate-time" (list symbol ymdhms) want got)))

(let ((ymdhms '(2005 1 7  7 0 0)))  ;; Friday 7:00:00
  (apply fake-localtime-ymdhm! (timezone-chicago) ymdhms)
  (let* ((symbol "MWE.MGEX")
	 (want   (list (ymd->adate 2005 1 7)
		       (hms->seconds 6 2 0)))
	 (got    (mgex-quote-adate-time symbol)))
    (test "mgex-quote-adate-time" (list symbol ymdhms) want got)))

(let ((ymdhms '(2005 1 7  12 0 0)))  ;; Friday 12:00:00
  (apply fake-localtime-ymdhm! (timezone-chicago) ymdhms)
  (let* ((symbol "MWE.MGEX")
	 (want   (list (ymd->adate 2005 1 7)
		       (hms->seconds 6 2 0)))
	 (got    (mgex-quote-adate-time symbol)))
    (test "mgex-quote-adate-time" (list symbol ymdhms) want got)))

(let ((ymdhms '(2005 1 7  20 0 0)))  ;; Friday 20:00:00
  (apply fake-localtime-ymdhm! (timezone-chicago) ymdhms)
  (let* ((symbol "MWE.MGEX")
	 (want   (list (ymd->adate 2005 1 7)
		       (hms->seconds 6 2 0)))
	 (got    (mgex-quote-adate-time symbol)))
    (test "mgex-quote-adate-time" (list symbol ymdhms) want got)))

(let ((ymdhms '(2005 1 8  12 0 0)))  ;; Saturday 12:00:00
  (apply fake-localtime-ymdhm! (timezone-chicago) ymdhms)
  (let* ((symbol "MWE.MGEX")
	 (want   (list (ymd->adate 2005 1 7)
		       (hms->seconds 6 2 0)))
	 (got    (mgex-quote-adate-time symbol)))
    (test "mgex-quote-adate-time" (list symbol ymdhms) want got)))

(let ((ymdhms '(2005 1 9  12 0 0)))  ;; Sunday 12:00:00
  (apply fake-localtime-ymdhm! (timezone-chicago) ymdhms)
  (let* ((symbol "MWE.MGEX")
	 (want   (list (ymd->adate 2005 1 7)
		       (hms->seconds 6 2 0)))
	 (got    (mgex-quote-adate-time symbol)))
    (test "mgex-quote-adate-time" (list symbol ymdhms) want got)))

(let ((ymdhms '(2005 1 9  20 0 0)))  ;; Sunday 20:00:00
  (apply fake-localtime-ymdhm! (timezone-chicago) ymdhms)
  (let* ((symbol "MWE.MGEX")
	 (want   (list (ymd->adate 2005 1 9)
		       (hms->seconds 19 50 0)))
	 (got    (mgex-quote-adate-time symbol)))
    (test "mgex-quote-adate-time" (list symbol ymdhms) want got)))

(let ((ymdhms '(2005 1 10  5 0 0)))  ;; Monday 5:00:00
  (apply fake-localtime-ymdhm! (timezone-chicago) ymdhms)
  (let* ((symbol "MWE.MGEX")
	 (want   (list (ymd->adate 2005 1 10)
		       (hms->seconds 4 50 0)))
	 (got    (mgex-quote-adate-time symbol)))
    (test "mgex-quote-adate-time" (list symbol ymdhms) want got)))

(let ((ymdhms '(2005 1 10  7 0 0)))  ;; Monday 7:00:00
  (apply fake-localtime-ymdhm! (timezone-chicago) ymdhms)
  (let* ((symbol "MWE.MGEX")
	 (want   (list (ymd->adate 2005 1 10)
		       (hms->seconds 6 2 0)))
	 (got    (mgex-quote-adate-time symbol)))
    (test "mgex-quote-adate-time" (list symbol ymdhms) want got)))

(let ((ymdhms '(2005 1 10  12 0 0)))  ;; Monday 12:00:00
  (apply fake-localtime-ymdhm! (timezone-chicago) ymdhms)
  (let* ((symbol "MWE.MGEX")
	 (want   (list (ymd->adate 2005 1 10)
		       (hms->seconds 6 2 0)))
	 (got    (mgex-quote-adate-time symbol)))
    (test "mgex-quote-adate-time" (list symbol ymdhms) want got)))

;; IC

(let ((ymdhms '(2005 1 7  5 0 0)))  ;; Friday 5:00:00
  (apply fake-localtime-ymdhm! (timezone-chicago) ymdhms)
  (let* ((symbol "IC.MGEX")
	 (want   (list (ymd->adate 2005 1 7)
		       (hms->seconds 5 0 0)))
	 (got    (mgex-quote-adate-time symbol)))
    (test "mgex-quote-adate-time" (list symbol ymdhms) want got)))

(let ((ymdhms '(2005 1 7  7 0 0)))  ;; Friday 7:00:00
  (apply fake-localtime-ymdhm! (timezone-chicago) ymdhms)
  (let* ((symbol "IC.MGEX")
	 (want   (list (ymd->adate 2005 1 7)
		       (hms->seconds 7 0 0)))
	 (got    (mgex-quote-adate-time symbol)))
    (test "mgex-quote-adate-time" (list symbol ymdhms) want got)))

(let ((ymdhms '(2005 1 7  12 0 0)))  ;; Friday 12:00:00
  (apply fake-localtime-ymdhm! (timezone-chicago) ymdhms)
  (let* ((symbol "IC.MGEX")
	 (want   (list (ymd->adate 2005 1 7)
		       (hms->seconds 12 0 0)))
	 (got    (mgex-quote-adate-time symbol)))
    (test "mgex-quote-adate-time" (list symbol ymdhms) want got)))

(let ((ymdhms '(2005 1 7  15 0 0)))  ;; Friday 15:00:00
  (apply fake-localtime-ymdhm! (timezone-chicago) ymdhms)
  (let* ((symbol "IC.MGEX")
	 (want   (list (ymd->adate 2005 1 7)
		       (hms->seconds 13 47 0)))
	 (got    (mgex-quote-adate-time symbol)))
    (test "mgex-quote-adate-time" (list symbol ymdhms) want got)))

(let ((ymdhms '(2005 1 7  20 0 0)))  ;; Friday 20:00:00
  (apply fake-localtime-ymdhm! (timezone-chicago) ymdhms)
  (let* ((symbol "IC.MGEX")
	 (want   (list (ymd->adate 2005 1 7)
		       (hms->seconds 13 47 0)))
	 (got    (mgex-quote-adate-time symbol)))
    (test "mgex-quote-adate-time" (list symbol ymdhms) want got)))

(let ((ymdhms '(2005 1 8  12 0 0)))  ;; Saturday 12:00:00
  (apply fake-localtime-ymdhm! (timezone-chicago) ymdhms)
  (let* ((symbol "IC.MGEX")
	 (want   (list (ymd->adate 2005 1 7)
		       (hms->seconds 13 47 0)))
	 (got    (mgex-quote-adate-time symbol)))
    (test "mgex-quote-adate-time" (list symbol ymdhms) want got)))

(let ((ymdhms '(2005 1 9  12 0 0)))  ;; Sunday 12:00:00
  (apply fake-localtime-ymdhm! (timezone-chicago) ymdhms)
  (let* ((symbol "IC.MGEX")
	 (want   (list (ymd->adate 2005 1 7)
		       (hms->seconds 13 47 0)))
	 (got    (mgex-quote-adate-time symbol)))
    (test "mgex-quote-adate-time" (list symbol ymdhms) want got)))

(let ((ymdhms '(2005 1 9  20 0 0)))  ;; Sunday 20:00:00
  (apply fake-localtime-ymdhm! (timezone-chicago) ymdhms)
  (let* ((symbol "IC.MGEX")
	 (want   (list (ymd->adate 2005 1 9)
		       (hms->seconds 20 0 0)))
	 (got    (mgex-quote-adate-time symbol)))
    (test "mgex-quote-adate-time" (list symbol ymdhms) want got)))

(let ((ymdhms '(2005 1 10  5 0 0)))  ;; Monday 5:00:00
  (apply fake-localtime-ymdhm! (timezone-chicago) ymdhms)
  (let* ((symbol "IC.MGEX")
	 (want   (list (ymd->adate 2005 1 10)
		       (hms->seconds 5 0 0)))
	 (got    (mgex-quote-adate-time symbol)))
    (test "mgex-quote-adate-time" (list symbol ymdhms) want got)))

(let ((ymdhms '(2005 1 10  7 0 0)))  ;; Monday 7:00:00
  (apply fake-localtime-ymdhm! (timezone-chicago) ymdhms)
  (let* ((symbol "IC.MGEX")
	 (want   (list (ymd->adate 2005 1 10)
		       (hms->seconds 7 0 0)))
	 (got    (mgex-quote-adate-time symbol)))
    (test "mgex-quote-adate-time" (list symbol ymdhms) want got)))

(let ((ymdhms '(2005 1 10  12 0 0)))  ;; Monday 12:00:00
  (apply fake-localtime-ymdhm! (timezone-chicago) ymdhms)
  (let* ((symbol "IC.MGEX")
	 (want   (list (ymd->adate 2005 1 10)
		       (hms->seconds 12 0 0)))
	 (got    (mgex-quote-adate-time symbol)))
    (test "mgex-quote-adate-time" (list symbol ymdhms) want got)))

(let ((ymdhms '(2005 1 10  15 0 0)))  ;; Monday 15:00:00
  (apply fake-localtime-ymdhm! (timezone-chicago) ymdhms)
  (let* ((symbol "IC.MGEX")
	 (want   (list (ymd->adate 2005 1 10)
		       (hms->seconds 13 47 0)))
	 (got    (mgex-quote-adate-time symbol)))
    (test "mgex-quote-adate-time" (list symbol ymdhms) want got)))

(fake-time-end)


(tests-end)
