;; Exercise misc.c functions.

;; Copyright 2005, 2006, 2007 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(load "tests.scm")

(if (= 0 (mallinfo-used))
    (display "Warning: no mallinfo(), memory leak checking will be minimal\n"))


;;
;; locale-d-fmt
;;

(malloced-steady
 (lambda ()
   (let* ((want    #t)
	  (got     (not (string-null? (locale-d-fmt)))))
     (test "locale-d-fmt" #f want got))))


;;
;; locale-decimal-point
;;

(malloced-steady
 (lambda ()
   (let* ((want    #t)
	  (got     (not (string-null? (locale-decimal-point)))))
     (test "locale-decimal-point" #f want got))))


;;
;; locale-thousands-separator
;;

(malloced-steady
 (lambda ()
   (let* ((want    #t)
	  (got     (string? (locale-thousands-sep))))
     (test "locale-thousands-sep" #f want got))))


;;
;; max-maybe
;;

(let* ((args '())
       (want #f)
       (got  (apply max-maybe args)))
  (test "max-maybe" args want got))

(let* ((args '(#f))
       (want #f)
       (got  (apply max-maybe args)))
  (test "max-maybe" args want got))

(let* ((args '(99))
       (want 99)
       (got  (apply max-maybe args)))
  (test "max-maybe" args want got))

(let* ((args '(#f 123))
       (want 123)
       (got  (apply max-maybe args)))
  (test "max-maybe" args want got))

(let* ((args '(123 #f))
       (want 123)
       (got  (apply max-maybe args)))
  (test "max-maybe" args want got))

(let* ((args '(50 80))
       (want 80)
       (got  (apply max-maybe args)))
  (test "max-maybe" args want got))

(let* ((args '(80 50))
       (want 80)
       (got  (apply max-maybe args)))
  (test "max-maybe" args want got))

(let* ((args '(1 2 3))
       (want 3)
       (got  (apply max-maybe args)))
  (test "max-maybe" args want got))

(let* ((args '(2 3 1))
       (want 3)
       (got  (apply max-maybe args)))
  (test "max-maybe" args want got))

(let* ((args '(x))
       (want 'wrong-type-arg)
       (got  (catch #t
	       (lambda ()
		 (apply max-maybe args))
	       (lambda (key . args)
		 key))))
  (test "max-maybe" args want got))

(let* ((args '(1+i))
       (want 'wrong-type-arg)
       (got  (catch #t
	       (lambda ()
		 (apply max-maybe args))
	       (lambda (key . args)
		 key))))
  (test "max-maybe" args want got))


;;
;; min-maybe
;;

(let* ((args '())
       (want #f)
       (got  (apply min-maybe args)))
  (test "min-maybe" args want got))

(let* ((args '(99))
       (want 99)
       (got  (apply min-maybe args)))
  (test "min-maybe" args want got))

(let* ((args '(#f))
       (want #f)
       (got  (apply min-maybe args)))
  (test "min-maybe" args want got))

(let* ((args '(#f 123))
       (want 123)
       (got  (apply min-maybe args)))
  (test "min-maybe" args want got))

(let* ((args '(123 #f))
       (want 123)
       (got  (apply min-maybe args)))
  (test "min-maybe" args want got))

(let* ((args '(50 80))
       (want 50)
       (got  (apply min-maybe args)))
  (test "min-maybe" args want got))

(let* ((args '(80 50))
       (want 50)
       (got  (apply min-maybe args)))
  (test "min-maybe" args want got))

(let* ((args '(1 2 3))
       (want 1)
       (got  (apply min-maybe args)))
  (test "min-maybe" args want got))

(let* ((args '(2 3 1))
       (want 1)
       (got  (apply min-maybe args)))
  (test "min-maybe" args want got))

(let* ((args '(x))
       (want 'wrong-type-arg)
       (got  (catch #t
	       (lambda ()
		 (apply min-maybe args))
	       (lambda (key . args)
		 key))))
  (test "min-maybe" args want got))

(let* ((args '(1+i))
       (want 'wrong-type-arg)
       (got  (catch #t
	       (lambda ()
		 (apply min-maybe args))
	       (lambda (key . args)
		 key))))
  (test "min-maybe" args want got))


;;
;; mb-cur-max
;;

(define mb-cur-max (@@ (chart misc) mb-cur-max))

(let* ((want    #t)
       (got     (number? (mb-cur-max))))
  (test "mb-cur-max" #f want got))

(let* ((want    #t)
       (got     (exact? (mb-cur-max))))
  (test "mb-cur-max" #f want got))

(let* ((want    #t)
       (got     (integer? (mb-cur-max))))
  (test "mb-cur-max" #f want got))


;;
;; output-port-width
;;
;; Note port create/gc is outside malloced-steady because in guile 1.6.7 the
;; port table entry code didn't correctly set mallocated (missing an
;; scm_done_free in scm_remove_from_port_table).
;;

(malloced-steady
 (lambda ()
   (let* ((want #t)
	  (got  (number? (output-port-width))))
     (test "output-port-width: default" 'no-arg want got))))

(let ((port (%make-void-port "w")))
  (malloced-steady
   (lambda ()
     (let* ((want 79)
	    (got  (output-port-width port)))
       (test "output-port-width: void" port want got)))))

(let ((port (open-output-string)))
  (malloced-steady
   (lambda ()
     (let* ((want 79)
	    (got  (output-port-width port)))
       (test "output-port-width: string" port want got)))))


;;
;; plist-get
;;

(let* ((plist '())
       (key   #:x)
       (want  #f)
       (got   (plist-get plist key)))
  (test "plist-get" (vector key plist) want got))

(let* ((plist '(#:y #:x))
       (key   #:x)
       (want  #f)
       (got   (plist-get plist key)))
  (test "plist-get" (vector key plist) want got))

(let* ((plist '(#:y #:x #:x 123))
       (key   #:x)
       (want  123)
       (got   (plist-get plist key)))
  (test "plist-get" (vector key plist) want got))


;;
;; square
;;

(let* ((x    5)
       (want 25)
       (got  (square x)))
  (test "square" x want got))

(let* ((x    -3.0)
       (want 9.0)
       (got  (square x)))
  (test "square" x want got))


(tests-end)
