;; Exercise dates functions from misc.scm.

;; Copyright 2003, 2004, 2005, 2006, 2007 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(load "tests.scm")


;; time_t to tdate, in GMT
(define (REF:time->tdate t)
  (let* ((d (- (quotient-floor t 86400) 4))) ;; cal days 0 = Mon 5 Jan 1970
    (- d
       ;; weekends
       (* 2 (quotient-floor d 7))
       ;; on saturday and sunday, give prev friday
       (max 0 (- (modulo d 7) 4)))))

(define (REF:tm->tdate tm)
  (REF:time->tdate (car (mktime tm))))
(putenv "TZ=GMT")
(tzset)

;; YEAR 4 digit, MONTH 1 to 12, return tdate
(define (REF:ymd->tdate year month day)
  (let ((tm (tm-new)))
    (set-tm:year  tm (- year 1900))
    (set-tm:mon   tm (1- month))
    (set-tm:mday  tm day)
    (set-tm:hour  tm 0)
    (set-tm:min   tm 0)
    (set-tm:sec   tm 0)
    (REF:tm->tdate tm)))

;; tdate to time_t, midnight GMT on the date
(define (REF:tdate->time d)
  (* 86400 (+ 4 (+ d (* 2 (quotient-floor d 5))))))

(define (REF:tdate->tm d)
  (gmtime (REF:tdate->time d)))

;; return list (year month day)
(define (REF:tdate->ymd t)
  (let ((tm (REF:tdate->tm t)))
    (list (+ (tm:year tm) 1900)
	  (1+ (tm:mon tm))
	  (tm:mday    tm))))


;;
;; adate->ymd
;;

(let* ((a    0)  ;; Mon 5 Jan 1970
       (want '(1970 1 5))
       (got  (adate->ymd a)))
  (test "adate->ymd" a want got))

(let* ((a    1)  ;; Tue 6 Jan 1970
       (want '(1970 1 6))
       (got  (adate->ymd a)))
  (test "adate->ymd" a want got))

(let* ((a    -1)  ;; Sun 4 Jan 1970
       (want '(1970 1 4))
       (got  (adate->ymd a)))
  (test "adate->ymd" a want got))

;;
;; adate->tdate
;;

(let* ((a    -14) ;; Mon 22 Dec 1969
       (want -10)
       (got  (adate->tdate a)))
  (test "adate->tdate" a want got))

(let* ((a    -10) ;; Fri 26 Dec 1969
       (want -6)
       (got  (adate->tdate a)))
  (test "adate->tdate" a want got))

(let* ((a    -7) ;; Mon 29 Dec 1969
       (want -5)
       (got  (adate->tdate a)))
  (test "adate->tdate" a want got))

(let* ((a    -4) ;; Thu 1 Jan 1970
       (want -2)
       (got  (adate->tdate a)))
  (test "adate->tdate" a want got))

(let* ((a    -3) ;; Fri 2 Jan 1970
       (want -1)
       (got  (adate->tdate a)))
  (test "adate->tdate" a want got))

(let* ((a    -2) ;; Sat 3 Jan 1970
       (want -1)
       (got  (adate->tdate a)))
  (test "adate->tdate" a want got))

(let* ((a    -1) ;; Sun 4 Jan 1970
       (want -1)
       (got  (adate->tdate a)))
  (test "adate->tdate" a want got))

(let* ((a    0)  ;; Mon 5 Jan 1970
       (want 0)
       (got  (adate->tdate a)))
  (test "adate->tdate" a want got))

(let* ((a    1)  ;; Tue 6 Jan 1970
       (want 1)
       (got  (adate->tdate a)))
  (test "adate->tdate" a want got))

(let* ((a    2)  ;; Wed 7 Jan 1970
       (want 2)
       (got  (adate->tdate a)))
  (test "adate->tdate" a want got))

(let* ((a    3)  ;; Thu 8 Jan 1970
       (want 3)
       (got  (adate->tdate a)))
  (test "adate->tdate" a want got))

(let* ((a    4)  ;; Fri 9 Jan 1970
       (want 4)
       (got  (adate->tdate a)))
  (test "adate->tdate" a want got))

(let* ((a    5)  ;; Sat 9 Jan 1970
       (want 4)
       (got  (adate->tdate a)))
  (test "adate->tdate" a want got))

(let* ((a    6)  ;; Sun 10 Jan 1970
       (want 4)
       (got  (adate->tdate a)))
  (test "adate->tdate" a want got))

(let* ((a    7)  ;; Mon 11 Jan 1970
       (want 5)
       (got  (adate->tdate a)))
  (test "adate->tdate" a want got))

;;
;; adate->tdate-ceil
;;

(let* ((a    -4) ;; Thu 1 Jan 1970
       (want -2)
       (got  (adate->tdate-ceil a)))
  (test "adate->tdate-ceil" a want got))

(let* ((a    -3) ;; Fri 2 Jan 1970
       (want -1)
       (got  (adate->tdate-ceil a)))
  (test "adate->tdate-ceil" a want got))

(let* ((a    -2) ;; Sat 3 Jan 1970
       (want 0)
       (got  (adate->tdate-ceil a)))
  (test "adate->tdate-ceil" a want got))

(let* ((a    -1) ;; Sun 4 Jan 1970
       (want 0)
       (got  (adate->tdate-ceil a)))
  (test "adate->tdate-ceil" a want got))

(let* ((a    0)  ;; Mon 5 Jan 1970
       (want 0)
       (got  (adate->tdate-ceil a)))
  (test "adate->tdate-ceil" a want got))

(let* ((a    1)  ;; Tue 6 Jan 1970
       (want 1)
       (got  (adate->tdate-ceil a)))
  (test "adate->tdate-ceil" a want got))

(let* ((a    2)  ;; Wed 7 Jan 1970
       (want 2)
       (got  (adate->tdate-ceil a)))
  (test "adate->tdate-ceil" a want got))

(let* ((a    3)  ;; Thu 8 Jan 1970
       (want 3)
       (got  (adate->tdate-ceil a)))
  (test "adate->tdate-ceil" a want got))

(let* ((a    4)  ;; Fri 9 Jan 1970
       (want 4)
       (got  (adate->tdate-ceil a)))
  (test "adate->tdate-ceil" a want got))

(let* ((a    5)  ;; Sat 9 Jan 1970
       (want 5)
       (got  (adate->tdate-ceil a)))
  (test "adate->tdate-ceil" a want got))

(let* ((a    6)  ;; Sun 10 Jan 1970
       (want 5)
       (got  (adate->tdate-ceil a)))
  (test "adate->tdate-ceil" a want got))

(let* ((a    7)  ;; Mon 11 Jan 1970
       (want 5)
       (got  (adate->tdate-ceil a)))
  (test "adate->tdate-ceil" a want got))

;;
;; adate->tm
;;

(let* ((a    -2)  ;; Sat 3 Jan 1970
       (want (let ((tm (tm-new)))
	       (set-tm:year tm 70)
	       (set-tm:mon  tm 0)
	       (set-tm:mday tm 3)
	       (set-tm:wday tm 6)
	       tm))
       (got  (adate->tm a)))
  (test "adate->tm" a want got))

(let* ((a    -1)  ;; Sun 4 Jan 1970
       (want (let ((tm (tm-new)))
	       (set-tm:year tm 70)
	       (set-tm:mon  tm 0)
	       (set-tm:mday tm 4)
	       (set-tm:wday tm 0)
	       tm))
       (got  (adate->tm a)))
  (test "adate->tm" a want got))

(let* ((a    0)  ;; Mon 5 Jan 1970
       (want (let ((tm (tm-new)))
	       (set-tm:year tm 70)
	       (set-tm:mon  tm 0)
	       (set-tm:mday tm 5)
	       (set-tm:wday tm 1)
	       tm))
       (got  (adate->tm a)))
  (test "adate->tm" a want got))

(let* ((a    4)  ;; Fri 9 Jan 1970
       (want (let ((tm (tm-new)))
	       (set-tm:year tm 70)
	       (set-tm:mon  tm 0)
	       (set-tm:mday tm 9)
	       (set-tm:wday tm 5)
	       tm))
       (got  (adate->tm a)))
  (test "adate->tm" a want got))

(let* ((a    5)  ;; Sat 10 Jan 1970
       (want (let ((tm (tm-new)))
	       (set-tm:year tm 70)
	       (set-tm:mon  tm 0)
	       (set-tm:mday tm 10)
	       (set-tm:wday tm 6)
	       tm))
       (got  (adate->tm a)))
  (test "adate->tm" a want got))

(let* ((a    6)  ;; Sun 11 Jan 1970
       (want (let ((tm (tm-new)))
	       (set-tm:year tm 70)
	       (set-tm:mon  tm 0)
	       (set-tm:mday tm 11)
	       (set-tm:wday tm 0)
	       tm))
       (got  (adate->tm a)))
  (test "adate->tm" a want got))


;;
;; ddate->year
;;

(let* ((d    0)
       (want 0)
       (got  (ddate->year d)))
  (test "ddate->year" d want got))

(let* ((d    200)
       (want 2000)
       (got  (ddate->year d)))
  (test "ddate->year" d want got))


;;
;; ddate->qdate
;;

(let* ((d    197)  ;; 1970
       (want 0)
       (got  (ddate->qdate d)))
  (test "ddate->qdate" d want got))

(let* ((d    198)
       (want 40)
       (got  (ddate->qdate d)))
  (test "ddate->qdate" d want got))

(let* ((d    196)
       (want -40)
       (got  (ddate->qdate d)))
  (test "ddate->qdate" d want got))


;;
;; mdate->qdate
;;

(let* ((m    0)
       (want 0)
       (got  (mdate->qdate m)))
  (test "mdate->qdate" m want got))

(let* ((m    1)
       (want 0)
       (got  (mdate->qdate m)))
  (test "mdate->qdate" m want got))

(let* ((m    2)
       (want 0)
       (got  (mdate->qdate m)))
  (test "mdate->qdate" m want got))

(let* ((m    3)
       (want 1)
       (got  (mdate->qdate m)))
  (test "mdate->qdate" m want got))

(let* ((m    6)
       (want 2)
       (got  (mdate->qdate m)))
  (test "mdate->qdate" m want got))

(let* ((m    -1)
       (want -1)
       (got  (mdate->qdate m)))
  (test "mdate->qdate" m want got))

(let* ((m    -2)
       (want -1)
       (got  (mdate->qdate m)))
  (test "mdate->qdate" m want got))

(let* ((m    -3)
       (want -1)
       (got  (mdate->qdate m)))
  (test "mdate->qdate" m want got))

(let* ((m    -6)
       (want -2)
       (got  (mdate->qdate m)))
  (test "mdate->qdate" m want got))

;;
;; mdate->tdate
;;

(for-each (lambda (data)
	    (receive-list (mdate want)
		data
	      (let ((got (mdate->tdate mdate)))
		(test "mdate->tdate" mdate want got))))
	  '((0   -2)  ;; Thu 1 Jan 1970
	    (1   20)  ;; Mon 2 Feb 1970
	    (-1 -25)  ;; Mon 1 Dec 1969
	    ))

;;
;; mdate->ymd
;;

(for-each (lambda (data)
	    (receive-list (m want)
		data
	      (let ((got (mdate->ymd m)))
		(test "mdate->ymd" m want got))))
	  '((0   (1970  1 1))
	    (1   (1970  2 1))
	    (11  (1970 12 1))
	    (12  (1971  1 1))

	    (-1  (1969 12 1))
	    (-12 (1969  1 1))
	    (-13 (1968 12 1))))

;;
;; mdate->year
;;

(for-each (lambda (data)
	    (receive-list (m want)
		data
	      (let ((got (mdate->year m)))
		(test "mdate->year" m want got))))
	  '((0   1970)
	    (1   1970)
	    (11  1970)
	    (12  1971)

	    (-1  1969)
	    (-12 1969)
	    (-13 1968)))

;;
;; Mmm-str->month
;;

(let* ((str  "MAY")
       (want 5)
       (got  (Mmm-str->month str)))
  (test "Mmm-str->month" str want got))

;;
;; qdate->ddate
;;

(let* ((q    -1)   ;; Oct/Nov/Dec 1969
       (want 196)
       (got  (qdate->ddate q)))
  (test "qdate->ddate" q want got))

(let* ((q    0)    ;; Jan/Feb/Mar 1970
       (want 197)
       (got  (qdate->ddate q)))
  (test "qdate->ddate" q want got))

(let* ((q    1)    ;; Apr/May/Jun 1970
       (want 197)
       (got  (qdate->ddate q)))
  (test "qdate->ddate" q want got))

(let* ((q    40)   ;; Jan/Feb/Mar 1980
       (want 198)
       (got  (qdate->ddate q)))
  (test "qdate->ddate" q want got))

;;
;; qdate->mdate
;;

(let* ((q    0)
       (want 0)
       (got  (qdate->mdate q)))
  (test "qdate->mdate" q want got))

(let* ((q    1)
       (want 3)
       (got  (qdate->mdate q)))
  (test "qdate->mdate" q want got))

(let* ((q    2)
       (want 6)
       (got  (qdate->mdate q)))
  (test "qdate->mdate" q want got))

(let* ((q    -1)
       (want -3)
       (got  (qdate->mdate q)))
  (test "qdate->mdate" q want got))

(let* ((q    -2)
       (want -6)
       (got  (qdate->mdate q)))
  (test "qdate->mdate" q want got))

;;
;; qdate->tdate
;;

(let* ((q    0)
       (want -2)
       (got  (qdate->tdate q)))
  (test "qdate->tdate" q want got))

(let* ((q    1)
       (want 62)
       (got  (qdate->tdate q)))
  (test "qdate->tdate" q want got))

;;
;; qdate->wdate
;;

(let* ((q    0)
       (want 0)
       (got  (qdate->wdate q)))
  (test "qdate->wdate" q want got))

(let* ((q    1)
       (want 13)
       (got  (qdate->wdate q)))
  (test "qdate->wdate" q want got))

;;
;; qdate->year
;;

(for-each (lambda (elem)
	    (let* ((qdate (first elem))
		   (want  (second elem))
		   (got   (qdate->year qdate)))
	      (test "qdate->year" qdate want got)))
	  `((0 1970)
	    (1 1970)
	    (3 1970)
	    (4 1971)
	    (-1 1969)
	    (-2 1969)
	    (-4 1969)
	    (-5 1968)))

;;
;; qdate->ymd
;;

(for-each (lambda (elem)
	    (let* ((qdate (first elem))
		   (want  (second elem))
		   (got   (qdate->ymd qdate)))
	      (test "qdate->ymd" qdate want got)))
	  `((0  (1970 1 1))
	    (1  (1970 4 1))
	    (2  (1970 7 1))
	    (-1 (1969 10 1))
	    (-9 (1967 10 1))))

;;
;; tdate->adate
;;

(for-each (lambda (elem)
	    (let* ((tdate (first elem))
		   (want  (second elem))
		   (got   (tdate->adate tdate)))
	      (test "tdate->adate" tdate want got)))
	  `((-10  -14)  ;; Mon 22 Dec 1969
	    (-6   -10)  ;; Fri 26 Dec 1969
	    (-5   -7)   ;; Mon 29 Dec 1969
	    (-2   -4)   ;; Thu 1 Jan 1970
	    (-1   -3)   ;; Fri 2 Jan 1970
	    (0     0)   ;; Mon 5 Jan 1970
	    (1     1)   ;; Tue 6 Jan 1970
	    (2     2)   ;; Wed 7 Jan 1970
	    (3     3)   ;; Thu 8 Jan 1970
	    (4     4)   ;; Fri 9 Jan 1970
	    (5     7)   ;; Mon 11 Jan 1970
	    ))

;;
;; tdate->ddate
;;

(for-each (lambda (data)
	    (receive-list (tdate want)
		data
	      (let ((got (tdate->ddate tdate)))
		(test "tdate->ddate" tdate want got))))
	  '((0   197)    ;; Mon  5 Jan 1970
	    (-2  197)    ;; Thu  1 Jan 1970
	    (-3  196)    ;; Wed 31 Dec 1969
	    ))

;;
;; tdate->wdate
;;

(for-each (lambda (data)
	    (receive-list (tdate want)
		data
	      (let ((got (tdate->wdate tdate)))
		(test "tdate->wdate" tdate want got))))
	  `((0     0)
	    (1     0)
	    (4     0)
	    (5     1)
	    (6     1)

	    (-1   -1)
	    (-4   -1)
	    (-5   -1)
	    (-6   -2)
	    (-10  -2)
	    (-11  -3)

	    (50    10)
	    (-50  -10)))

;;
;; tdate->ymd
;;

(for-each (lambda (elem)
	    (let* ((tdate (first elem))
		   (want  (second elem))
		   (got   (tdate->ymd tdate)))
	      (test "tdate->ymd" tdate want got)))
	  `((562  (1972 3 1))
	    (584  (1972 3 31))
	    (585  (1972 4 3))
	    (604  (1972 4 28))
	    (605  (1972 5 1))
	    (0    (1970 1 5)) ;; Mon
	    (1    (1970 1 6)) ;; Tue
	    (2    (1970 1 7)) ;; Wed
	    (3    (1970 1 8)) ;; Thu
	    (4    (1970 1 9)) ;; Fri
	    (5    (1970 1 12)) ;; Mon
	    (6    (1970 1 13)) ;; Tue
	    (-2   (1970 1 1)) ;; Thu
	    (-1   (1970 1 2)) ;; Fri
	    ))

(do ((t -10000 (1+ t)))
    ((>= t 10000))

  (let ((want (REF:tdate->ymd t))
	(got  (tdate->ymd t)))
    (test "tdate->ymd (vs REF)" t want got)))

;;
;; timezone-known?
;;

(let* ((zone "GMT")
       (want #t)
       (got  (->bool (timezone-known? zone))))
  (test "timezone-known?" zone want got))

(let* ((zone "Nosuchtimezone/blah/blah/blah")
       (want #f)
       (got  (->bool (timezone-known? zone))))
  (test "timezone-known?" zone want got))

;;
;; tm->adate-time-within
;;

(let* ((tm   (gmtime (* 5 86400)))           ;; Tue 6 Jan 1970 midnight
       (lo   (hms->seconds 9 0 0))
       (hi   (hms->seconds 17 0 0))
       (want (list 0 (hms->seconds 17 0 0))) ;; back to Mon 17:00
       (got  (tm->adate-time-within tm lo hi)))
  (test "tm->adate-time-within" (list (strftime "%c" tm) lo hi) want got))

(let* ((tm   (gmtime (+ (* 5 86400)          ;; Tue 6 Jan 1970 8:58
			(hms->seconds 8 58 0))))
       (lo   (hms->seconds 9 0 0))
       (hi   (hms->seconds 17 0 0))
       (want (list 1 (hms->seconds 8 58 0)))
       (got  (tm->adate-time-within tm lo hi)))
  (test "tm->adate-time-within" (list (strftime "%c" tm) lo hi) want got))

(let* ((tm   (gmtime (+ (* 5 86400)          ;; Tue 6 Jan 1970 12:00
			(hms->seconds 12 0 0))))
       (lo   (hms->seconds 9 0 0))
       (hi   (hms->seconds 17 0 0))
       (want (list 1 (hms->seconds 12 0 0)))
       (got  (tm->adate-time-within tm lo hi)))
  (test "tm->adate-time-within" (list (strftime "%c" tm) lo hi) want got))

(let* ((tm   (gmtime (+ (* 5 86400)          ;; Tue 6 Jan 1970 17:30
			(hms->seconds 17 30 0))))
       (lo   (hms->seconds 9 0 0))
       (hi   (hms->seconds 17 0 0))
       (want (list 1 (hms->seconds 17 0 0)))
       (got  (tm->adate-time-within tm lo hi)))
  (test "tm->adate-time-within" (list (strftime "%c" tm) lo hi) want got))

(let* ((tm   (gmtime (+ (* 9 86400)          ;; Sat 10 Jan 1970 12:00
			(hms->seconds 12 0 0))))
       (lo   (hms->seconds 9 0 0))
       (hi   (hms->seconds 17 0 0))
       (want (list 4 (hms->seconds 17 0 0))) ;; back to Fri 17:00
       (got  (tm->adate-time-within tm lo hi)))
  (test "tm->adate-time-within" (list (strftime "%c" tm) lo hi) want got))

(let* ((tm   (gmtime (+ (* 10 86400)         ;; Sun 10 Jan 1970 12:00
			(hms->seconds 12 0 0))))
       (lo   (hms->seconds 9 0 0))
       (hi   (hms->seconds 17 0 0))
       (want (list 4 (hms->seconds 17 0 0))) ;; back to Fri 17:00
       (got  (tm->adate-time-within tm lo hi)))
  (test "tm->adate-time-within" (list (strftime "%c" tm) lo hi) want got))

(let* ((tm   (gmtime (+ (* 11 86400))))      ;; Mon 12 Jan 1970 0:00
       (lo   (hms->seconds 9 0 0))
       (hi   (hms->seconds 17 0 0))
       (want (list 4 (hms->seconds 17 0 0))) ;; back to Fri 17:00
       (got  (tm->adate-time-within tm lo hi)))
  (test "tm->adate-time-within" (list (strftime "%c" tm) lo hi) want got))

;;
;; wdate->tdate
;;

(for-each (lambda (data)
	    (receive-list (wdate want)
		data
	      (let ((got (wdate->tdate wdate)))
		(test "wdate->tdate" wdate want got))))
	  `((0     0)
	    (10    50)
	    (-10  -50)))

;;
;; wdate->ymd
;;

(for-each (lambda (elem)
	    (let* ((wdate (first elem))
		   (want  (second elem))
		   (got   (wdate->ymd wdate)))
	      (test "wdate->ymd" wdate want got)))
	  `((3  (1970 1 26))  ;; Mon 26 Jan 1970
	    (0  (1970 1 5))   ;; Mon 5 Jan 1970
	    (-3 (1969 12 15)) ;; Mon 15 Dec 1969
	    ))

;;
;; y->year
;;

(define y->year (@@ (chart misc) y->year))

(let ((old-today-year   today-year)
      (fake-today-year  #f))
  (set! today-year (lambda ()
		     fake-today-year))
  (for-each
   (lambda (data)
     (receive-list (current-year y want)
	 data
       (set! fake-today-year  current-year)
       (let ((got (y->year y)))
	 (test "y->year" (list current-year y)
	       want got))))
   '((1970   0  1970)
     (1970   1  1971)
     (1970   4  1974)
     (1970   5  1965)
     (1970   6  1966)
     (1970   9  1969)
    
     (1975   0  1970)
     (1975   5  1975)
     (1975   9  1979)

     (2001   0  2000)
     (2001   1  2001)
     (2001   2  2002)
     (2001   3  2003)
     (2001   4  2004)
     (2001   5  2005)
     (2001   6  1996)
     (2001   7  1997)
     (2001   8  1998)
     (2001   9  1999)

     (2006   0  2010)
     (2006   1  2001)
     (2006   2  2002)
     (2006   3  2003)
     (2006   4  2004)
     (2006   5  2005)
     (2006   6  2006)
     (2006   7  2007)
     (2006   8  2008)
     (2006   9  2009)

     (2009   0  2010)
     (2009   1  2011)
     (2009   2  2012)
     (2009   3  2013)
     (2009   4  2004)
     (2009   5  2005)
     (2009   6  2006)
     (2009   7  2007)
     (2009   8  2008)
     (2009   9  2009)))

  (set! today-year old-today-year))

;;
;; ymd->mdate
;;

(let* ((ymd  '(1970 1 1))
       (want 0)
       (got  (apply ymd->mdate ymd)))
  (test "ymd->mdate" ymd want got))

(let* ((ymd  '(1970 2 1))
       (want 1)
       (got  (apply ymd->mdate ymd)))
  (test "ymd->mdate" ymd want got))

(let* ((ymd  '(1971 3 1))
       (want 14)
       (got  (apply ymd->mdate ymd)))
  (test "ymd->mdate" ymd want got))

(let* ((ymd  '(1969 12 1))
       (want -1)
       (got  (apply ymd->mdate ymd)))
  (test "ymd->mdate" ymd want got))

(let* ((ymd  '(1968 11 1))
       (want -14)
       (got  (apply ymd->mdate ymd)))
  (test "ymd->mdate" ymd want got))


;;
;; year->mdate
;;

(let* ((y    1970)
       (want 0)
       (got  (year->mdate y)))
  (test "year->mdate" y want got))

(let* ((y    1971)
       (want 12)
       (got  (year->mdate y)))
  (test "year->mdate" y want got))

(let* ((y    1969)
       (want -12)
       (got  (year->mdate y)))
  (test "year->mdate" y want got))

;;
;; year->ddate
;;

(let* ((y    0)
       (want 0)
       (got  (year->ddate y)))
  (test "year->ddate" y want got))

(let* ((y    1)
       (want 0)
       (got  (year->ddate y)))
  (test "year->ddate" y want got))

(let* ((y    9)
       (want 0)
       (got  (year->ddate y)))
  (test "year->ddate" y want got))

(let* ((y    10)
       (want 1)
       (got  (year->ddate y)))
  (test "year->ddate" y want got))

(let* ((y    -1)
       (want -1)
       (got  (year->ddate y)))
  (test "year->ddate" y want got))

(let* ((y    -10)
       (want -1)
       (got  (year->ddate y)))
  (test "year->ddate" y want got))

(let* ((y    -11)
       (want -2)
       (got  (year->ddate y)))
  (test "year->ddate" y want got))

;;
;; year->qdate
;;

(let* ((y    1970)
       (want 0)
       (got  (year->qdate y)))
  (test "year->qdate" y want got))

(let* ((y    1971)
       (want 4)
       (got  (year->qdate y)))
  (test "year->qdate" y want got))

(let* ((y    1969)
       (want -4)
       (got  (year->qdate y)))
  (test "year->qdate" y want got))

;;
;; year->tdate
;;

(for-each (lambda (data)
	    (receive-list (year want)
		data
	      (let ((got (year->tdate year)))
		(test "year->tdate" year want got))))
	  `((1970   -2)                          ;; Thu  1 Jan 1970
	    (1971   ,(REF:ymd->tdate 1971 1 1))  ;; Fri 1 Jan 1971
	    (1972   ,(REF:ymd->tdate 1972 1 3))  ;; Mon 3 Jan 1972
	    (1969   ,(REF:ymd->tdate 1969 1 1))  ;; Wed 1 Jan 1969
	    ))

;;
;; ymd->qdate
;;

(for-each (lambda (data)
	    (receive-list (ymd want)
		data
	      (let ((got (apply ymd->qdate ymd)))
		(test "ymd->qdate" ymd want got))))
	  '(((1970 1  1)   0)     ;;  Jan 1970
	    ((1970 2  1)   0)     ;;  Feb 1970
	    ((1970 3  1)   0)     ;;  Mar 1970
	    ((1970 4  1)   1)     ;;  Apr 1970
	    ((1971 1  1)   4)     ;;  Jan 1971

	    ((1969 12 1)  -1)     ;;  Dec 1969
	    ((1969 11 1)  -1)     ;;  Nov 1969
	    ((1969 10 1)  -1)     ;;  Oct 1969

	    ((1969 9 1)   -2)     ;;  Sep 1969
	    ((1969 1 1)   -4)     ;;  Jan 1969
	    ))

;;
;; ymd->tdate
;;

(for-each (lambda (data)
	    (receive-list (ymd want)
		data
	      (let ((got (apply ymd->tdate ymd)))
		(test "ymd->tdate" ymd want got))))
	  '(((1970 1  1)   -2)    ;; Thu  1 Jan 1970
	    ((1970 1  2)   -1)    ;; Fri  2 Jan 1970
	    ((1970 1  3)   -1)    ;; Sat  3 Jan 1970
	    ((1970 1  4)   -1)    ;; Sun  4 Jan 1970
	    ((1970 1  5)    0)    ;; Mon  5 Jan 1970
	    ((1970 1  6)    1)    ;; Tue  6 Jan 1970
	    ((1970 1  7)    2)    ;; Wed  7 Jan 1970
	    ((1970 1  8)    3)    ;; Thu  8 Jan 1970
	    ((1970 1  9)    4)    ;; Fri  9 Jan 1970
	    ((1970 1 10)    4)    ;; Sat 10 Jan 1970
	    ((1970 1 11)    4)    ;; Sun 11 Jan 1970

	    ((1970 2  2)   20)    ;; Mon  2 Feb 1970
	    ))

;;
;; ymd->tdate
;;

(for-each (lambda (data)
	    (receive-list (ymd want)
		data
	      (let ((got (apply ymd->tdate ymd)))
		(test "ymd->tdate" ymd want got))))
	  '(((1970 1  1)   -2)    ;; Thu  1 Jan 1970
	    ((1970 1  2)   -1)    ;; Fri  2 Jan 1970
	    ((1970 1  3)   -1)    ;; Sat  3 Jan 1970
	    ((1970 1  4)   -1)    ;; Sun  4 Jan 1970
	    ((1970 1  5)    0)    ;; Mon  5 Jan 1970
	    ((1970 1  6)    1)    ;; Tue  6 Jan 1970
	    ((1970 1  7)    2)    ;; Wed  7 Jan 1970
	    ((1970 1  8)    3)    ;; Thu  8 Jan 1970
	    ((1970 1  9)    4)    ;; Fri  9 Jan 1970
	    ((1970 1 10)    4)    ;; Sat 10 Jan 1970
	    ((1970 1 11)    4)    ;; Sun 11 Jan 1970
	    ))

;; vs REF:ymd->tdate on all YMD from 1970 to 1980
(do ((year 1970 (1+ year)))
    ((> year 1980))
  (do ((month 1 (1+ month)))
      ((> month 12))
    (do ((day 1 (1+ day)))
	((> day 31))

      (let ((want (REF:ymd->tdate year month day))
	    (got  (ymd->tdate year month day)))
	(test "ymd->tdate (vs REF)" (list year month day) want got)))))

;;
;; ymd->tdate-ceil
;;

(for-each (lambda (data)
	    (receive-list (ymd want)
		data
	      (let ((got (apply ymd->tdate-ceil ymd)))
		(test "ymd->tdate-ceil" ymd want got))))
	  '(((1969 12 31)  -3)    ;; Wed 12 Dec 1969
	    ((1970 1  1)   -2)    ;; Thu  1 Jan 1970
	    ((1970 1  2)   -1)    ;; Fri  2 Jan 1970
	    ((1970 1  3)    0)    ;; Sat  3 Jan 1970
	    ((1970 1  4)    0)    ;; Sun  4 Jan 1970
	    ((1970 1  5)    0)    ;; Mon  5 Jan 1970
	    ((1970 1  6)    1)    ;; Tue  6 Jan 1970
	    ((1970 1  7)    2)    ;; Wed  7 Jan 1970
	    ((1970 1  8)    3)    ;; Thu  8 Jan 1970
	    ((1970 1  9)    4)    ;; Fri  9 Jan 1970
	    ((1970 1 10)    5)    ;; Sat 10 Jan 1970
	    ((1970 1 11)    5)    ;; Sun 11 Jan 1970
	    ((1970 1 12)    5)    ;; Mon 12 Jan 1970
	    ((1970 1 13)    6)    ;; Tue 13 Jan 1970
	    ))

(tests-end)
