;; Test miscellaneous functions.

;; Copyright 2003, 2004, 2005, 2006, 2007 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(load "tests.scm")
(use-modules (chart http))


(define (make-random-string len)
  (string-tabulate (lambda (i)
		     (string-ref "abcdefghijklmnopqrstuvwxyz" (random 26)))
		   len))


;;
;; arithmetic-mean
;;

(let* ((args '(2 0))
       (want 1.0)
       (got  (apply arithmetic-mean args)))
  (test "arithmetic-mean" args want got))

(let* ((args '(3.0 1.0 1.0 1.0))
       (want 1.5)
       (got  (apply arithmetic-mean args)))
  (test "arithmetic-mean" args want got))

(let* ((args '())
       (want 'wrong-number-of-args)
       (got  (catch #t
	       (lambda ()
		 (apply arithmetic-mean args))
	       (lambda (key . args)
		 key))))
  (test "arithmetic-mean" args want got))


;;
;; ceiling-exact
;;

(let* ((x    3)
       (want 3)
       (got  (ceiling-exact x)))
  (test "ceiling-exact" x want got))

(let* ((x    3/2)
       (want 2)
       (got  (ceiling-exact x)))
  (test "ceiling-exact" x want got))

(let* ((x    -3/2)
       (want -1)
       (got  (ceiling-exact x)))
  (test "ceiling-exact" x want got))

(let* ((x    -1.5)
       (want -1)
       (got  (ceiling-exact x)))
  (test "ceiling-exact" x want got))


;;
;; chart-symbol-sans-hat
;;

(let* ((symbol "^FOO")
       (want   "FOO")
       (got    (chart-symbol-sans-hat symbol)))
  (test "chart-symbol-sans-hat" symbol want got))

(let* ((symbol "FOO")
       (want   "FOO")
       (got    (chart-symbol-sans-hat symbol)))
  (test "chart-symbol-sans-hat" symbol want got))

(let* ((symbol "")
       (want   "")
       (got    (chart-symbol-sans-hat symbol)))
  (test "chart-symbol-sans-hat" symbol want got))

(let* ((symbol "^")
       (want   "")
       (got    (chart-symbol-sans-hat symbol)))
  (test "chart-symbol-sans-hat" symbol want got))


;;
;; floor-exact
;;

(let* ((x    3)
       (want 3)
       (got  (floor-exact x)))
  (test "floor-exact" x want got))

(let* ((x    3/2)
       (want 1)
       (got  (floor-exact x)))
  (test "floor-exact" x want got))

(let* ((x    -3/2)
       (want -2)
       (got  (floor-exact x)))
  (test "floor-exact" x want got))

(let* ((x    -1.5)
       (want -2)
       (got  (floor-exact x)))
  (test "floor-exact" x want got))


;;
;; c-main-unqueue!
;;

(define c-main-unqueue! (@@ (chart misc) c-main-unqueue!))

(let* ((want #f)
       (got  (c-main-unqueue! noop)))
  (test "c-main-unqueue!" noop want got))

(let* ((want #t)
       (got  (begin
	       (c-main-enq! noop)
	       (c-main-unqueue! noop))))
  (test "c-main-unqueue!" noop want got))




;;
;; c-idle, inline
;;

(let* ((want #t)
       (got  (begin
	       (c-idle)
	       #t)))
  (test "c-idle inline" #f want got))

(let* ((priority 123)
       (want     #t)
       (got      (begin
		   (c-idle priority)
		   #t)))
  (test "c-idle inline" priority want got))

;;
;; c-input-wait, inline
;;

(let* ((port  (open-input-file "/dev/null"))
       (conds '(read))
       (want  #t)
       (got   (begin
		(c-input-wait port conds)
		#t)))
  (test "c-input-wait inline" (list port conds) want got))

;;
;; c-input-timeout-wait, inline
;;

(let* ((port    (open-input-file "/dev/null"))
       (conds   '(read))
       (timeout 20)
       (want    #t)
       (got     (c-input-timeout-wait port conds timeout)))
  (test "c-input-timeout-wait inline" (list port conds timeout) want got))

(let* ((pip     (pipe))    ;; pipe nothing to read
       (port    (car pip))
       (conds   '(read))
       (timeout 0.1)
       (want    #f)
       (got     (c-input-timeout-wait port conds timeout)))
  (test "c-input-timeout-wait inline" (list port conds timeout) want got))



;;
;; c-idle, non-gui
;;

(let* ((want #t)
       (got  'zzz))
  (c-main-enq! (lambda ()
		 (c-idle)
		 (set! got #t)))
  (c-main)
  (test "c-idle non-gui" #f want got))

(let* ((priority 123)
       (want     #t)
       (got      'zzz))
  (c-main-enq! (lambda ()
		 (c-idle priority)
		 (set! got #t)))
  (c-main)
  (test "c-idle non-gui" priority want got))

;;
;; c-input-wait, non-gui
;;

(let* ((port  (open-input-file "/dev/null"))
       (conds '(read))
       (want  #t)
       (got   'zzz))
  (c-main-enq! (lambda ()
		 (c-input-wait port conds)
		 (set! got #t)))
  (c-main)
  (test "c-input-wait non-gui" (list port conds) want got))

;;
;; c-input-timeout-wait, non-gui
;;

(let* ((port    (open-input-file "/dev/null"))
       (conds   '(read))
       (timeout 20)
       (want    #t)
       (got     'zzz))
  (c-main-enq! (lambda ()
		 (set! got (c-input-timeout-wait port conds timeout))))
  (c-main)
  (test "c-input-timeout-wait non-gui" (list port conds timeout) want got))

(let* ((pip     (pipe))    ;; pipe nothing to read
       (port    (car pip))
       (conds   '(read))
       (timeout 0.1)
       (want    #f)
       (got     'zzz))
  (c-main-enq! (lambda ()
		 (set! got (c-input-timeout-wait port conds timeout))))
  (c-main)
  (test "c-input-timeout-wait non-gui" (list port conds timeout) want got))


;;
;; datatype-name
;;

;; should have names before autoloads
(for-each (lambda (datatype)
	    (let* ((want #t)
		   (got  (string? (datatype-name datatype))))
	      (test "datatype-name" datatype want got)))
	  (datatype-list-all))

;; load all modules
(for-each datatype-proc datatype-list-current)

;; then after loads shouldn't have any strays under different keys
(for-each (lambda (datatype)
	    (let* ((want #t)
		   (got  (string? (datatype-name datatype))))
	      (test "datatype-name" datatype want got)))
	  datatype-list-current)

;;
;; datatype-list autoloads (see that they load ok)
;;

(for-each (lambda (datatype)
	    (datatype-linestyle datatype))
	  datatype-list-current)


;;
;; job-start
;;

(let* ((want #t)
       (got  'zzz))
  (job-start (lambda ()
	       (set! got #t)))
  (c-main)
  (test "job-start" #t want got))

(let* ((want #t)
       (got  'zzz))
  (job-start (lambda ()
	       (c-idle)
	       (set! got #t)))
  (c-main)
  (test "job-start idle" #t want got))

(let* ((want #t)
       (got  'zzz))
  (job-start (lambda ()
	       (c-idle)
	       (c-idle)
	       (c-idle)
	       (set! got #t)))
  (c-main)
  (test "job-start 3*idle" #t want got))

(let* ((port  (open-input-file "/dev/null"))
       (conds '(read))
       (want  #t)
       (got   'zzz))
  (job-start (lambda ()
	       (c-input-wait port conds)
	       (set! got #t)))
  (c-main)
  (test "job-start input" (list port conds) want got))

(let* ((pip   (pipe))    ;; pipe nothing to read
       (port  (car pip))
       (conds '(read))
       (want  #t)
       (got   'zzz))
  (job-start (lambda ()
	       (c-input-wait port conds)
	       (set! got #t)))
  (c-main-enq! (lambda ()
		 (c-idle)
		 (c-idle)
		 (c-idle)
		 (if (eq? #t got)
		     (error "job didn't wait"))
		 (close-port (cdr pip))))
  (c-main)
  (test "job-start pipe close" (list port conds) want got))

;;
;; job-stop
;;

(let* ((want #t)
       (got  #t)
       (job  (job-start (lambda ()
			   (c-idle)
			   (c-idle)
			   (c-idle)
			   (set! got 'stopped)))))
  (job-stop job)
  (c-main)
  (test "job-stop immediate" #f want got))

(let* ((want  #t)
       (got   'running)
       (job   (job-start (lambda ()
			   (c-idle)
			   (c-idle)
			   (c-idle)
			   (c-idle)
			   (set! got 'stopped)))))
  (c-main-enq! (lambda ()
		 (c-idle)
		 (or (eq? got 'running)
		     (error "job didn't wait:" got))
		 (job-stop job)
		 (set! got #t)))
  (c-main)
  (test "job-stop" 'idle want got))

(let* ((pip   (pipe))    ;; pipe nothing to read
       (port  (car pip))
       (conds '(read))
       (want  #t)
       (got   'running)
       (job   (job-start (lambda ()
			   (c-input-wait port conds)
			   (set! got 'stopped)))))
  (c-main-enq! (lambda ()
		 (c-idle)
		 (c-idle)
		 (c-idle)
		 (or (eq? got 'running)
		     (error "job didn't wait:" got))
		 (job-stop job)
		 (set! got #t)))
  (c-main)
  (test "job-stop input" (list port conds) want got))

(let* ((pip     (pipe))    ;; pipe nothing to read
       (port    (car pip))
       (conds   '(read))
       (timeout 20)
       (want    #t)
       (got     'running)
       (job     (job-start (lambda ()
			     (c-input-timeout-wait port conds timeout)
			     (set! got 'stopped)))))
  (c-main-enq! (lambda ()
		 (c-idle)
		 (c-idle)
		 (c-idle)
		 (or (eq? got 'running)
		     (error "job didn't wait:" got))
		 (job-stop job)
		 (set! got #t)))
  (c-main)
  (test "job-stop input-timeout" (list port conds timeout) want got))

(let* ((want  'outside)
       (got   #f)
       (job   (job-start (lambda ()
			   (dynamic-wind
			       (lambda ()
				 (set! got 'inside))
			       (lambda ()
				 (c-idle)
				 (c-idle)
				 (c-idle)
				 (c-idle))
			       (lambda ()
				 (set! got 'outside)))
			   (set! got 'stopped)))))
  (c-main-enq! (lambda ()
		 (c-idle)
		 (job-stop job)))
  (c-main)
  (test "job-stop unwind" 'idle want got))

(let* ((want  'caught)
       (got   #f)
       (job   (job-start (lambda ()
			   (catch #t
			     (lambda ()
			       (c-idle)
			       (c-idle)
			       (c-idle)
			       (c-idle))
			     (lambda args
			       (set! got 'caught)
			       (apply throw args)))
			   (set! got 'stopped)))))
  (c-main-enq! (lambda ()
		 (c-idle)
		 (job-stop job)))
  (c-main)
  (test "job-stop catch" 'idle want got))



;;
;; chart-symbol-commodity
;;

(let* ((symbol "CLF70.NYMEX")
       (want   "CL")
       (got    (chart-symbol-commodity symbol)))
  (test "chart-symbol-commodity" symbol want got))

(let* ((symbol "CL.NYMEX")
       (want   "CL")
       (got    (chart-symbol-commodity symbol)))
  (test "chart-symbol-commodity" symbol want got))

(let* ((symbol "RT JAN 05.NYMEX")
       (want   "RT")
       (got    (chart-symbol-commodity symbol)))
  (test "chart-symbol-commodity" symbol want got))

(let* ((symbol "Nat Gas Jan 2005.TOCOM")
       (want   "Nat Gas")
       (got    (chart-symbol-commodity symbol)))
  (test "chart-symbol-commodity" symbol want got))

(let* ((symbol "FOO")
       (want   "FOO")
       (got    (chart-symbol-commodity symbol)))
  (test "chart-symbol-commodity" symbol want got))

(let* ((symbol "FOO.NZ")
       (want   "FOO")
       (got    (chart-symbol-commodity symbol)))
  (test "chart-symbol-commodity" symbol want got))

(let* ((symbol "^FOO")
       (want   "^FOO")
       (got    (chart-symbol-commodity symbol)))
  (test "chart-symbol-commodity" symbol want got))

(let* ((symbol "H5.WTB")
       (want   "H5")
       (got    (chart-symbol-commodity symbol)))
  (test "chart-symbol-commodity" symbol want got))

(let* ((symbol "H5J06.WTB")
       (want   "H5")
       (got    (chart-symbol-commodity symbol)))
  (test "chart-symbol-commodity" symbol want got))

(let* ((symbol "TIN.LME")
       (want   "TIN")
       (got    (chart-symbol-commodity symbol)))
  (test "chart-symbol-commodity" symbol want got))

(let* ((symbol "TIN 3.LME")
       (want   "TIN 3")
       (got    (chart-symbol-commodity symbol)))
  (test "chart-symbol-commodity" symbol want got))

;;
;; chart-symbol-mdate
;;

(let* ((symbol "^N225")
       (want   #f)
       (got    (chart-symbol-mdate symbol)))
  (test "chart-symbol-mdate" symbol want got))

(let* ((symbol "CLF70.NYMEX")
       (want   0)
       (got    (chart-symbol-mdate symbol)))
  (test "chart-symbol-mdate" symbol want got))

(let* ((symbol "CLF71.NYMEX")
       (want   12)
       (got    (chart-symbol-mdate symbol)))
  (test "chart-symbol-mdate" symbol want got))

(let* ((symbol "CLF70")
       (want   0)
       (got    (chart-symbol-mdate symbol)))
  (test "chart-symbol-mdate" symbol want got))

(let* ((symbol "^AXJO")
       (want   #f)
       (got    (chart-symbol-mdate symbol)))
  (test "chart-symbol-mdate" symbol want got))

(let* ((symbol "FOO.AX")
       (want   #f)
       (got    (chart-symbol-mdate symbol)))
  (test "chart-symbol-mdate" symbol want got))

(let* ((symbol "Gas Oil.TOCOM")
       (want   #f)
       (got    (chart-symbol-mdate symbol)))
  (test "chart-symbol-mdate" symbol want got))

(let* ((symbol "Rubber Apr 2005.TOCOM")
       (want   (ymd->mdate 2005 4 1))
       (got    (chart-symbol-mdate symbol)))
  (test "chart-symbol-mdate" symbol want got))

(let* ((symbol "RT APR 05.SICOM")
       (want   (ymd->mdate 2005 4 1))
       (got    (chart-symbol-mdate symbol)))
  (test "chart-symbol-mdate" symbol want got))

(let* ((symbol "PP MAY 06.LME")
       (want   (ymd->mdate 2006 5 1))
       (got    (chart-symbol-mdate symbol)))
  (test "chart-symbol-mdate" symbol want got))

;;
;; chart-symbol-sans-dot
;;

(let* ((symbol "FOO.AX")
       (want   "FOO")
       (got    (chart-symbol-sans-dot symbol)))
  (test "chart-symbol-sans-dot" symbol want got))

(let* ((symbol "FOO.")
       (want   "FOO")
       (got    (chart-symbol-sans-dot symbol)))
  (test "chart-symbol-sans-dot" symbol want got))

(let* ((symbol "FOO")
       (want   "FOO")
       (got    (chart-symbol-sans-dot symbol)))
  (test "chart-symbol-sans-dot" symbol want got))

(let* ((symbol "")
       (want   "")
       (got    (chart-symbol-sans-dot symbol)))
  (test "chart-symbol-sans-dot" symbol want got))

(let* ((symbol ".")
       (want   "")
       (got    (chart-symbol-sans-dot symbol)))
  (test "chart-symbol-sans-dot" symbol want got))


;;
;; chart-symbol<?
;;

(let ((order '("^AORD"
	       "AAA.NZ"
	       "AAA BBB.NZ"
	       "CL.AX"
	       "CL.CME"
	       "CLF05.CME"
	       "CLG05.CME"
	       "CLF05.NYMEX"
	       "CLG05.NYMEX"
	       "CLF06.NYMEX"
	       "GAS.AX"
	       "Gasoil.TOCOM"
	       "GC.COMEX"
	       "RT.FOOCOM"
	       "RT JAN 05.FOOCOM"
	       "RT FEB 06.FOOCOM"
	       "RT JAN 05.SICOM"
	       "RT FEB 06.SICOM")))
  (pair-for-each
   (lambda (lst)
     (let ((x (first lst)))
       (for-each (lambda (y)
		   (let ((want #t)
			 (got  (->bool (chart-symbol<? x y))))
		     (test "chart-symbol<?" (list x y) want got))
		   (let ((want #f)
			 (got  (->bool (chart-symbol<? y x))))
		     (test "chart-symbol<?" (list y x) want got)))
		 (cdr lst))))
   order))

(let* ((x        "ALUMINIUM.LME")
       (y        "ALUMINIUM ALLOY.LME")
       (want     #t)
       (got      (->bool (chart-symbol<? x y))))
  (test "chart-symbol<?" (list x y) want got))

(let* ((x        "ALUMINIUM ALLOY.LME")
       (y        "ALUMINIUM.LME")
       (want     #f)
       (got      (->bool (chart-symbol<? x y))))
  (test "chart-symbol<?" (list x y) want got))

;;
;; chart-version
;;

(let* ((want #t)
       (got  (string? chart-version)))
  (test "chart-version" "string?" want got))

;;
;; iota-range
;;

(let* ((lo     0)
       (hi     0)
       (step   1)
       (want   '(0))
       (got    (iota-range lo hi step)))
  (test "iota-range" (list lo hi step) want got))

(let* ((lo     0)
       (hi     1)
       (step   1)
       (want   '(0 1))
       (got    (iota-range lo hi step)))
  (test "iota-range" (list lo hi step) want got))

(let* ((lo     3)
       (hi     5)
       (step   1)
       (want   '(3 4 5))
       (got    (iota-range lo hi step)))
  (test "iota-range" (list lo hi step) want got))

(let* ((lo     1)
       (hi     5)
       (step   2)
       (want   '(1 3 5))
       (got    (iota-range lo hi step)))
  (test "iota-range" (list lo hi step) want got))


;;
;; length>=?
;;

(let* ((lst  '())
       (n    0)
       (want #t)
       (got  (length>=? lst n)))
  (test "length>=?" (list lst n) want got))

(let* ((lst  '())
       (n    1)
       (want #f)
       (got  (length>=? lst n)))
  (test "length>=?" (list lst n) want got))

(let* ((lst  '(a))
       (n    0)
       (want #t)
       (got  (length>=? lst n)))
  (test "length>=?" (list lst n) want got))

(let* ((lst  '(a))
       (n    1)
       (want #t)
       (got  (length>=? lst n)))
  (test "length>=?" (list lst n) want got))

(let* ((lst  '(a))
       (n    99)
       (want #f)
       (got  (length>=? lst n)))
  (test "length>=?" (list lst n) want got))

(let* ((lst  '(a b c d))
       (n    2)
       (want #t)
       (got  (length>=? lst n)))
  (test "length>=?" (list lst n) want got))


;;
;; locale-number+decimals
;;

(set! locale-thousands-sep (lambda () "--"))
(set! locale-decimal-point (lambda () "'"))

(for-each (lambda (elem)
	    (let* ((n        (first elem))
		   (decimals (second elem))
		   (want     (third elem))
		   (got      (locale-number+decimals n decimals)))
	      (test "locale-number+decimals" (list n decimals) want got)))
	  '((0       0 "0")
	    (1       0 "1")
	    (12      0 "12")
	    (123     0 "123")
	    (1234    0 "1--234")
	    (12345   0 "12--345")
	    (123456  0 "123--456")
	    (1234567 0 "1--234--567")

	    (0        1 "0'0")
	    (1        1 "0'1")
	    (12       1 "1'2")
	    (123      1 "12'3")
	    (1234     1 "123'4")
	    (12345    1 "1--234'5")
	    (123456   1 "12--345'6")
	    (1234567  1 "123--456'7")
	    (12345678 1 "1--234--567'8")

	    (102      2 "1'02")
	    (1002     3 "1'002")

	    (-1       0 "-1")
	    (-12      0 "-12")
	    (-123     0 "-123")
	    (-1234    0 "-1--234")
	    (-12345   0 "-12--345")
	    (-123456  0 "-123--456")
	    (-1234567 0 "-1--234--567")

	    (-1        1 "-0'1")
	    (-12       1 "-1'2")
	    (-123      1 "-12'3")
	    (-1234     1 "-123'4")
	    (-12345    1 "-1--234'5")
	    (-123456   1 "-12--345'6")
	    (-1234567  1 "-123--456'7")
	    (-12345678 1 "-1--234--567'8")

	    (-102      2 "-1'02")
	    (-1002     3 "-1'002")

	    (1.0       0 "1")
	    (1.5       0 "1'5")
	    (1.5       1 "0'15")
	    (5.07      0 "5'07")))

;;
;; make-shared-array-columns
;;

(let* ((array  #2((1 2 3 4) (5 6 7 8)))
       (x      1)
       (y      3)
       (want   (make-shared-array array list '(0 1) '(1 3)))
       (got    (make-shared-array-columns array x y)))
  (test "make-shared-array-columns" (list x y array) want got))


;;
;; quotient-ceiling
;;

(for-each (lambda (elem)
	    (let* ((args (first elem))
		   (want (second elem))
		   (got  (apply quotient-ceiling args)))
	      (test "quotient-ceiling" args want got)))
	  '(((12 3)  4)
	    ((13 3)  5)
	    ((14 3)  5)
	    ((-12 3) -4)
	    ((-13 3) -4)
	    ((-14 3) -4)))

;;
;; quotient-floor
;;

(for-each (lambda (elem)
	    (let* ((args (first elem))
		   (want (second elem))
		   (got  (apply quotient-floor args)))
	      (test "quotient-floor" args want got)))
	  '(((12 3)  4)
	    ((13 3)  4)
	    ((14 3)  4)
	    ((-12 3) -4)
	    ((-13 3) -5)
	    ((-14 3) -5)))

;; negative divisors not currently used
;;
;; (let* ((args '(12 -3))
;;        (want -4)
;;        (got  (apply quotient-floor args)))
;;   (test "quotient-floor" args want got))
;; (let* ((args '(13 -3))
;;        (want -5)
;;        (got  (apply quotient-floor args)))
;;   (test "quotient-floor" args want got))
;; (let* ((args '(14 -3))
;;        (want -5)
;;        (got  (apply quotient-floor args)))
;;   (test "quotient-floor" args want got))
;; (let* ((args '(-12 -3))
;;        (want 4)
;;        (got  (apply quotient-floor args)))
;;   (test "quotient-floor" args want got))
;; (let* ((args '(-13 -3))
;;        (want 4)
;;        (got  (apply quotient-floor args)))
;;   (test "quotient-floor" args want got))
;; (let* ((args '(-14 -3))
;;        (want 4)
;;        (got  (apply quotient-floor args)))
;;   (test "quotient-floor" args want got))


;;
;; read-string-to-eof
;;

(call-with-tempfileport (mkstemp-ext "")
  (lambda (port)

    (define (fill-port str)
      (seek port 0 SEEK_SET)
      (display str port)
      (truncate-file port)
      (seek port 0 SEEK_SET))

    ;; block size used inside read-string-to-eof
    (define bufsize 4096)

    (let* ((str  "")
	   (want str)
	   (got  (begin
		   (fill-port str)
		   (read-string-to-eof port))))
      (test "read-string-to-eof" str want got))

    (let* ((str  "a")
	   (want str)
	   (got  (begin
		   (fill-port str)
		   (read-string-to-eof port))))
      (test "read-string-to-eof" str want got))

    (let* ((str  "ab")
	   (want str)
	   (got  (begin
		   (fill-port str)
		   (read-string-to-eof port))))
      (test "read-string-to-eof" str want got))

    (let* ((str  (make-random-string (1- bufsize)))
	   (want str)
	   (got  (begin
		   (fill-port str)
		   (read-string-to-eof port))))
      (test "read-string-to-eof" str want got))

    (let* ((str  (make-random-string bufsize))
	   (want str)
	   (got  (begin
		   (fill-port str)
		   (read-string-to-eof port))))
      (test "read-string-to-eof" str want got))

    (let* ((str  (make-random-string (1+ bufsize)))
	   (want str)
	   (got  (begin
		   (fill-port str)
		   (read-string-to-eof port))))
      (test "read-string-to-eof" str want got))

    (let* ((str  (make-random-string (1- (* 2 bufsize))))
	   (want str)
	   (got  (begin
		   (fill-port str)
		   (read-string-to-eof port))))
      (test "read-string-to-eof" str want got))

    (let* ((str  (make-random-string (* 2 bufsize)))
	   (want str)
	   (got  (begin
		   (fill-port str)
		   (read-string-to-eof port))))
      (test "read-string-to-eof" str want got))

    (let* ((str  (make-random-string (1+ (* 2 bufsize))))
	   (want str)
	   (got  (begin
		   (fill-port str)
		   (read-string-to-eof port))))
      (test "read-string-to-eof" str want got))))

;;
;; rec-accessor
;;

(let* ((x-rtd (make-record-type "x" '(xf)))
       (x-new (record-constructor x-rtd))
       (xr    (x-new 123))
       (y-rtd (make-record-type "y" '(yf)))
       (y-get (rec-accessor y-rtd 'yf))
       (want  'wrong-type-arg)
       (got   (catch 'wrong-type-arg
		(lambda ()
		  (y-get xr))
		(lambda (key . args)
		  key))))
  (test "rec-accessor" x-rtd want got))


;;
;; round-exact
;;

(for-each (lambda (elem)
	    (let* ((x    (first elem))
		   (want (second elem))
		   (got  (round-exact x)))
	      (test "round-exact" x want got)))
	  '((3     3)
	    (3/4   1)
	    (1/4   0)
	    (-4/3  -1)
	    (-1.25 -1)))

;;
;; series-datatype
;;

(let* ((x-rtd (make-record-type "x" '(xf)))
       (x-new (record-constructor x-rtd))
       (xr    (x-new 123))
       (want  'wrong-type-arg)
       (got   (catch 'wrong-type-arg
		(lambda ()
		  (series-datatype xr))
		(lambda (key . args)
		  key))))
  (test "series-datatype" xr want got))

(let* ((obj   datatype-ohlcv)
       (want  'wrong-type-arg)
       (got   (catch 'wrong-type-arg
		(lambda ()
		  (series-datatype obj))
		(lambda (key . args)
		  key))))
  (test "series-datatype" obj want got))


;;
;; series-today
;;

(define (wdate-today . zone)
  (apply ymd->wdate (apply today-ymd zone)))

(let* ((series (series-new #:symbol "FOO.AX"
			   #:timebase timebase-days))
       (want   (tdate-today (timezone-sydney)))
       (got    (series-today series)))
  (test "series-today" series want got))

(let* ((series (series-new #:symbol "BLT.L"
			   #:timebase timebase-weeks))
       (want   (wdate-today (timezone-london)))
       (got    (series-today series)))
  (test "series-today" series want got))

(let* ((series (series-new #:symbol "GM"
			   #:timebase timebase-months))
       (want   (mdate-today (timezone-newyork)))
       (got    (series-today series)))
  (test "series-today" series want got))


;;
;; string-decimals
;;

(for-each (lambda (elem)
	    (let* ((str  (first elem))
		   (want (second elem))
		   (got  (string-decimals str)))
	      (test "string-decimals" str want got)))
	  '(("123"    0)
	    ("123."   0)
	    ("123.4"  1)
	    ("123.45" 2)))

;;
;; string-join-notnull
;;

(for-each (lambda (elem)
	    (let* ((lst  (first elem))
		   (want (second elem))
		   (got  (string-join-notnull lst)))
	      (test "string-join-notnull" lst want got)))
	  '((()         "")
	    ((#f)       "")
	    ((#f #f)    "")
	    ((#f "" #f) "")
	    (("abc" #f "def") "abc def")))

;;
;; string->number+decimals
;;

(for-each (lambda (elem)
	    (let* ((str      (first elem))
		   (decimals (second elem))
		   (want     (third elem))
		   (got      (string->number+decimals str decimals)))
	      (test "string->number+decimals" elem want got)))

	  '(("1" 0 1)
	    ("1" 1 10)
	    ("1." 0 1)
	    ("1." 1 10)
	    ("1.5" 1 15)
	    ("1.5" 2 150)
	    ("10" -1 1)
	    ("100" -1 10)
	    ("150" -2 1.5)
	    ("0" -100 0)
	    ("00" -100 0)
	    ("000" -100 0)
	    ("0" 100 0)
	    ("00" 100 0)
	    ("000" 100 0)

	    ("+0"    5 0)
	    ("-0"    5 0)
	    ("+0.00" 5 0)
	    ("-0.00" 5 0)
	    ("+.0"   5 0)
	    ("-.0"   5 0)
	    ("+0."   5 0)
	    ("-0."   5 0)))

;;
;; strings->numbers+decimals
;;

(for-each (lambda (elem)
	    (let* ((str-lst  (first elem))
		   (want     (second elem))
		   (got      (apply strings->numbers+decimals str-lst)))
	      (test "strings->numbers+decimals" str-lst want got)))
	  '((("1.5")    (1 15))
	    (("1.5" #f) (1 15 #f))
	    (("1.5" "") (1 15 #f))
	    ((#f)       (0 #f))
	    (("" "")    (0 #f #f))
	    (("1" "1." "1.5") (1 10 10 15))
	    (("1.8" "1.82")   (2 180 182))))

;;
;; sublist
;;

(let* ((lst   '())
       (start 0)
       (end   0)
       (want  '())
       (got   (sublist lst start end)))
  (test "sublist" (list lst start end) want got))

(let* ((lst   '(a b c d))
       (start 1)
       (end   3)
       (want  '(b c))
       (got   (sublist lst start end)))
  (test "sublist" (list lst start end) want got))

(let* ((lst   '(a b c d))
       (start 1)
       (end   4)
       (want  '(b c d))
       (got   (sublist lst start end)))
  (test "sublist" (list lst start end) want got))

(let* ((lst   '(a b c d))
       (start 0)
       (end   3)
       (want  '(a b c))
       (got   (sublist lst start end)))
  (test "sublist" (list lst start end) want got))


;;
;; sympred-assoc
;;

(let* ((alist (list (cons (lambda (x) #f) 'a)
		    (cons (lambda (x) #t) 'b)))
       (key   'c)
       (want  (second alist))
       (got   (sympred-assoc key alist)))
  (test "sympred-assoc" (list alist key) want got))

;;
;; sympred-assoc-ref
;;

(let* ((alist (list (cons (lambda (x) #f) 'a)
		    (cons (lambda (x) #t) 'b)))
       (key   'c)
       (want  'b)
       (got   (sympred-assoc-ref alist key)))
  (test "sympred-assoc-ref" (list alist key) want got))

;;
;; symbol-setups-alist
;;

(define symbol-setups-alist (@@ (chart misc) symbol-setups-alist))

(for-each (lambda (elem)
	    (let* ((pred (car elem))
		   (want #t)
		   (got  (sympred? pred)))
	      (test "symbol-setups-alist -- sympred?" elem want got)))
	  symbol-setups-alist)

(format #t "total ~a symbol-setups\n" (symbol-setups-load-everything))


;;
;; symbol-timezone-list
;;

(define symbol-timezone-list (@@ (chart misc) symbol-timezone-list))

(for-each (lambda (elem)
	    (let* ((pred (car elem))
		   (want #t)
		   (got  (sympred? pred)))
	      (test "symbol-timezone-list -- sympred?" elem want got)))
	  symbol-timezone-list)

;;
;; two<
;;

(for-each (lambda (elem)
	    (let* ((args (first elem))
		   (want (second elem))
		   (got  (apply two< args)))
	      (test "two<" args want got)))
	  '(((0 0  0 0) 	#f)
	    ((1 0  0 0) 	#f)
	    ((0 1  0 0) 	#f)
	    ((0 0  1 0) 	#t)
	    ((0 0  0 1) 	#t)

	    ((1 0  1 0) 	#f)
	    ((0 1  0 1) 	#f)
	    ((1 0  0 1) 	#f)
	    ((0 1  1 0) 	#t)

	    ((0 1  1 1) 	#t)
	    ((1 0  1 1) 	#t)
	    ((1 1  0 1) 	#f)
	    ((1 1  1 0) 	#f)))

;;
;; weblink-handler-list
;;

(for-each (lambda (elem)
	    (let* ((pred (first elem))
		   (got  (sympred? pred))
		   (want #t))
	      (test "weblink-handler-list -- sympred?" elem want got)))
	  weblink-handler-list)


(tests-end)
