;; Test newzealand.scm functions.

;; Copyright 2006, 2007 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(load "tests.scm")


;;
;; source-help
;;

(let* ((symbol "TEL.NZ")
       (want   (_ "CONTEXT(manual-node): New Zealand Stock Exchange"))
       (got    (source-help symbol)))
  (test "source-help" symbol want got))

(let* ((symbol "^NZ50")
       (want   (_ "CONTEXT(manual-node): New Zealand Stock Exchange"))
       (got    (source-help symbol)))
  (test "source-help" symbol want got))

;;
;; newzealand-dividend-parse
;;

(define newzealand-dividend-parse
  (@@ (chart newzealand) newzealand-dividend-parse))

(let* ((args '("something" "0.0000"))
       (want "something")
       (got  (apply newzealand-dividend-parse args)))
  (test "newzealand-dividend-parse" args want got))

(let* ((args '("0.55" "0.0000"))
       (want '(amount 55 (decimals . 4)))
       (got  (apply newzealand-dividend-parse args)))
  (test "newzealand-dividend-parse" args want got))

(let* ((args '("0.55 NZD" "0.0000"))
       (want '(amount 55 (decimals . 4)))
       (got  (apply newzealand-dividend-parse args)))
  (test "newzealand-dividend-parse" args want got))

(let* ((args '("15.00 NZD" "7.3881"))
       (want '(amount 150000 (decimals . 6) (imputation . 73881)))
       (got  (apply newzealand-dividend-parse args)))
  (test "newzealand-dividend-parse" args want got))

(let* ((args '("15.00 GBP" "7.3881"))
       (want "15.00 GBP + 7.3881")
       (got  (apply newzealand-dividend-parse args)))
  (test "newzealand-dividend-parse" args want got))


;;
;; symbol-location
;;

(let* ((symbol "CAH.NZ")
       (want   (_ "New Zealand"))
       (got    (symbol-location symbol)))
  (test "symbol-location" symbol want got))

;;
;; yahoo-quote-delay
;;

(define yahoo-quote-delay (@@ (chart yahoo) yahoo-quote-delay))

(let* ((symbol "CAH.NZ")
       (want   (hms->seconds 0 20 0))
       (got    (begin
		 (symbol-setups symbol)
		 (yahoo-quote-delay symbol ""))))
  (test "yahoo-quote-delay" symbol want got))

(let* ((symbol "^NZ50")
       (want   0)
       (got    (begin
		 (symbol-setups symbol)
		 (yahoo-quote-delay symbol ""))))
  (test "yahoo-quote-delay" symbol want got))


(tests-end)
