;; Exercise obv.scm.

;; Copyright 2006, 2007 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(load "tests.scm")
(use-modules (chart obv))

(define (close+volume->series lst)
  (define array (make-array #f (length lst) '(0 4)))
  (for-each (lambda (i elem)
	      (array-set! array (first elem)  i 3)   ;; close
	      (array-set! array (second elem) i 4))  ;; volume
	    (iota (length lst)) lst)
  (series-new #:symbol   "FOO"
	      #:array    array
	      #:datatype datatype-ohlcv))

;;
;; obv-series
;;

(let* ((data       '((200  1)
		     (210  20)
		     (220  30)))
       (want       '(0 20 50))
       (parent     (close+volume->series data))
       (got-series (obv-series parent))
       (got        (array->list (series-array got-series 0 2))))
  (test "obv-series" data want got))

(let* ((data       '((200 1)
		     (210 20)
		     (190 30)
		     (191 40)))
       (want       '(0 20 -10 30))
       (parent     (close+volume->series data))
       (got-series (obv-series parent))
       (got        (array->list (series-array got-series 0 3))))
  (test "obv-series" data want got))

(let* ((data       '((200 1)
		     (210 20)
		     (190 30)
		     (191 40)))
       (want       '(0 20 -10 30))
       (parent     (close+volume->series data))
       (got-series (obv-series parent))
       (got        (begin
		     (series-array got-series 0 2)
		     (array->list (series-array got-series 0 3)))))
  (test "obv-series" data want got))

(let* ((data       '((200 1)
		     (210 20)
		     (190 30)
		     (191 40)))
       (want       '(-30 -10 -40 0))
       (parent     (close+volume->series data))
       (got-series (obv-series parent))
       (got        (begin
		     (series-array got-series 3 3)
		     (array->list (series-array got-series 0 3)))))
  (test "obv-series" data want got))


;; one non-#f

(let* ((data       '((#f 1)
		     (210 20)
		     (#f 30)
		     (#f 40)))
       (want       '(#f 0 #f #f))
       (parent     (close+volume->series data))
       (got-series (obv-series parent))
       (got        (array->list (series-array got-series 0 3))))
  (test "obv-series 0-3" data want got))

(let* ((data       '((#f 1)
		     (210 20)
		     (#f 30)
		     (#f 40)))
       (want       '(#f 0 #f #f))
       (parent     (close+volume->series data))
       (got-series (obv-series parent))
       (got        (begin
		     (series-array got-series 3 3)
		     (array->list (series-array got-series 0 3)))))
  (test "obv-series 3-3 then 0-3" data want got))


;; all #f's

(let* ((data       '((#f 1)
		     (#f 20)
		     (#f 30)
		     (#f 40)))
       (want       '(#f #f #f #f))
       (parent     (close+volume->series data))
       (got-series (obv-series parent))
       (got        (array->list (series-array got-series 0 3))))
  (test "obv-series" data want got))

(let* ((data       '((#f 1)
		     (#f 20)
		     (#f 30)
		     (#f 40)))
       (want       '(#f #f #f #f))
       (parent     (close+volume->series data))
       (got-series (obv-series parent))
       (got        (begin
		     (series-array got-series 3 3)
		     (array->list (series-array got-series 0 3)))))
  (test "obv-series" data want got))



(tests-end)
