;; Test sfe.scm functions.

;; Copyright 2005, 2006, 2007 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(load "tests.scm")
(use-modules (chart newzealand))


;;
;; symbol-timezone
;;

(let* ((symbol "XT.SFE")
       (want   (timezone-sydney))
       (got    (symbol-timezone symbol)))
  (test "symbol-timezone" symbol want got))

(let* ((symbol "BB DEC 05.SFE")
       (want   (timezone-newzealand))
       (got    (symbol-timezone symbol)))
  (test "symbol-timezone" symbol want got))

;;
;; sfe-available-tdate
;;

(define sfe-available-tdate (@@ (chart sfe) sfe-available-tdate))

(let ((ymdhm  '(2005 1 6  7 0))) ;; Thursday
  (apply fake-localtime-ymdhm! (timezone-sydney) ymdhm)
  (let ((want (ymd->tdate 2005 1 4))
	(got  (sfe-available-tdate)))
    (test "sfe-available-tdate" ymdhm want got)))

(let ((ymdhm  '(2005 1 6  8 0))) ;; Thursday
  (apply fake-localtime-ymdhm! (timezone-sydney) ymdhm)
  (let ((want (ymd->tdate 2005 1 5))
	(got  (sfe-available-tdate)))
    (test "sfe-available-tdate" ymdhm want got)))

(let ((ymdhm  '(2005 1 7  7 0))) ;; Friday
  (apply fake-localtime-ymdhm! (timezone-sydney) ymdhm)
  (let ((want (ymd->tdate 2005 1 5))
	(got  (sfe-available-tdate)))
    (test "sfe-available-tdate" ymdhm want got)))

(let ((ymdhm  '(2005 1 7  8 0))) ;; Friday
  (apply fake-localtime-ymdhm! (timezone-sydney) ymdhm)
  (let ((want (ymd->tdate 2005 1 6))
	(got  (sfe-available-tdate)))
    (test "sfe-available-tdate" ymdhm want got)))

(let ((ymdhm  '(2005 1 8  7 0))) ;; Saturday
  (apply fake-localtime-ymdhm! (timezone-sydney) ymdhm)
  (let ((want (ymd->tdate 2005 1 6))
	(got  (sfe-available-tdate)))
    (test "sfe-available-tdate" ymdhm want got)))

(let ((ymdhm  '(2005 1 8  8 0))) ;; Saturday
  (apply fake-localtime-ymdhm! (timezone-sydney) ymdhm)
  (let ((want (ymd->tdate 2005 1 7))
	(got  (sfe-available-tdate)))
    (test "sfe-available-tdate" ymdhm want got)))

(let ((ymdhm  '(2005 1 9  7 0))) ;; Sunday
  (apply fake-localtime-ymdhm! (timezone-sydney) ymdhm)
  (let ((want (ymd->tdate 2005 1 7))
	(got  (sfe-available-tdate)))
    (test "sfe-available-tdate" ymdhm want got)))

(let ((ymdhm  '(2005 1 9  8 0))) ;; Sunday
  (apply fake-localtime-ymdhm! (timezone-sydney) ymdhm)
  (let ((want (ymd->tdate 2005 1 7))
	(got  (sfe-available-tdate)))
    (test "sfe-available-tdate" ymdhm want got)))

(let ((ymdhm  '(2005 1 10  7 0))) ;; Monday
  (apply fake-localtime-ymdhm! (timezone-sydney) ymdhm)
  (let ((want (ymd->tdate 2005 1 7))
	(got  (sfe-available-tdate)))
    (test "sfe-available-tdate" ymdhm want got)))

(let ((ymdhm  '(2005 1 10  8 0))) ;; Monday
  (apply fake-localtime-ymdhm! (timezone-sydney) ymdhm)
  (let ((want (ymd->tdate 2005 1 7))
	(got  (sfe-available-tdate)))
    (test "sfe-available-tdate" ymdhm want got)))

(let ((ymdhm  '(2005 1 11  7 0))) ;; Tuesday
  (apply fake-localtime-ymdhm! (timezone-sydney) ymdhm)
  (let ((want (ymd->tdate 2005 1 7))
	(got  (sfe-available-tdate)))
    (test "sfe-available-tdate" ymdhm want got)))

(let ((ymdhm  '(2005 1 11  8 0))) ;; Tuesday
  (apply fake-localtime-ymdhm! (timezone-sydney) ymdhm)
  (let ((want (ymd->tdate 2005 1 10))
	(got  (sfe-available-tdate)))
    (test "sfe-available-tdate" ymdhm want got)))

(let ((ymdhm  '(2005 1 12  7 0))) ;; Wednesday
  (apply fake-localtime-ymdhm! (timezone-sydney) ymdhm)
  (let ((want (ymd->tdate 2005 1 10))
	(got  (sfe-available-tdate)))
    (test "sfe-available-tdate" ymdhm want got)))

(let ((ymdhm  '(2005 1 12  8 0))) ;; Wednesday
  (apply fake-localtime-ymdhm! (timezone-sydney) ymdhm)
  (let ((want (ymd->tdate 2005 1 11))
	(got  (sfe-available-tdate)))
    (test "sfe-available-tdate" ymdhm want got)))


(tests-end)
