;; Test symlists.scm.

;; Copyright 2006, 2007 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(load "tests.scm")
(use-modules (chart database)
	     (chart extra-glib)
	     (chart symlists))


(set! user-directory        (g-build-filename (getcwd) "database-temp"))
(set! database-directory    (g-build-filename (getcwd) "database-temp"))
(set! data-directory        (g-build-filename database-directory "data"))
(set! cache-directory       (g-build-filename database-directory "cache"))
(set! user-notes-directory  (g-build-filename database-directory "notes"))


;;
;; merge-one-index
;;

(define merge-one-index (@@ (chart symlists) merge-one-index))

(let* ((lst  '())
       (elem 1)
       (want 0)
       (got  (merge-one-index lst elem <)))
  (test "merge-one-index" (list elem lst) want got))

(let* ((lst  '(5))
       (elem 1)
       (want 0)
       (got  (merge-one-index lst elem <)))
  (test "merge-one-index" (list elem lst) want got))

(let* ((lst  '(5))
       (elem 7)
       (want 1)
       (got  (merge-one-index lst elem <)))
  (test "merge-one-index" (list elem lst) want got))

(let* ((lst  '(5 10 15))
       (elem 1)
       (want 0)
       (got  (merge-one-index lst elem <)))
  (test "merge-one-index" (list elem lst) want got))

(let* ((lst  '(5 10 15))
       (elem 6)
       (want 1)
       (got  (merge-one-index lst elem <)))
  (test "merge-one-index" (list elem lst) want got))

(let* ((lst  '(5 10 15))
       (elem 12)
       (want 2)
       (got  (merge-one-index lst elem <)))
  (test "merge-one-index" (list elem lst) want got))

(let* ((lst  '(5 10 15))
       (elem 17)
       (want 3)
       (got  (merge-one-index lst elem <)))
  (test "merge-one-index" (list elem lst) want got))

;;
;; smarker symlist-delete
;;

(let* ((symlist (symlist-new #:key 'foo))
       (smarker (smarker-new)))
  (symlist-list! symlist '("ONE" "TWO" "THREE" "FOUR"))
  (smarker-goto smarker symlist "TWO")
  (symlist-delete symlist 1 1)
  (let ((want "THREE")
	(got  (smarker-next smarker)))
    (test "smarker symlist-delete" symlist want got)))

;;
;; sublist-maybe
;;

(define sublist-maybe (@@ (chart symlists) sublist-maybe))

(let* ((lst   '())
       (start 0)
       (end   0)
       (want  '())
       (got   (sublist-maybe lst start end)))
  (test "sublist-maybe" (list lst start end) want got))

(let* ((lst   '(a b c d))
       (start 1)
       (end   3)
       (want  '(b c))
       (got   (sublist-maybe lst start end)))
  (test "sublist-maybe" (list lst start end) want got))

(let* ((lst   '(a b c d))
       (start 1)
       (end   4)
       (want  '(b c d))
       (got   (sublist-maybe lst start end)))
  (test "sublist-maybe" (list lst start end) want got))

(let* ((lst   '(a b c d))
       (start 0)
       (end   3)
       (want  '(a b c))
       (got   (sublist-maybe lst start end)))
  (test "sublist-maybe" (list lst start end) want got))

(let* ((lst   '(a b c d))
       (start 0)
       (end   10)
       (want  '(a b c d))
       (got   (sublist-maybe lst start end)))
  (test "sublist-maybe" (list lst start end) want got))

(let* ((lst   '(a b c d))
       (start 10)
       (end   20)
       (want  '())
       (got   (sublist-maybe lst start end)))
  (test "sublist-maybe" (list lst start end) want got))

(let* ((lst   '(a b c d))
       (start 3)
       (end   5)
       (want  '(d))
       (got   (sublist-maybe lst start end)))
  (test "sublist-maybe" (list lst start end) want got))

;;
;; smarker-goto-first-of-symlist
;;

(let* ((symlist (symlist-new #:key 'foo))
       (smarker (smarker-new))
       (want    #f)
       (got     (smarker-goto-first-of-symlist smarker symlist)))
  (test "smarker-goto-first-of-symlist" symlist want got))

(let* ((symlist (symlist-new #:key 'foo))
       (smarker (smarker-new)))
  (symlist-list! symlist '("ONE"))
  (let ((want    "ONE")
	(got     (smarker-goto-first-of-symlist smarker symlist)))
    (test "smarker-goto-first-of-symlist" symlist want got)))

(let* ((symlist (symlist-new #:key 'foo))
       (smarker (smarker-new)))
  (symlist-list! symlist '("ONE" "TWO" "THREE"))
  (let ((want    "ONE")
	(got     (smarker-goto-first-of-symlist smarker symlist)))
    (test "smarker-goto-first-of-symlist" symlist want got)))


(tests-end)
