;; Exercise unzip.scm.

;; Copyright 2006, 2007 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(load "tests.scm")
(use-modules (chart gzip)
	     (chart unzip))

(define (string-or-f? obj)
  (or (string? obj)
      (eq? #f obj)))

;; return true if the "zip" program is available
(define (zip-available?)
  (program-version-proc "zip" noop))

(define (slurp filename)
  (call-with-input-file filename read-string-to-eof))


;;
;; read-input-pipe
;;

(define read-input-pipe (@@ (chart unzip) read-input-pipe))

(let* ((command "echo hello")
       (want    "hello\n")
       (got     (read-input-pipe command)))
  (test "read-input-pipe" command want got))


;;
;; unzip-version
;;

(let* ((want #t)
       (got  (string-or-f? (unzip-version))))
  (test "unzip-version" "string-or-f?" want got))


;;
;; unzip-string
;;

(if (and (unzip-version)
	 (zip-available?))
    (begin

      ;; empty zip file
      (system "zip testfile.zip Makefile >/dev/null;
               zip -d testfile.zip Makefile >/dev/null 2>/dev/null")
      (let* ((want "")
	     (got  (unzip-string (slurp "testfile.zip"))))
	(test "unzip-string" "empty" want got))
      (delete-file "testfile.zip")))



(tests-end)
