;; Test yahoo functions.

;; Copyright 2005, 2006, 2007, 2009 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(load "tests.scm")


;;
;; close-fraction
;;

(define close-fraction (@@ (chart yahoo) close-fraction))

(let* ((x    1)
       (want '(1 1))
       (got  (close-fraction x)))
  (test "close-fraction" x want got))

(let* ((x    1.5)
       (want '(3 2))
       (got  (close-fraction x)))
  (test "close-fraction" x want got))

(let* ((x    2.0001)
       (want '(2 1))
       (got  (close-fraction x)))
  (test "close-fraction" x want got))

(let* ((x    1.999)
       (want '(2 1))
       (got  (close-fraction x)))
  (test "close-fraction" x want got))

(let* ((x    1.49999)
       (want '(3 2))
       (got  (close-fraction x)))
  (test "close-fraction" x want got))

(let* ((x    1.1)
       (want '(11 10))
       (got  (close-fraction x)))
  (test "close-fraction" x want got))

(let* ((x    3.25)
       (want '(13 4))
       (got  (close-fraction x)))
  (test "close-fraction" x want got))

;;
;; div-date-str->tdate
;;

(define div-date-str->tdate (@@ (chart yahoo) div-date-str->tdate))
(define old-month->nearest-year month->nearest-year)
(set! month->nearest-year (lambda (month)
			    1970))

(let* ((str  "N/A")
       (want #f)
       (got  (div-date-str->tdate str)))
  (test "div-date-str->tdate" str want got))

(let* ((str  "Jan  6")
       (want 1)
       (got  (div-date-str->tdate str)))
  (test "div-date-str->tdate" str want got))

(let* ((str  " 5 Jan")
       (want 0)
       (got  (div-date-str->tdate str)))
  (test "div-date-str->tdate" str want got))

(let* ((str  "31 Dec")
       (want (ymd->tdate 1970 12 31))
       (got  (div-date-str->tdate str)))
  (test "div-date-str->tdate" str want got))

(let* ((str  " 5-Jan-70")
       (want 0)
       (got  (div-date-str->tdate str)))
  (test "div-date-str->tdate" str want got))

(let* ((str  " 5 Jan, 1970")
       (want 0)
       (got  (div-date-str->tdate str)))
  (test "div-date-str->tdate" str want got))

(let* ((str  "Jan  5, 1970")
       (want 0)
       (got  (div-date-str->tdate str)))
  (test "div-date-str->tdate" str want got))

(set! month->nearest-year old-month->nearest-year)

;;
;; hist-date-str->tdate
;;

(define hist-date-str->tdate (@@ (chart yahoo) hist-date-str->tdate))
(define old-month->nearest-year month->nearest-year)
(set! month->nearest-year (lambda (month)
			    1970))

(let* ((str  "zzz")
       (want 'exception)
       (got  (catch #t
	       (lambda ()
		 (hist-date-str->tdate str))
	       (lambda args
		 'exception))))
  (test "hist-date-str->tdate" str want got))

(let* ((str  "5-Jan-70")
       (want 0)
       (got  (hist-date-str->tdate str)))
  (test "hist-date-str->tdate" str want got))

(let* ((str  "6-Jan-70")
       (want 1)
       (got  (hist-date-str->tdate str)))
  (test "hist-date-str->tdate" str want got))

(let* ((str  "1970-01-05")
       (want 0)
       (got  (hist-date-str->tdate str)))
  (test "hist-date-str->tdate" str want got))

(let* ((str  "1970-01-06")
       (want 1)
       (got  (hist-date-str->tdate str)))
  (test "hist-date-str->tdate" str want got))

(set! month->nearest-year old-month->nearest-year)


;;
;; nearly-integer?
;;

(define nearly-integer? (@@ (chart yahoo) nearly-integer?))

(let* ((x    1)
       (want #t)
       (got  (nearly-integer? x)))
  (test "nearly-integer?" x want got))

(let* ((x    1.0001)
       (want #t)
       (got  (nearly-integer? x)))
  (test "nearly-integer?" x want got))

(let* ((x    2.99999)
       (want #t)
       (got  (nearly-integer? x)))
  (test "nearly-integer?" x want got))

(let* ((x    2.5)
       (want #f)
       (got  (nearly-integer? x)))
  (test "nearly-integer?" x want got))

;;
;; symbol-timezone
;;

(let* ((symbol "^TV.N")  ;; NYSE total volume
       (want   (timezone-newyork))
       (got    (symbol-timezone symbol)))
  (test "symbol-timezone" symbol want got))

(let* ((symbol "^TV.O")  ;; NASDAQ total volume
       (want   (timezone-newyork))
       (got    (symbol-timezone symbol)))
  (test "symbol-timezone" symbol want got))

;;
;; tm-date<
;;

(define tm-date< (@@ (chart yahoo) tm-date<))

(let* ((x-ymd '(2005 6 6))
       (y-ymd '(2005 6 6))
       (want  #f)
       (got   (tm-date< (apply ymd->tm x-ymd) (apply ymd->tm y-ymd))))
  (test "tm-date<" (list x-ymd y-ymd) want got))

(let* ((x-ymd '(2005 6 7))
       (y-ymd '(2005 6 8))
       (want  #t)
       (got   (tm-date< (apply ymd->tm x-ymd) (apply ymd->tm y-ymd))))
  (test "tm-date<" (list x-ymd y-ymd) want got))

(let* ((x-ymd '(2005 6 8))
       (y-ymd '(2005 6 7))
       (want  #f)
       (got   (tm-date< (apply ymd->tm x-ymd) (apply ymd->tm y-ymd))))
  (test "tm-date<" (list x-ymd y-ymd) want got))

(let* ((x-ymd '(2005 7 9))
       (y-ymd '(2005 8 6))
       (want  #t)
       (got   (tm-date< (apply ymd->tm x-ymd) (apply ymd->tm y-ymd))))
  (test "tm-date<" (list x-ymd y-ymd) want got))

(let* ((x-ymd '(2005 8 6))
       (y-ymd '(2005 7 9))
       (want  #f)
       (got   (tm-date< (apply ymd->tm x-ymd) (apply ymd->tm y-ymd))))
  (test "tm-date<" (list x-ymd y-ymd) want got))

(let* ((x-ymd '(2004 9 9))
       (y-ymd '(2005 1 1))
       (want  #t)
       (got   (tm-date< (apply ymd->tm x-ymd) (apply ymd->tm y-ymd))))
  (test "tm-date<" (list x-ymd y-ymd) want got))

(let* ((x-ymd '(2006 1 1))
       (y-ymd '(2005 7 9))
       (want  #f)
       (got   (tm-date< (apply ymd->tm x-ymd) (apply ymd->tm y-ymd))))
  (test "tm-date<" (list x-ymd y-ymd) want got))

;;
;; yahoo-dividend-parse
;;

(define yahoo-dividend-parse (@@ (chart yahoo) yahoo-dividend-parse))

(let* ((str  "1.23")
       (want '(amount 123 (decimals . 2)))
       (got  (yahoo-dividend-parse str)))
  (test "yahoo-dividend-parse" str want got))

(let* ((str  "1.230")
       (want '(amount 123 (decimals . 2)))
       (got  (yahoo-dividend-parse str)))
  (test "yahoo-dividend-parse" str want got))

(let* ((str  "1.23000000000000")
       (want '(amount 123 (decimals . 2)))
       (got  (yahoo-dividend-parse str)))
  (test "yahoo-dividend-parse" str want got))

(let* ((str  "25c")
       (want '(amount 25 (decimals . 2)))
       (got  (yahoo-dividend-parse str)))
  (test "yahoo-dividend-parse" str want got))

(let* ((str  "9.5c")
       (want '(amount 95 (decimals . 3)))
       (got  (yahoo-dividend-parse str)))
  (test "yahoo-dividend-parse" str want got))

(let* ((str  "something")
       (want "something")
       (got  (yahoo-dividend-parse str)))
  (test "yahoo-dividend-parse" str want got))


;;
;; ymd-hmx-strs->adate-time
;;

(define ymd-hmx-strs->adate-time (@@ (chart yahoo) ymd-hmx-strs->adate-time))

(let* ((data '("1/5/99999" "10:00am" "GMT" "GMT"))
       (want '(#f #f))
       (got  (apply ymd-hmx-strs->adate-time data)))
  (test "ymd-hmx-strs->adate-time" data want got))

(let* ((data '("N/A" "10:00am" "GMT" "GMT"))
       (want '(#f #f))
       (got  (apply ymd-hmx-strs->adate-time data)))
  (test "ymd-hmx-strs->adate-time" data want got))

(let* ((data '("1/5/2005" "10:00am" "GMT" "GMT"))
       (want (list (ymd->adate 2005 1 5) (hms->seconds 10 0 0)))
       (got  (apply ymd-hmx-strs->adate-time data)))
  (test "ymd-hmx-strs->adate-time" data want got))

(let* ((data '("1/5/2005" "12:00am" "GMT" "CET-1"))
       (want (list (ymd->adate 2005 1 5) (hms->seconds 1 0 0)))
       (got  (apply ymd-hmx-strs->adate-time data)))
  (test "ymd-hmx-strs->adate-time" data want got))

(let* ((data '("1/5/2005" "12:00am" "GMT" "EST+6"))
       (want (list (ymd->adate 2005 1 4) (hms->seconds 18 0 0)))
       (got  (apply ymd-hmx-strs->adate-time data)))
  (test "ymd-hmx-strs->adate-time" data want got))

(let* ((data '("1/5/2005" "7:00pm" "EST+6" "GMT"))
       (want (list (ymd->adate 2005 1 6) (hms->seconds 1 0 0)))
       (got  (apply ymd-hmx-strs->adate-time data)))
  (test "ymd-hmx-strs->adate-time" data want got))

(let* ((data '("1/4/2005" "5:00pm" "EST+6" "GMT"))
       (want (list (ymd->adate 2005 1 4) (hms->seconds 23 0 0)))
       (got  (apply ymd-hmx-strs->adate-time data)))
  (test "ymd-hmx-strs->adate-time" data want got))

(let* ((data '("1/5/2005" "11:00am" "AET-10" "GMT"))
       (want (list (ymd->adate 2005 1 5) (hms->seconds 1 0 0)))
       (got  (apply ymd-hmx-strs->adate-time data)))
  (test "ymd-hmx-strs->adate-time" data want got))

(let* ((data '("1/4/2005" "9:00am" "AET-10" "GMT"))
       (want (list (ymd->adate 2005 1 3) (hms->seconds 23 0 0)))
       (got  (apply ymd-hmx-strs->adate-time data)))
  (test "ymd-hmx-strs->adate-time" data want got))

(let* ((data '("1/4/2005" "5:00pm" "EST+6" "AET-10"))
       (want (list (ymd->adate 2005 1 5) (hms->seconds 9 0 0)))
       (got  (apply ymd-hmx-strs->adate-time data)))
  (test "ymd-hmx-strs->adate-time" data want got))

(let* ((data '("1/4/2005" "9:00am" "AET-10" "EST+6"))
       (want (list (ymd->adate 2005 1 3) (hms->seconds 17 0 0)))
       (got  (apply ymd-hmx-strs->adate-time data)))
  (test "ymd-hmx-strs->adate-time" data want got))

(tests-end)
