;; Ticker as standalone program.

;; Copyright 2006, 2007 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(define-module (chart ticker-main)
  #:use-module (srfi srfi-1)
  #:use-module (gtk gtk)
  #:use-module (gtk gdk)
  #:use-module (chart config)
  #:use-module (chart extra-gtk)
  #:use-module (chart gui-common)
  #:use-module (chart gnome)
  #:use-module (chart keymap)
  #:use-module (chart misc)
  #:use-module (chart symlists)
  #:use-module (chart ticker))


(define toplevel #f)
(define ticker   #f)
(define accel-group #f)

(define (toggle-run)
  (ticker-run! ticker (not (ticker-run ticker))))

(define (go-right)
  (ticker-scroll-chars ticker -6))

(define (go-left)
  (ticker-scroll-chars ticker 6))

(define (help)
  (manual-popup (_ "CONTEXT(manual-node): Ticker") toplevel))

(define (add-accelerator item keystr)
  (receive-list (keyval mods)
      (gtk-accelerator-parse keystr)
    (gtk-widget-add-accelerator item "activate" accel-group
				keyval mods '(visible))
    ;; with <Shift>, this is essential for "!" which can usually
    ;; only be typed with shift in the modifiers
    (gtk-widget-add-accelerator item "activate" accel-group keyval
				(cons 'shift-mask mods) '())))

(define (menu-additions menu)
  (let ((item (gtk-image-menu-item-new-from-stock "gtk-help" accel-group)))
    (gtk-menu-shell-append menu item)
    (gtk-signal-connect item "activate" help)
    (add-accelerator item "F1"))

  (let ((item (gtk-image-menu-item-new-from-stock "gtk-quit" accel-group)))
    (gtk-menu-shell-append menu item)
    (gtk-signal-connect item "activate" exit)
    (add-accelerator item "<Ctrl>Q")))

;; return true if more than one of ARGS is true
(define (multiple-of? . args)
  (> (count noop args) 1))

(define-public (ticker-main option-all option-alert option-favourites
			    specified-symbols symbol-list)
  (c-main-use-gtk)

  (let ((keymap   (keymap-new "Ticker" (lambda () #t))))

    (set! toplevel (gtk-window-new 'toplevel))
    (gtk-signal-connect toplevel "destroy" (lambda () (exit 0)))

    (set! accel-group (gtk-accel-group-new))
    (gtk-window-add-accel-group toplevel accel-group)

    (set! ticker (ticker-new menu-additions))
    (gtk-container-add toplevel ticker)

    (let ((progobj (gnome-program-init "chart" chart-version
				       (program-arguments) '())))
      (gnome-program-set-property progobj 'app-datadir chart-config-datadir))

    ;; 9/10 screen width, and leave the height for what the ticker wants
    (gtk-window-set-default-size toplevel
				 (floor-exact (* 9/10 (gdk-screen-width)))
				 -1)
    (gtk-window-set-title toplevel (c-make-window-title (_ "Ticker")))

    (keymap-set! keymap "Left"    go-left)
    (keymap-set! keymap "Right"   go-right)
    (keymap-set! keymap "space"   toggle-run)
    (keymap-set! keymap "F1"      help)  ;; until menu created
    (keymap-set! keymap "<Ctrl>Q" quit)  ;; until menu created

    (gtk-widget-add-events toplevel '(key-press-mask))
    (gtk-signal-connect toplevel "key-press-event" keymap-dispatch-key))

  ;; the default symlist is favourites
  ;; a single given --all, --favourites or --alert uses that
  ;; any listed or extra listed symbols creates a special symlist
  ;;
  (if (and option-alert
	   (or option-all option-favourites specified-symbols))
      (begin
	(display "chart: cannot show alert list plus other symbols (yet), sorry")
	(exit 1)))
  (let ((symlist  favourites-symlist))
    (cond ((or specified-symbols
	       (multiple-of? option-all option-alert option-favourites))
	   (set! symlist (symlist-new #:key  'command-line
				      #:name (_ "Command Line List")))
	   (symlist-list! symlist symbol-list))
	  (option-all
	   (set! symlist all-symlist))
	  (option-alert
	   (set! symlist alert-symlist)))
    (ticker-symlist! ticker symlist))

  (gtk-widget-show-all toplevel)
  (c-main))
