;; Main window toolbar.

;; Copyright 2004, 2005, 2006, 2007 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(define-module (chart toolbar)
  #:use-module (gtk gtk)
  #:use-module (gtk gdk)
  #:autoload   (chart back-menu)  (back-menu-popup forward-menu-popup)
  #:use-module (chart gnome-ui)
  #:use-module (chart main)
  #:use-module (chart misc))


(define-public toolbar-widget (gtk-toolbar-new))
(gtk-toolbar-set-orientation toolbar-widget 'horizontal)
(gtk-toolbar-set-style toolbar-widget 'both)
;; (gtk-object-set toolbar-widget #:space_size 4)
(gnome-app-set-toolbar app-widget toolbar-widget)

;; Stock "gtk-media-previous" and "gtk-media-next" (from
;; `gtk-button-new-from-stock') don't really look right, they're meant for
;; music players etc rather than a general sequence of viewing.
;;

(let ((button (gtk-button-new-with-label (_ "Prev"))))
  (gtk-toolbar-append-widget toolbar-widget button
			     (_ "Go to previous chart.") "")
  (gtk-signal-connect button "clicked" chart-prev)
  (gtk-widget-set-sensitive button (chart-prev?))
  (gtk-signal-connect outer "series-changed"
    (lambda ()
      (gtk-widget-set-sensitive button (chart-prev?)))))

(let ((button (gtk-button-new-with-label (_ "Next"))))
  (gtk-toolbar-append-widget toolbar-widget button (_ "Go to next chart.") "")
  (gtk-signal-connect button "clicked" chart-next)
  (gtk-widget-set-sensitive button (chart-next?))
  (gtk-signal-connect outer "series-changed"
    (lambda ()
      (gtk-widget-set-sensitive button (chart-next?))))

  ;; initial focus (doesn't work when switched to vertical, since the
  ;; reparent into the contents-hbox loses the focus)
  (gtk-widget-grab-focus button))

;; These stock items show I and O as mnemonics, where instead the bindings
;; are Z and Shift-Z, which is a bit confusing

(let ((button (gtk-button-new-from-stock "gtk-go-back")))
  (gtk-toolbar-append-widget toolbar-widget button
			     (_ "Go back to the previously viewed symbol.") "")
  (gtk-signal-connect button "clicked" chart-back)
  (gtk-widget-set-sensitive button (chart-back?))
  (notify-connect 'chart-open-history-changed
    (lambda (dummy)
      (gtk-widget-set-sensitive button (chart-back?))))
  (gtk-signal-connect button "button_press_event"
    (lambda (event)
      (if (= 3 (gdk-event-button event))
	  (back-menu-popup event)))))

(let ((button (gtk-button-new-from-stock "gtk-go-forward")))
  (gtk-toolbar-append-widget toolbar-widget button
			     (_ "Go forward again, after \"back\".") "")
  (gtk-signal-connect button "clicked" chart-forward)
  (gtk-widget-set-sensitive button (chart-forward?))
  (notify-connect 'chart-open-history-changed
    (lambda (dummy)
      (gtk-widget-set-sensitive button (chart-forward?))))
  (gtk-signal-connect button "button_press_event"
    (lambda (event)
      (if (= 3 (gdk-event-button event))
	  (forward-menu-popup event)))))

;; These stock items show I and O as mnemonics, where instead the bindings
;; are Z and Shift-Z, which is a bit confusing

(let ((button (gtk-button-new-from-stock "gtk-zoom-in")))
  (gtk-toolbar-append-widget toolbar-widget button (_ "Zoom in.") "")
  (gtk-signal-connect button "clicked" zoom-in))

(let ((button (gtk-button-new-from-stock "gtk-zoom-out")))
  (gtk-toolbar-append-widget toolbar-widget button (_ "Zoom out.") "")
  (gtk-signal-connect button "clicked" zoom-out))
