;; Toronto Stock Exchange setups.

;; Copyright 2005, 2006, 2007 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.



(define-module (chart toronto)
  #:use-module (srfi srfi-13)
  #:use-module (chart misc)
  #:use-module (chart yahoo))


(define-public (toronto-symbol? symbol)
  (or (string-suffix? ".TO" symbol)   ;; tsx
      (string-suffix? ".V" symbol)))  ;; tsx venture

;; http://ca.finance.yahoo.com/indices?u
;; ^GSPTSE - S&P TSX Composite
;; ^SPTSECP - S&P TSX 60 Capped
;; ^SPTSECP3 - S&P TSX 300 Capped
;; ^SPTSEM - Midcap
;; ^SPTSES - Smallcap
;; ^SPTTEN - Energy
;; ^SPTTFS - Financials
;; ^SPTTGD - Gold
;; ^SPTTTK - Information Technology
;; ^GSPTTCD - Consumer Discretionary
;; ^GSPTTCS - Consumer Staples
;; ^GSPTTHC - Health Care
;; ^GSPTTIN - Industrials
;; ^GSPTTMN - Diversified Metals & Mining
;; ^GSPTTMT - Materials
;; ^GSPTTRE - Real Estate
;; ^GSPTTTS - Telecommunications Services
;; ^GSPTTUT - Utilities
;;
;; TSX Venture
;; ^SPCDNX - CDNX Index
;;
(define (yahoo-index-symbol-toronto? symbol)
  (or (string-prefix? "^GSPT" symbol)
      (string-prefix? "^SPT" symbol)
      (string-prefix? "^SPC" symbol)))

(define-public timezone-toronto
  (make-procedure-with-cache-0
   (lambda ()
     (timezone-choose (_ "Toronto")
		      '("America/Toronto"
			;; Canada: second Sun in Mar, to first Sun in Nov
			"EST+5EDT,M3.2.0,M11.1.0")
		      "EST+5"))))

(symbol-timezone! toronto-symbol? (_ "Toronto") timezone-toronto)
(symbol-timezone! yahoo-index-symbol-toronto? (_ "Toronto") timezone-toronto)

;; TSX trades 9:30 to 4pm and extended session 4:10pm to 5pm for member firms
;; see: http://www.tsx.com/en/contactUs/index.html#holidays
(yahoo-quote-lock! toronto-symbol?
		   #,(hms->seconds 9 30 0) #,(hms->seconds 17 0 0))

;; dunno if indexes update in extended session, cover that just in case
(yahoo-quote-lock! yahoo-index-symbol-toronto?
		   #,(hms->seconds 9 30 0) #,(hms->seconds 17 0 0))

(yahoo-quote-delay! toronto-symbol? 15)
