;; TRIX indicator.

;; Copyright 2006, 2007 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(define-module (chart trix)
  #:use-module (ice-9 and-let-star)
  #:use-module (chart indicator)
  #:use-module (chart ema-3)
  #:use-module (chart misc)
  #:use-module (chart roc))


;; http://www.incrediblecharts.com/technical/trix_indicator.htm
;; http://www.incrediblecharts.com/technical/trix_construction.htm
;;     Also called "Trix Oscillator".


(define-public (trix-calc-proc count)
  (define ema3-proc (ema-3-calc-proc count))
  (define roc-proc  (roc-calc-proc 1))
  (lambda (x)
    (and-let* ((ema3 (ema3-proc x)))
      (roc-proc ema3))))

(define (trix-fill series new-array lo hi)
  (receive-list (count warmup)
      (series-data series)
    (let* ((parent (series-parent series))
	   (array  (series-array-closes parent lo hi))
	   (proc   (trix-calc-proc count)))

      ;; warmup
      (for-each proc (series-prev-n-closes parent #:before lo #:count warmup))

      ;; points lo to hi inclusive
      (do ((i lo (1+ i)))
	  ((> i hi))
	(if (array-ref array i)
	    (array-set! new-array (proc (array-ref array i)) i))))))

(define-public (trix-series series count)
  (let ((new-series (series-copy series))
	(warmup     (1+ (ema-3-warmup-count count))))
    (series-name!      new-series (format #f "~a ~a" (_ "TRIX") count))
    (series-datatype!  new-series trix-datatype)
    (series-parent!    new-series series)
    (series-data!      new-series (list count warmup))
    (series-require!   new-series (indicator-require-parts trix-fill warmup))
    (series-array!     new-series #f)
    (series-decimals!  new-series 0)
    (series-dividends! new-series '())
    (series-notes!     new-series '())
    new-series))

(define-public trix-datatype
  (datatype-new!
   #:key       'trix
   #:name      (_ "TRIX")
   #:units     'price-slope
   #:linestyle 'line
   #:hlines    '(0)
   #:manual    (_ "CONTEXT(manual-node): TRIX")
   #:proc      trix-series
   #:parameter-list
   (list (datatype-param-new
	  #:name    (_ "Days")
	  #:key     'trix-days
	  #:type    'integer
	  #:minimum 1
	  #:default 20))))
