#!/bin/sh
exec chart -s "$0" "$@"
!#

;; Copyright 2005, 2008 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 2, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License along
;; with Chart; see the file COPYING.  If not, write to the Free Software
;; Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
;; USA.


(use-modules (ice-9 and-let-star)
	     (ice-9 optargs)
	     (srfi srfi-1)
	     (chart extra-glib))

(define (form-tm tm)
  (format #f "~a ~a ~a  ~a:~a:~a  ~a ~a   ~a y~a"
	  (+ 1900 (tm:year tm))
	  (1+ (tm:mon tm))
	  (tm:mday tm)
	  (tm:hour tm)
	  (tm:min tm)
	  (tm:sec tm)
	  (tm:isdst tm)
	  (tm:zone tm)
	  (vector-ref '#("Sun" "Mon" "Tue" "Wed" "Thu" "Fri" "Sat") (tm:wday tm))
	  (tm:yday tm)))

(define (make-days tz)
  (let ((tm (gmtime (current-time))))
    (set-tm:hour 12)
    (set-tm:min 0)
    (set-tm:sec 0)
    (let ((t (car (mktime tm tz))))
      (iota (* 365 30) t 86400))))





;; (pk (form-tm (localtime (- 1130632200 3600) "Europe/Athens")))
;; (pk (form-tm (localtime (- 1130632200 3600) "EET-2EEST,M3.5.0/3,M10.5.0/4")))

(pk (form-tm (localtime 1143966600 "America/Winnipeg")))
(pk (form-tm (localtime 1143966600 "CST+6CDT,M4.1.0,M10.5.0/3")))

;; (pk (form-tm (localtime (+ 1130632200 3600) "Europe/Athens")))
;; (pk (form-tm (localtime (+ 1130632200 3600) "EET-2EEST,M3.5.0/3,M10.5.0/4")))



;; on the half hour every hour for now to +30 years
(define start (current-time))
(set! start (- start (modulo start 3600)))
(set! start (+ start 1800))
(define point-list (iota (* 30 365 24) start 3600))
(format #t "~a points\n" (length point-list))


(define print-filename #f)

(define (timezone-equality x y)
  (if print-filename
      (begin
	(format #t "~a:\n" print-filename)
	(set! print-filename #f)))
  (format #t "~a versus ~a\n" x y)
  (setenv "TZ" x)
  (let ((xt-list (days-list x))) ;; (map localtime point-list)))
    (setenv "TZ" y)
    (let ((yt-list (map localtime point-list)))
      (for-each (lambda (t xtm ytm)
		  (set-tm:zone xtm #f)
		  (set-tm:zone ytm #f)
		  (if (not (equal? xtm ytm))
		      (begin
			(format #t "  diff ~a\n" t)
			(format #t "      x  ~a\n" (form-tm xtm))
			(format #t "      y  ~a\n" (form-tm ytm)))))
		point-list xt-list yt-list))))

(define (process-timezone expr)
  (receive-list (func name zone-list default-zone)
      expr
    (if (list? zone-list) ;; should be (quote ("foo" ...))
	(begin
	  (set! zone-list (second zone-list))
	  (while (length>=? zone-list 2)
	    (timezone-equality (first zone-list) (second zone-list))
	    (set! zone-list (cdr zone-list)))))))

(define (process-expr expr)
  (if (and (list? expr)
	   (not (null? expr)))
      (cond ((eq? 'timezone-choose (first expr))
	     (process-timezone expr))
	    ((not (null? expr))
	     (process-expr (car expr))
	     (process-expr (cdr expr))))))

(define (process-file filename)
  (set! print-filename filename)
  (call-with-input-file filename
    (lambda (port)
      (do ((obj (read port) (read port)))
	  ((eof-object? obj))
	(process-expr obj)))))

(define* (directory-files dir #:optional full (pred noop))
  (and-let* ((stream (false-if-exception (opendir dir))))
    (let more ((lst '()))
      (let ((name (readdir stream)))
	(if (eof-object? name)
	    (begin
	      (closedir stream)
	      lst)
	    (begin
	      (if full
		  (set! name (g-build-filename dir name)))
	      (if (pred name)
		  (more (cons name lst))
		  (more lst))))))))


(timezone-equality "Africa/Accra"
			"GMT"
		   )
(exit 0)

(for-each process-file
	  (directory-files "/home/gg/chart"
			   #t
			   (lambda (path)
			     (string-suffix? ".scm" path))))
