;; Copyright 2005, 2008 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 2, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License along
;; with Chart; see the file COPYING.  If not, write to the Free Software
;; Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
;; USA.




;; i18n

  ;;   (define (category-value category)
  ;;     (or (getenv-notempty "LANGUAGE")  ;; GNU extension
  ;; 	(getenv-notempty "LC_ALL")    ;; overrides any LC_xxx
  ;; 	(getenv-notempty category)
  ;; 	(getenv-notempty "LANG")))

  ;;   ;; Return (list 'lang 'territory 'codeset 'modifier))
  ;;   (define (explode-locale locale)
  ;;     #f)

  (set! category (or category "LC_ALL"))  ;; default

(define (gnome-gettext str)
  (gettext str "gnome"))

;; gettext, or if unchanged then with libgnome domain
(define (gnome-app-helper-gettext str)
  (let ((trans (gettext str)))
    (if (string= str trans)
	(gettext str "libgnome")
	trans)))




;; return a procedure to be called (PROC widget value), which is a record
;; modifier for FIELD in REC-TYPE in a gnome-prop
;;
;; unused
;;
(define (gnome-prop-record-modifier rec-type field)
  (let ((rec-mod-proc (record-modifier rec-type field)))
    (lambda (widget value)
      (rec-mod-proc (gnome-prop widget) value))))





;; prefer GtkMessageDialog

;;-----------------------------------------------------------------------------
;; gnome-dialog-util

(define-public (gnome-error-dialog-parented message parent)
  (let ((dialog (gnome-message-box-new message 'error 'ok)))
    (gnome-dialog-button-connect dialog 0
				 (lambda () (gtk-widget-destroy dialog)))
    (gnome-dialog-grab-focus dialog 0)
    (gtk-widget-show-all dialog)
    dialog))

;;-----------------------------------------------------------------------------
;; GnomeMessageBox

(let-toplevel ((gnome-message-box-type
		(gtk-class-new 'GnomeDialog "GnomeMessageBox")))
  (define title-alist '((info     . "Information")
			(warning  . "Warning")
			(error    . "Error")
			(question . "Question")))
  
  (define-public (gnome-message-box-new message type . buttons)
    (let* ((dialog  (gtk-widget-new gnome-message-box-type)))
      (gtk-window-set-title (format #f "~a (~a)"
				    (gnome-gettext (or (assq-ref title-alist
								 type)
						       "Question")))
			    (gnome-program-get-human-readable-name #f))
      (apply gnome-dialog-append-buttons buttons)
      (let ((label (gtk-label-new (string-append message "\n"))))
	(gtk-box-pack-start (gtk-dialog-vbox dialog) label #t #t 0)
	(gtk-widget-show label))
      dialog))
  )


;;-----------------------------------------------------------------------------
;; GnomeDialog

(let-toplevel
    ;; normally a direct subclass of GtkWindow actually
    ((gnome-dialog-type (gtk-class-new 'GtkDialog "GnomeDialog"))
     (rec-type          (make-record-type
			 "gnome-dialog"
			 '(buttons         ;; list of widgets
			   close           ;; #t or #f
			   close-hides)))) ;; #t or #f

  (define rec-new          (record-constructor rec-type))
  (define rec-buttons      (record-accessor rec-type 'buttons))
  (define rec-buttons!     (record-modifier rec-type 'buttons))
  (define rec-close        (record-accessor rec-type 'close))
  (define rec-close!       (record-modifier rec-type 'close))
  (define rec-close-hides  (record-accessor rec-type 'close-hides))

  (define (button-widget dialog n)
    (list-ref (rec-buttons (gnome-prop dialog)) n))

  ;; get action-area of the dialog
  (define-public (gnome-dialog-action-area dialog)
    (gtk-dialog-action-area dialog))

  ;; connect to button, numbered starting from 0
  (define-public (gnome-dialog-button-connect dialog button-number proc)
    (gtk-signal-connect (button-widget dialog button-number) "clicked" proc))

  (define-public (gnome-dialog-set-sensitive dialog button-number setting)
    (gtk-widget-set-sensitive (button-widget dialog button-number) setting))

  (define-public (gnome-dialog-close dialog)
    (if (rec-close-hides (gnome-prop dialog))
	(gtk-widget-hide dialog)
	(gtk-widget-destroy dialog)))

  ;; set whether gnome-dialog-close hides or destroys
  (define-public gnome-dialog-close-hides
    (gnome-prop-record-modifier rec-type 'close-hides))

  (define-public (gnome-dialog-grab-focus dialog button-number)
    (gtk-widget-grab-focus (button-widget dialog button-number)))

  ;; button-names are gnome-stock-button items, or strings
  ;;
  (define-public (gnome-dialog-new title . button-names)
    (let* ((dialog  (gtk-widget-new gnome-dialog-type #:title title))
	   (action  (gtk-dialog-action-area dialog))
	   (rec     (rec-new #f #f #f)) ;; default destroy
	   (buttons (map
		     (lambda (button-name)
		       (let ((button (gnome-stock-button button-name)))
			 (gtk-widget-set-flags button '(can-default))
			 (gtk-signal-connect
			  button "clicked"
			  (lambda () (if (rec-close rec)
					 (gnome-dialog-close dialog))))
			 (gtk-box-pack-start action button #f #f 0)
			 (gtk-widget-show button)
			 button))
		     button-names)))
      (rec-buttons! rec buttons)
      (set! (gnome-prop dialog) rec)

      ;; window manager close
      (gtk-signal-connect dialog "delete_event"
			  (lambda (event)
			    (gnome-dialog-close dialog)))

      ;; escape closes
      (gtk-widget-add-events dialog '(key-press-mask))
      (gtk-signal-connect dialog "key-press-event"
			  (lambda (event)
			    (if (= (gdk-keyval-from-name "Escape")
				   (gdk-event-keyval event))
				(gnome-dialog-close dialog))))

      (if (not (null? buttons))
	  (gnome-dialog-set-default dialog 0))

      dialog))

  ;; whether clicking buttons closes
  (define-public (gnome-dialog-set-close dialog close)
    (rec-close! (gnome-prop dialog) close))

  (define-public (gnome-dialog-set-default dialog button-number)
    (gtk-widget-grab-default (button-widget dialog button-number)))

  ;; get vbox of the dialog
  (define-public (gnome-dialog-vbox dialog)
    (gtk-dialog-vbox dialog))
  )


;;-----------------------------------------------------------------------------
;; GnomeColorPicker

;; COLOUR should be a GdkColor, or a string will work too
;;
;; (define-public (c-gnome-color-picker-set-gdk picker colour)
;;   (gnome-color-picker-set-i16 picker
;; 			      (gdk-color-red   colour)
;; 			      (gdk-color-green colour)
;; 			      (gdk-color-blue  colour)
;; 			      65535))  ;; alpha


;;-----------------------------------------------------------------------------
;; GnomeDialog

;; set the given button to close the dialog
(define-public (c-gnome-dialog-button-connect-close dialog button)
  (gnome-dialog-button-connect dialog button
			       (lambda () (gtk-widget-hide dialog))))



;;-----------------------------------------------------------------------------
;; gnome-window       deprecated

;; WINDOW a GtkWindow
;; DOCNAME and APPNAME strings
;; EXTEN string or #f
;;
(define-public (gnome-window-toplevel-set-title window docname appname exten)
  (if (and exten
	   (string-suffix? exten docname))
      (set! docname (string-drop-right docname (string-length exten))))
  (gtk-window-set-title window (string-append appname " : " docname)))




;;-----------------------------------------------------------------------------
;; GnomeEntry

;; deprecated

;; lacking history
(define-public (gnome-entry-new history-id)
  (gtk-entry-new))
(define-public (gnome-entry-gtk-entry entry)
  entry)


;;-----------------------------------------------------------------------------
;; gnome-client

;; not really ready to attempt recording restart info

;; GnomeRestartStyle: if-running, anyway, immediately, never

(define-public (gnome-client-set-restart-style client style)
  #f)

(define-public (gnome-client-set-restart-command client argv)
  #f)

(define-public (gnome-client-set-clone-command client argv)
  #f)


