;; Weighted close series.

;; Copyright 2007, 2009 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(define-module (chart weighted-close)
  #:use-module (ice-9 and-let-star)
  #:use-module (chart indicator)
  #:use-module (chart misc))


;; If there's no high or low then the weighted close is just the close.  The
;; close is copied directly for that case, so the value can stay exact
;; (arithmetic-mean always returns an inexact).
;;
(define (weighted-close-fill series new-array lo hi)
  (let* ((parent (series-parent series))
	 (array  (series-array parent lo hi)))

    ;; points lo to hi inclusive
    (do ((i lo (1+ i)))
	((> i hi))
      (and-let* ((close (array-ref array i 3)))
	(array-set! new-array (if (or (array-ref array i 1)          ;; high
				      (array-ref array i 2))         ;; low
				  (arithmetic-mean
				   (or (array-ref array i 1) close)  ;; high
				   (or (array-ref array i 2) close)  ;; low
				   close
				   close)

				  close)
		    i)))))

(define-public (weighted-close-series series)
  (if (eq? datatype-ohlcv (series-datatype series))
      (let ((new-series (series-copy series)))
	(series-name!     new-series (_ "Weighted Close"))
	(series-datatype! new-series weighted-close-datatype)
	(series-parent!   new-series series)
	(series-require!  new-series (indicator-require-parts
				      weighted-close-fill 0))
	(series-array!    new-series #f)
	new-series)

      series)) ;; not ohlcv, unchanged

(define-public weighted-close-datatype
  (datatype-new! #:key      'weighted-close
		 #:name     (_ "Weighted Close (H+L+2*C)")
		 #:priority -1000
		 #:proc     weighted-close-series))
