;; Williams Accumulation/Distribution index.

;; Copyright 2006, 2007 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(define-module (chart williams-accdist)
  #:use-module (chart indicator)
  #:use-module (chart misc))


(define-public (williams-accdist-func array prev this)
  (let ((prev-close (array-ref array prev 3))
	(close      (array-ref array this 3)))

    (cond ((> close prev-close)
	   (- close (min (or (array-ref array this 2) close) ;; low
			 prev-close)))
	  ((< close prev-close)
	   (- close (max (or (array-ref array this 1) close) ;; high
			 prev-close)))
	  (else
	   0))))

(define-public (williams-accdist-series series)
  (accumulation-series series (_ "Williams Acc/Dist")
		       williams-accdist-datatype williams-accdist-func))

(define-public williams-accdist-datatype
  (datatype-new!
   #:key    'williams-accdist
   #:name   (_ "Williams Accumulation/Distribution")
   #:units  'williams-accdist
   #:manual (_ "CONTEXT(manual-node): Williams Accumulation/Distribution")
   #:proc   williams-accdist-series))
