;; Zlib functions.

;; Copyright 2005, 2006, 2007 Kevin Ryde

;; This file is part of Chart.
;;
;; Chart is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Chart is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(define-module (chart zlib)
  #:use-module (chart config)
  #:use-module (chart extra-glib))

(load-extension (g-build-filename chart-config-libchartdir "zlib.la")
		"chart_zlib_init")


;;-----------------------------------------------------------------------------
;; gzopen
;;
;; either actual functions from zlib, or piping through gzip

(define-public (gzopen-available?)
  #t)  ;; builtin zlib

;; otherwise spawning gzip
(if (not (defined? 'gzopen))
    (begin
      ;; equivalents running gzip, with a pipe port for the gzFile object

      (use-modules (ice-9 popen)
		   (chart gzip)
		   (chart http)
		   (chart ole))

      (set! gzopen-available? gzip-version)

      (define-public (gzopen filename mode)
	(or (equal? mode "rb")
	    (error "gzopen using gzip: unsupported mode:" mode))
	(or (file-exists? filename)
	    (error "gzopen using gzip: no such file:" filename))
	(open-input-pipe (string-append "gzip -cd " filename)))

      (define-public (gzread port n)
	(read-bytes port n)) ;; from ole.scm

      (define-public (gzclose port)
	;; discard remaining input, so we can check exit code correctly
	(do ((c (read-char port) (read-char port)))
	    ((eof-object? c)))
	(or (eqv? 0 (status:exit-val (close-pipe port)))
	    (error "chart: error from gzip -cd")))))
